#include "board.h"

void board::move_mine(int x_from,int y_from,int x_to,int y_to)
{
	squares[x_from][y_from].mine=1-squares[x_from][y_from].mine;
	squares[x_to][y_to].mine=1-squares[x_to][y_to].mine;
	squares[x_from][y_from].num=mines_around(x_from,y_from);
	squares[x_to][y_to].num=mines_around(x_to,y_to);
}

void board::init_luck_library()
{
	luck_pattern* lp;
	
	// the 3-1 pattern aka 1x2
	// ***
	//  ? 
	//  ? 
	// ***
	lp=new luck_pattern(3,4,1,true);
	lp->set_moveable_mine(0,1,1,1,2);
	for(int i=0;i<3;++i) lp->pattern[i][0]=lp->pattern[i][3]=lpe_unopenable;
	lp_library << lp;
	
	// the 2x2 pattern
	// *  *
	//  ?? 
	//  ?? 
	// *  *
	lp=new luck_pattern(4,4,2,false);
	lp->set_moveable_mine(0,1,1,2,1);
	lp->set_moveable_mine(1,2,2,1,2);
	lp->pattern[0][0]=lp->pattern[3][0]=lp->pattern[0][3]=lp->pattern[3][3]=lpe_unopenable;
	lp_library << lp;
}

bool board::try_luck_library(int id,int x,int y,luck_pattern_element mine_from,luck_pattern_element mine_to)
{
	if(x<0 || y<0 || x+lp_library[id]->width-1>D.width+1 || y+lp_library[id]->height-1>D.height+1) 
		return false;
	for(int i=0;i<lp_library[id]->width;++i)
		for(int j=0;j<lp_library[id]->height;++j)
		{
			square* s=&squares[x+i][y+j];
			if(lp_library[id]->pattern[i][j]==lpe_unopenable && !s->unopenable()) return false;
			else if(lp_library[id]->pattern[i][j]==mine_from && !s->unknownmine()) return false;
			else if(lp_library[id]->pattern[i][j]==mine_to && !s->unknownsafe()) return false;
			else if(lp_library[id]->pattern[i][j]==lpe_unmoveable_mine && !s->mine) return false;
		}
	for(int i=0;i<lp_library[id]->n_moveable_mines;++i)
		move_mine(x+lp_library[id]->mm_x1[i],y+lp_library[id]->mm_y1[i],
			x+lp_library[id]->mm_x2[i],y+lp_library[id]->mm_y2[i]);
	cout<<"luck! pattern "<<id<<", x="<<x<<", y="<<y<<endl;
	return true;
}
bool board::try_luck_library_r(int id,int x,int y,luck_pattern_element mine_from,luck_pattern_element mine_to)
{
	if(x<0 || y<0 || x+lp_library[id]->height-1>D.width+1 || y+lp_library[id]->width-1>D.height+1) return false;
	for(int i=0;i<lp_library[id]->width;++i)
		for(int j=0;j<lp_library[id]->height;++j)
		{
			square* s=&squares[x+j][y+i];	// TODO replace unopenable with is_possible
			if(lp_library[id]->pattern[i][j]==lpe_unopenable && !s->unopenable()) return false;
			else if(lp_library[id]->pattern[i][j]==mine_from && !s->unknownmine()) return false;
			else if(lp_library[id]->pattern[i][j]==mine_to && !s->unknownsafe()) return false;
			else if(lp_library[id]->pattern[i][j]==lpe_unmoveable_mine && !s->mine) return false;
		}
	for(int i=0;i<lp_library[id]->n_moveable_mines;++i)
		move_mine(x+lp_library[id]->mm_y1[i],y+lp_library[id]->mm_x1[i],
			x+lp_library[id]->mm_y2[i],y+lp_library[id]->mm_x2[i]);
	cout<<"luck! pattern "<<id<<", x="<<x<<", y="<<y<<endl;
	return true;
}

bool board::apply_luck_library(int x,int y)
{
	
	for(int i=0;i<lp_library.size();++i)
		for(int j=0;j<lp_library[i]->n_moveable_mines;++j)
		{
			if(try_luck_library(i,x-lp_library[i]->mm_x1[j],y-lp_library[i]->mm_y1[j],
			lpe_moveable_mine_1,lpe_moveable_mine_2)) 
				return true;
			if(try_luck_library(i,x-lp_library[i]->mm_x2[j],y-lp_library[i]->mm_y2[j],
			lpe_moveable_mine_2,lpe_moveable_mine_1)) 
				return true;
			if(lp_library[i]->rotate)
			{
				if(try_luck_library_r(i,x-lp_library[i]->mm_y1[j],y-lp_library[i]->mm_x1[j],
				lpe_moveable_mine_1,lpe_moveable_mine_2)) 
					return true;
				if(try_luck_library_r(i,x-lp_library[i]->mm_y2[j],y-lp_library[i]->mm_x2[j],
				lpe_moveable_mine_2,lpe_moveable_mine_1)) 
					return true;
			}
		}
	return false;
}

struct excluck{};

bool board::apply_luck_solver(int x,int y)
{
	cout<<"Lucky Solver starts"<<endl;
	
	// determine border
	p_reset_border();
	p_find_isolated();
	if(squares[x][y].psolved) return false;	// it was an already known mine

	for(int i=0;i<p_brd.size();++i) delete p_brd[i];
	p_brd.clear();
	p_brd<<new border;	// #0
	p_brd << (c_brd=new border);
	process_border_c(1,x,y);	// get the border containing the blasted square

	l_mines_required=0;
	for(int i=0;i<p_brd[1]->c.size();++i) l_mines_required+=p_brd[1]->c[i]->mine;
	
	int n_lucky_solutions;
	luck_iter=0;
	l_current_solution=0;
	try
	{
		cout<<"searching for lucky solutions"<<endl;
		squares[x][y].pmine=-1;
		l_mines=0;
		l_solve_border(0);	// get the solutions that haven't a mine in the blasted square
		if(!(n_lucky_solutions=l_current_solution)) return false;
		cout<<"there are "<<n_lucky_solutions<<" of them"<<endl;
		cout<<"searching for unlucky solutions"<<endl;
		squares[x][y].pmine=1;
		l_mines=1;
		l_solve_border(0);	// get the solutions that have a mine in the blasted square
		cout<<"there are "<<l_current_solution-n_lucky_solutions<<" of them"<<endl;
	}
	catch(excluck)
	{
		squares[x][y].pmine=0;
		cout<<"Too many iterations"<<endl;
		return false;	// too many iterations => no luck
	}

	cout<<"marking the solved safe squares"<<endl;
	// mark the squares that are known to be safe
	for(int i=0;i<c_brd->c.size();++i)
	{
		bool safe=true;
		for(int s=0;s<l_current_solution;++s) safe=safe&&(c_brd->c[i]->pr[s]==-1);
		if(safe) c_brd->c[i]->pmine=-1;
	}

	cout<<"checking the goodness of solutions"<<endl;
	QList<int> good_solutions;
	for(int s=0;s<n_lucky_solutions;++s) 
	{
		bool good=true;
		for(int i=1;i<=D.width;++i)	// check if there is an openable square eliminating a solution
			for(int j=1;j<=D.height;++j)
				if(!squares[i][j].opened && squares[i][j].pmine!=1 && 
				(!squares[i][j].border || squares[i][j].pmine==-1))
					good = good && l_check(s,i,j);
		if(good) 
		{
			good_solutions << s;
			cout<<"solution "<<s<<" is good"<<endl;
		}
	}

	cout<<"Lucky Solver ends"<<endl;
	if(!good_solutions.size()) 
		return false;
	else
	{	// shuffle the mines
		int r=good_solutions[rg.randInt(good_solutions.size()-1)];
		cout<<"applying solution "<<r<<endl;
		for(int i=0;i<c_brd->c.size();++i) c_brd->c[i]->mine=(c_brd->c[i]->pr[r]==1);
		init_numbers();
		return true;
	}
}

bool board::l_check(int s,int x,int y)
{
	int m_sol=0,m_req=0,b=0;
	for(int i=x-1;i<=x+1;++i)
		for(int j=y-1;j<=y+1;++j)
			if(squares[i][j].border && !squares[i][j].opened)
			{
				++b;
				m_sol+=(squares[i][j].pr[s]==1);
				m_req+=squares[i][j].mine;
			}
/*	if(b)
		for(int i=x-1;i<=x+1;++i)
			for(int j=y-1;j<=y+1;++j)
				if(!squares[i][j].border && !squares[i][j].opened)
					return false;*/
	return m_sol==m_req;
}

void board::l_solve_border(int sq)
{
	if(++luck_iter>n_luck_iterations) throw excluck();
	if(sq==c_brd->o_x.size())
	{
		if(l_mines!=l_mines_required) return;

		// add a solution
		for(int i=0;i<c_brd->c.size();++i) 
			c_brd->c[i]->pr[l_current_solution]=c_brd->c[i]->pmine;
		++l_current_solution;
		return;
	}
	
	// determine unknown squares adjacent to c_brd->o[sq]
	square* adj[8];
	int n_adj=0;
	int x=c_brd->o_x[sq],y=c_brd->o_y[sq];
	int a_mines=squares[x][y].num-known_mines_around(x,y);	// unknown mines around square
	for(int i=x-1;i<=x+1;++i)
		for(int j=y-1;j<=y+1;++j)
			if(squares[i][j].unknown())
				adj[n_adj++]=&squares[i][j];
	if(n_adj<a_mines || a_mines<0) return;	// not enough places or too many mines around

	++sq;
	if(!a_mines)
	{	// enough mines!
		for(int i=0;i<n_adj;++i) adj[i]->pmine=-1;
		l_solve_border(sq);
		for(int i=0;i<n_adj;++i) adj[i]->pmine=0;
		return;
	}
	l_mines+=a_mines;
	if(l_mines>l_mines_required) 
	{
		l_mines-=a_mines;
		return;
	}	// too many mines

	// first combination
	for(int i=1;i<=a_mines;++i) adj[n_adj-i]->pmine=1;
	for(int i=0;i<n_adj-a_mines;++i) adj[i]->pmine=-1;
	while(1)
	{
		int cur,o;
		l_solve_border(sq);
		for(cur=n_adj-1;cur>=0 && adj[cur]->pmine==-1;--cur);	// generate the next combination
		for(o=0;cur>=0 && adj[cur]->pmine==1;--cur,++o);
		if(cur<0) break;
		adj[cur]->pmine=1;adj[++cur]->pmine=-1;
		for(++cur;cur<n_adj-o+1;++cur) adj[cur]->pmine=-1;
		for(;cur<n_adj;++cur) adj[cur]->pmine=1;
	}

	// clear
	l_mines-=a_mines;
	for(int i=0;i<n_adj;++i) adj[i]->pmine=0;
}

