#include "board.h"

board::board(translation* T,skin* sk,timer* Timer):
T(T),sk(sk),Timer(Timer),squares(0),size(0)
{
	setMouseTracking(true);
	init_luck_library();
#ifdef _WIN32
	if(!CryptAcquireContext(&hprov,0,0,PROV_RSA_FULL,CRYPT_VERIFYCONTEXT))
		QCoreApplication::exit(1);
	generate_seed();
#endif	
}

#ifdef _WIN32
void board::generate_seed()
{
	const int N=624;
	long unsigned int buf[624];
	if(!CryptGenRandom(hprov,sizeof(long unsigned int)*N,(byte*)&buf))
		QCoreApplication::exit(1);
	rg.seed(buf,N);
}
#endif

board::~board()
{
	delete_squares();
	clear_boards();
	for(int i=0;i<lp_library.size();++i) delete lp_library[i];
#ifdef _WIN32
	CryptReleaseContext(hprov,0);
#endif
}

void board::init_squares()
{
	delete_squares();
	real_width = D.width>=sk->crit_width ? D.width : sk->crit_width;
	squares=new square*[real_width+2];	// failed new's should be dealt with
	for(int i=0;i<=real_width+1;++i) squares[i]=new square[D.height+2];
	init_borders();
}

void board::init_borders()
{

	// i=0 -- left border
	squares[0][0].type=1;	// left top
	for(int j=1;j<=D.height;++j) squares[0][j].type=2;	// left
	squares[0][D.height+1].type=3;	// left bottom

	for(int i=1;i<=D.width;++i)
	{
		squares[i][0].type=4;	// top
		for(int j=1;j<=D.height;++j) squares[i][j].type=0;	// the board
		squares[i][D.height+1].type=5;	// bottom
	}
	for(int i=D.width+1;i<=real_width;++i)
	{
		squares[i][0].type=4;	// top
		for(int j=1;j<=D.height;++j) squares[i][j].type=9;	// empty
		squares[i][D.height+1].type=5;	// bottom
	}
	
	// i=w+1 -- right border
	squares[real_width+1][0].type=6;		// right top
	for(int j=1;j<=D.height;++j) squares[real_width+1][j].type=7;	// right
	squares[real_width+1][D.height+1].type=8;	// right bottom

	for(int i=0;i<=real_width+1;++i)
		for(int j=0;j<=D.height+1;++j)
		{
			squares[i][j].reset();
			if(squares[i][j].type)
				squares[i][j].p=squares[i][j].pp=&sk->border[squares[i][j].type];
			else 
				squares[i][j].p=squares[i][j].pp=&sk->sq;
		}	
}

void board::correct_board(int& w,int& h,int& n)
{
	if(w<min_width) w=min_width;
	if(h<min_height) h=min_height;
	if(n<0 || n>w*h-1) n=0;
}
void board::correct_size(int& w,int& h)
{
	if(w<min_width) w=min_width;
	if(h<min_height) h=min_height;
}
void board::correct_n_mines(int& n)
{
	if(n<0 || n>size-1) n=0;
}
void board::resize(int w,int h,int n)
{
	correct_size(w,h);
	D.width=w;D.height=h;size=w*h;
	init_squares();
	scroll_x=(w>sk->max_width);
	scroll_y=(h>sk->max_height);
	cur_scr_x=cur_scr_y=0;

	setFixedSize(sk->square_size*(scroll_x?(sk->max_width+2):(real_width+2)),
			sk->square_size*(scroll_y?(sk->max_height+2):(h+2)));

	change_n_mines(n);	// it emits resized() which fits the board into the game widget
}
void board::change_n_mines(int n)
{
	has_layout=initialized=has_opening_borders=false;
	correct_n_mines(n);
	D.n_mines=n;

	D.level=l_custom;
	for(int i=l_beginner;i<=n_stand_levels;++i)
		if(level_width[i]==D.width && level_height[i]==D.height &&
			(level_mines[i]==n || custom_allowed))
		{
			D.level=(levelname)i;
			break;
		}

	reset_counters();
	cout<<"board level "<<D.level<<"; video size = "<<video.size()<<endl;
	emit resized();
}

void board::resize_if_needed(int w,int h,int n)
{
	cout<<"resize if needed, i'm "<<D.width<<" "<<D.height<<" "<<D.n_mines<<"; needed "<<w<<" "<<h<<" "<<n<<endl;
	if(w!=D.width || h!=D.height)
		resize(w,h,n);
	else if(n!=D.n_mines)
		change_n_mines(n);
}
void board::delete_squares()
{
	if(!squares) return;
	for(int i=0;i<=real_width+1;++i) delete[] squares[i];
	delete[] squares;
	squares=0;
}

void board::scrolling_x(int val)
{
	cur_scr_x=val;
	update();
	if(D.state<=playing) write_mouse_event(se_scroll_x,cur_scr_x,cur_scr_y);
}
void board::scrolling_y(int val)
{
	cur_scr_y=val;
	update();
	if(D.state<=playing) write_mouse_event(se_scroll_y,cur_scr_x,cur_scr_y);
}

void board::show_wasted_flags()
{
	for(int i=1;i<D.width;++i)
		for(int j=1;j<=D.height;++j)
			if(squares[i][j].flagged && squares[i][j].wasted_flag)
				show_wasted_flag(i,j);
}

void board::set_state(boardstate new_state)
{
	if(new_state==fail)
	{
		// do not react on unblasts while playing videos
		if(D.state==playing_video && Timer->get_time()<video_duration && !marathon) return; 
	
		if(D.state==playing) push_board();
		for(int i=1;i<=D.width;++i) // show misflags, mines and numbers
			for(int j=1;j<=D.height;++j)
				if(squares[i][j].flagged && !squares[i][j].mine)
					show_misflag(i,j);
				else if(squares[i][j].openable())
				{
					if(squares[i][j].mine)
						show_mine(i,j);
					else if(reveal_after_end)
						show_after_end(i,j);
				}
		if(show_wasted_flags_after_end) show_wasted_flags();
		update(); // draw it
		write_game_event(ge_term_lost);
		if(!marathon) end_video();
		if(D.state==playing && D.solved_bbbv*100<D.bbbv*clear_board_threshold) emit force_new_game();
	}
	else if(new_state==win)
	{
		++n_finished_boards;
		if(D.state==playing) push_board();
		write_game_event(ge_term_won);
		if(!marathon)
			end_video();
		else if(n_finished_boards==marathon_description.size())
		{
			end_video();
			n_finished_boards=0;
		}
		if(show_wasted_flags_after_end) show_wasted_flags();
		// set flags
		if(flag_mines_after_win)
			for(int i=1;i<=D.width;++i)
				for(int j=1;j<=D.height;++j)
					if(squares[i][j].mine && !squares[i][j].flagged && !squares[i][j].opened)
						flag(i,j);
		update();
	}
	else if(new_state==before_game) 
	{
		cur_lives=cheat_allowed?lives:1;
		if(D.state==playing_video) // video is interrupted by smiley's signal
		{
			n_finished_boards=0;
			Timer->stop();
			stop_video();
		}
		else
			if(!marathon || !n_finished_boards) clear_boards();	// clear board stack
	}
	else if(new_state==playing_video)
	{
		cur_lives=cheat_allowed?lives:1;
		mouse_path.clear();
	}
	else if(new_state==stopped_video)
	{
		update();
	}
	D.state=new_state;
	emit changed_state(D.state);
	setToolTip("");
}

int board::mines_around(int x,int y)
{	// returns the number of mines in the {neighborhood} plus the square (needed for is_bbbv)
	int res=0;
	for(int i=x-1;i<=x+1;++i)
		for(int j=y-1;j<=y+1;++j)
			res+=squares[i][j].mine;
	return res;
}
int board::flags_around(int x,int y)
{	// returns the number of flags in the {neighborhood}
	return squares[x-1][y-1].flagged+squares[x-1][y].flagged+
		   squares[x-1][y+1].flagged+squares[x][y-1].flagged+
		   squares[x][y+1].flagged+squares[x+1][y-1].flagged+
		   squares[x+1][y].flagged+squares[x+1][y+1].flagged;
}
int board::closed_sq_around(int x,int y)
{	// returns the number of closed squares in the {neighborhood}
	return 8-squares[x-1][y-1].opened-squares[x-1][y].opened-squares[x-1][y+1].opened
			-squares[x][y-1].opened-squares[x][y+1].opened
			-squares[x+1][y-1].opened-squares[x+1][y].opened-squares[x+1][y+1].opened;
}
int board::opened_sq_around(int x,int y)
{	// returns the number of opened squares in the {neighborhood}
	return squares[x-1][y-1].opened+squares[x-1][y].opened+squares[x-1][y+1].opened
			+squares[x][y-1].opened+squares[x][y+1].opened
			+squares[x+1][y-1].opened+squares[x+1][y].opened+squares[x+1][y+1].opened;
}
int board::opened_sq_around_without_border(int x,int y)
{	// returns the number of opened squares in the {neighborhood}
	int res=0;
	for(int i=x-1;i<=x+1;++i)
		for(int j=y-1;j<=y+1;++j)
			if(squares[i][j].opened && !squares[i][j].type)
				++res;
	return res;
}

int board::known_mines_around(int x,int y)
{	// returns the number of known mines in the {neighborhood}
	return (squares[x-1][y-1].pmine==1)+(squares[x-1][y].pmine==1)+(squares[x-1][y+1].pmine==1)+
			(squares[x][y-1].pmine==1)+(squares[x][y+1].pmine==1)+
			(squares[x+1][y-1].pmine==1)+(squares[x+1][y].pmine==1)+(squares[x+1][y+1].pmine==1);
}
int board::known_safe_around(int x,int y)
{	// returns the number of known safe squares in the {neighborhood}
	return (squares[x-1][y-1].pmine==-1)+(squares[x-1][y].pmine==-1)+(squares[x-1][y+1].pmine==-1)+
			(squares[x][y-1].pmine==-1)+(squares[x][y+1].pmine==-1)+
			(squares[x+1][y-1].pmine==-1)+(squares[x+1][y].pmine==-1)+(squares[x+1][y+1].pmine==-1);
}
int board::unsolved_sq_around(int x,int y)
{	// returns the number of unsolved squares in the {neighborhood}
	return squares[x-1][y-1].unsolved()+squares[x-1][y].unsolved()+squares[x-1][y+1].unsolved()+
			squares[x][y-1].unsolved()+squares[x][y+1].unsolved()+
			squares[x+1][y-1].unsolved()+squares[x+1][y].unsolved()+squares[x+1][y+1].unsolved();
}
int board::unknown_sq_around(int x,int y)
{	// returns the number of squares having no assumed value in the {neighborhood}
	return squares[x-1][y-1].unknown()+squares[x-1][y].unknown()+squares[x-1][y+1].unknown()+
			squares[x][y-1].unknown()+squares[x][y+1].unknown()+
			squares[x+1][y-1].unknown()+squares[x+1][y].unknown()+squares[x+1][y+1].unknown();
}

void board::flag_around(int x,int y)
{	// flag the openable {neighborhood}
	for(int i=x-1;i<=x+1;++i)
		for(int j=y-1;j<=y+1;++j)
			if(squares[i][j].flaggable() && !squares[i][j].flagged)
				set_flag(i,j);
}
void board::apply_autoflag()
{
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
			if(squares[i][j].opened && squares[i][j].num && squares[i][j].num>=closed_sq_around(i,j))
				flag_around(i,j);
}
void board::flag_all()
{
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
			if(squares[i][j].mine)
				set_flag(i,j);
}


void board::check_win()
{
	//cout<<"3BV: "<<D.solved_bbbv<<"/"<<"*"<<endl;//D.bbbv<<endl;
	if(D.bbbv==D.solved_bbbv) set_state(win);
}

void board::check_openings()
{
	for(int i=1;i<=D.openings;++i)
		if(!D.size_op[i])
		{
			D.size_op[i]=-1;
			++D.solved_ops;
			if(D.state==playing || D.state==playing_video) ++D.solved_bbbv;
			check_win();
		}
}

void board::show_opening(int x,int y)
{
	QQueue< QPair<int,int> > q;
	QPair<int,int> cur_sq;
	int i,j;

	q.enqueue(qMakePair(x,y));
	show(x,y);
	
	while(!q.isEmpty())
	{
		cur_sq=q.dequeue();
		i=cur_sq.first;j=cur_sq.second;
		
		if(!squares[i][j].num)
		{
			for(int k=i-1;k<=i+1;++k)
				for(int l=j-1;l<=j+1;++l)
					if(squares[k][l].openable())
					{
						q.enqueue(qMakePair(k,l));
						show(k,l);
					}
		}
	}
	check_openings();
}

void board::open(int x,int y)
{
	if(squares[x][y].mine)
	{
		if(luck_allowed &&
			((luck_library && apply_luck_library(x,y)) || (luck_solver && apply_luck_solver(x,y))))
		{
			open(x,y);
			return;
		}
		show_blast(x,y); 
		if(!(--cur_lives))
		{
			set_state(fail);
		}
	}
	else
	{
		if(!squares[x][y].num) 
			show_opening(x,y);
		else 
		{
			show(x,y);
			if(!squares[x][y].opening) 
			{
				if(D.state==playing || D.state==playing_video) ++D.solved_bbbv;
				if(!(--D.size_isl[squares[x][y].island])) ++D.solved_isls;
				check_win();
			}
			else
				check_openings();
		}
		if(cheat_allowed && autoflag) apply_autoflag();
		if(cheat_allowed && probabilities)
		{
			get_probabilities();
			update();
		}
	}
}
void board::set_flag(int x,int y)
{
	flag(x,y);
	emit flag();	// a signal for the red mines counter
	++D.flags;++D.wasted_flags;
	if(!squares[x][y].mine) 
		++D.misflags;
	else
		++D.right_flags;
}
void board::unset_flag(int x,int y)
{
	unflag(x,y);
	emit unflag();	// a signal for the red mines counter
	--D.flags;++D.unflags;
	if(squares[x][y].mine) 
	{
		++D.misunflags;
		--D.right_flags;
	}
}
void board::chord(int x,int y,bool onedotfive)
{
	bool wasted=true;
	if(mines_around(x,y)==flags_around(x,y) && squares[x][y].opened)
	{
		// the following check for {neighborhood} is done before opening to make solved3bv counter work right
		for(int i=x-1;i<=x+1;++i)
			for(int j=y-1;j<=y+1;++j)
				if(squares[x][y].mine && !squares[x][y].flagged)
					set_state(fail);
		// open the openable {neighborhood}
		for(int i=x-1;i<=x+1;++i)
			for(int j=y-1;j<=y+1;++j)
				if(squares[i][j].openable())
				{
					open(i,j);
					wasted=false;
				}
				else if(squares[i][j].flagged && squares[i][j].wasted_flag)
				{
					squares[i][j].wasted_flag=false;
					--D.wasted_flags;
				}
		if(wasted)
		{
			++D.wasted_d_clicks;
			if(onedotfive) ++D.wasted_clicks_15;
		}
	}
	else
	{
		pop_around(x,y);
		++D.wasted_d_clicks; 
		if(onedotfive) ++D.wasted_clicks_15;
	}
}

void board::init_board()
{
	init_board_basic();
	init_board_full();
}
void board::init_board_basic()
{
	init_numbers();
	init_openings();
	get_bbbv();
}
void board::init_board_full()
{
	init_islands();
	if(D.state>=playing_video) init_opening_borders();
	init_mines_counters();

	// ZiNi
	D.zini=get_zini(false);
	D.hzini=get_zini(true);
	initialized=true;

//	print_board();	// todelete
}

void board::print_board()
{
	for(int i=1;i<=D.height;++i)
	{
		for(int j=1;j<=D.width;++j) cout<<(squares[j][i].mine?"*":QString::number(squares[j][i].num));
		cout<<" | ";
		for(int j=1;j<=D.width;++j) cout<<squares[j][i].opening;
		cout<<" | ";
		for(int j=1;j<=D.width;++j) cout<<squares[j][i].opening2;
		cout<<endl;
	}
}

void board::init_numbers()
{
	for(int i=0;i<9;++i) D.numbers[i]=0;
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
			if((squares[i][j].num=mines_around(i,j)) && !squares[i][j].mine)
			{
				++D.numbers[squares[i][j].num];
				++D.numbers[0];
			}
}
void board::init_openings()
{
	D.size_op.clear();
	D.size_op<<0;	// opening #0 is non-existent
	D.openings=0;
	for(int i=1;i<=D.width;++i)	// delete existing info about openings
		for(int j=1;j<=D.height;++j)
			squares[i][j].opening=squares[i][j].opening2=0;
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
			if(!squares[i][j].num && !squares[i][j].mine && !squares[i][j].opening)
			{
				D.size_op<<0;
				process_opening(++D.openings,i,j);
			}
}
void board::process_opening(int op,int x,int y)
{
	if(squares[x][y].opening) squares[x][y].opening2=squares[x][y].opening;
	squares[x][y].opening=op;
	++D.size_op[op];
	if(!squares[x][y].num)	// here we check the {neighborhood} of the opening square
		for(int i=x-1;i<=x+1;++i)
			for(int j=y-1;j<=y+1;++j)
				if(squares[i][j].opening!=op && !squares[i][j].type)
					process_opening(op,i,j);
}
void board::get_bbbv()
{
	D.bbbv=0;
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
			if(!squares[i][j].opening && !squares[i][j].mine) ++D.bbbv;
	D.bbbv+=D.openings;
}
void board::init_islands()
{
	D.size_isl.clear();
	D.size_isl<<0;	// island #0 is non-existent
	D.islands=0;
	for(int i=1;i<=D.width;++i)	// delete existing info about openings
		for(int j=1;j<=D.height;++j)
			squares[i][j].island=0;
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
			if(!squares[i][j].opening && !squares[i][j].island && !squares[i][j].mine)
			{
				D.size_isl<<0;
				process_island(++D.islands,i,j);
			}
}
void board::process_island(int isl,int x,int y)
{
	squares[x][y].island=isl;
	++D.size_isl[isl];
	for(int i=x-1;i<=x+1;++i)	// we check if squares from the {neighborhood} belong to the same island
		for(int j=y-1;j<=y+1;++j)
			if(!squares[i][j].opening && !squares[i][j].island && 
					!squares[i][j].type && !squares[i][j].mine)
				process_island(isl,i,j);
}
void board::init_opening_borders()
{	// this terrible piece initializes the yellow frames of openings to be shown while watching video
	int o,bit;
	int* cox=new int[D.openings+1];
	int* coy=new int[D.openings+1];
	int* n=new int[D.openings+1];
	for(int i=1;i<=D.openings;++i) n[i]=cox[i]=coy[i]=0;

	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
			if(squares[i][j].num && squares[i][j].opening)
			{
				o=0;bit=1;
				for(int l=j-1;l<=j+1;++l)
					for(int k=i-1;k<=i+1;++k)
					{
						if(!squares[k][l].type && !squares[k][l].num && !squares[k][l].mine) 
							o|=bit;
						bit<<=1;
					}
				if( (o&8) && !(o&260) && !(o&130) ) squares[i][j].opborder=1;
				else if( (o&32) && !(o&65) && !(o&130) ) squares[i][j].opborder=1;
				else if( (o&2) && !(o&320) && !(o&40) )	squares[i][j].opborder=2;
				else if( (o&128) && !(o&5) && !(o&40) ) squares[i][j].opborder=2;
				else if( o==1 ) squares[i][j].opborder=3;
				else if( o==4 ) squares[i][j].opborder=4;
				else if( o==64 ) squares[i][j].opborder=5;
				else if( o==256 ) squares[i][j].opborder=6;
				else if( (o&128) && (o&32) ) squares[i][j].opborder=3;
				else if( (o&128) && (o&8) ) squares[i][j].opborder=4;
				else if( (o&32) && (o&2) ) squares[i][j].opborder=5;
				else if( (o&8) && (o&2) ) squares[i][j].opborder=6;
				else if( o&8 ) squares[i][j].opborder=7;
				else if( o&32 ) squares[i][j].opborder=8;
				else if( o&128 ) squares[i][j].opborder=9;
				else if( o&2 ) squares[i][j].opborder=10;
				else squares[i][j].opborder=11;
			}
			else
			{
				squares[i][j].opborder=0;
				if(!squares[i][j].num)
				{
					cox[squares[i][j].opening]+=i;
					coy[squares[i][j].opening]+=j;
					++n[squares[i][j].opening];
				}
			}
	
	for(int i=1;i<=D.openings;++i)
	{
		cox[i]=(int)( (double)cox[i]/n[i]+0.5 );
		coy[i]=(int)( (double)coy[i]/n[i]+0.5 );
		while((squares[cox[i]][coy[i]].num || squares[cox[i]][coy[i]].opening!=i)
				&& !squares[cox[i]][coy[i]].type)  
			--coy[i];
		if(squares[cox[i]][coy[i]].type)
		{
			++coy[i];
			while((squares[cox[i]][coy[i]].num || squares[cox[i]][coy[i]].opening!=i)
					&& !squares[cox[i]][coy[i]].type)  
				++coy[i];
		}
		squares[cox[i]][coy[i]].opborder=-i;
	}
	
	has_opening_borders=true;
	delete[] cox;
	delete[] coy;
	delete[] n;
}
void board::init_mines_counters()
{
	D.mines_on_walls=D.mines_in_corners=0;
	for(int j=1;j<=D.height;++j) D.mines_on_walls+=squares[1][j].mine+squares[D.width][j].mine;
	if(D.width==1) D.mines_on_walls/=2;
	if(D.height>1)
		for(int i=2;i<D.width;++i) D.mines_on_walls+=squares[i][1].mine+squares[i][D.height].mine;
	else
		for(int i=2;i<D.width;++i) D.mines_on_walls+=squares[i][1].mine;
	D.mines_in_corners=squares[1][1].mine+squares[1][D.height].mine+
			squares[D.width][1].mine+squares[D.width][D.height].mine;
	if(D.width==1) D.mines_in_corners/=2;
	if(D.height==1) D.mines_in_corners/=2;
}

void board::reinit_properties()
{
	for(int i=1;i<=D.openings;++i) D.size_op[i]=0;
	for(int i=1;i<=D.islands;++i) D.size_isl[i]=0;
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
			if(squares[i][j].opening)
			{
				++D.size_op[squares[i][j].opening];
				++D.size_op[squares[i][j].opening2];
			}
			else
				++D.size_isl[squares[i][j].island];
	if(D.state>=playing_video && !has_opening_borders) init_opening_borders();
}

bool board::is_bbbv(int x,int y)
{
	return	squares[x-1][y-1].num && squares[x-1][y].num && squares[x-1][y+1].num &&
		squares[x][y-1].num && squares[x][y+1].num &&
		squares[x+1][y-1].num && squares[x+1][y].num && squares[x+1][y+1].num;		  
}

void board::get_bbbv_fast()
{
	int cur_ops=0;
	D.openings=0;
	QList<int> op_p;
	op_p<<0;
	D.bbbv=0;

	init_numbers();

	// count bbbv
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
		{
			if(squares[i][j].mine) continue;
			if(squares[i][j].num)
			{
				if(is_bbbv(i,j)) ++D.bbbv;
			}
			else
			{
				int u=squares[i-1][j].opening,
					l=squares[i][j-1].opening,
					ul=squares[i-1][j-1].opening,
					ur=squares[i-1][j+1].opening;
				int fw;
				if(l)
				{
					if(u)
					{
						if(op_p[l]==u) l=u;
						if(u!=l && op_p[u]!=l)
						{
							--D.openings;
							fw=op_p[u];
							for(int k=1;k<=cur_ops;++k)
								if(op_p[k]==u || op_p[k]==fw)
								{
									op_p[k]=l;
								}
						}		
					}
					else 
					{
						if(ul)
						{
							if(op_p[l]==ul) l=ul;
							if(ul!=l && op_p[ul]!=l)
							{
								--D.openings;
								fw=op_p[ul];
								for(int k=1;k<=cur_ops;++k)
									if(op_p[k]==ul || op_p[k]==fw)
									{
										op_p[k]=l;
									}
							}
						}
						if(ur)	// ul & ur => u
						{
							if(op_p[l]==ur) l=ur;
							if(ur!=l && op_p[ur]!=l)
							{
								--D.openings;
								fw=op_p[ur];
								for(int k=1;k<=cur_ops;++k)
									if(op_p[k]==ur || op_p[k]==fw)
									{
										op_p[k]=l;
									}
							}
						}
					}
					squares[i][j].opening=op_p[l];
				}
				else if(u)	// only one bordering opening, use its id
					squares[i][j].opening=op_p[u];
				else if(ul)
					squares[i][j].opening=op_p[ul];
				else if(ur)
					squares[i][j].opening=op_p[ur];
				else
				{	// a new opening id
					squares[i][j].opening=++cur_ops;
					op_p << cur_ops;
					++D.openings;
				}
			}
		}
	D.bbbv+=D.openings;
}

void board::random_board(int* final,int* coord)
{
	for(int i=1;i<=D.n_mines;++i)
	{
		final[i]=coord[i]=rg.randInt(size-i-1);
	}
	for(int i=1;i<=D.n_mines;++i)
	{
		for(int j=0;j<i;++j) 
			if(coord[i]>=coord[j]) 
				++final[i];
			else
				--coord[j];
		int x=final[i]/D.height+1;
		int y=final[i]%D.height+1;
		squares[x][y].mine=1;
	}
	cout<<endl;
}
void board::random_board(int safe_x,int safe_y)
{
	int* final=new int[D.n_mines+1];
	int* coord=new int[D.n_mines+1];
	coord[0]=final[0]=D.height*(safe_x-1)+(safe_y-1);
	random_board(final,coord);
	delete[] final;
	delete[] coord;
}

void board::generate_with_zini(int safe_x,int safe_y,bool zini_l,bool hzini_l)
{
	int* final=new int[D.n_mines+1];
	int* coord=new int[D.n_mines+1];

	coord[0]=final[0]=D.height*(safe_x-1)+(safe_y-1);
	random_board(final,coord);
	init_board_basic();
	if(zini_l) D.zini=get_zini(false);
	if(hzini_l) D.hzini=get_zini(true);
	
	for(int iter=1; 
		iter<n_iter && D.state==generating && 
		( 
			(D.bbbv<lower_limit && lower_limit) || (D.bbbv>upper_limit && upper_limit) 
			|| (force_seven && !D.numbers[7]) || (force_eight && !D.numbers[8]) 
			|| (zini_l && ((D.zini<zn_lower_limit && zn_lower_limit) || 
							(D.zini>zn_upper_limit && zn_upper_limit)) )
			|| (hzini_l && ((D.hzini<hzn_lower_limit && hzn_lower_limit) || 
							(D.hzini>hzn_upper_limit && hzn_upper_limit)) )
		);
		++iter)
	{
		erase_board();
		coord[0]=final[0]=D.height*(safe_x-1)+(safe_y-1);
		random_board(final,coord);
		init_board_basic();
		if(zini_l) D.zini=get_zini(false);
		if(hzini_l) D.hzini=get_zini(true); 
			
		if(!(iter&0x3FF)) emit display_player(T->str[ins_iter]+" "+QString::number(iter));
		// show the number of the iteration 
	}
	
	init_board_full();

	delete[] final;
	delete[] coord;
}

void board::generate_fast(int safe_x,int safe_y)
{
	int* final=new int[D.n_mines+1];
	int* coord=new int[D.n_mines+1];

	cout<<"generate_fast"<<endl;
	cout<<"limits: "<<lower_limit<<" "<<upper_limit<<endl;

#ifdef _WIN32
	generate_seed();
#endif

	coord[0]=final[0]=D.height*(safe_x-1)+(safe_y-1);
	random_board(final,coord);
	get_bbbv_fast();
	
	for(int iter=1; 
		iter<n_iter && D.state==generating && 
		( 
			(D.bbbv<lower_limit && lower_limit) || (D.bbbv>upper_limit && upper_limit) ||
			(cheat_allowed && 
				((force_seven && !D.numbers[7]) || (force_eight && !D.numbers[8])))
		);
		++iter)
	{
	
	
		erase_board();
		coord[0]=final[0]=D.height*(safe_x-1)+(safe_y-1);
		random_board(final,coord);
		get_bbbv_fast();
			
		if(!(iter&0x3FF)) emit display_player(T->str[ins_iter]+" "+QString::number(iter));
		// show the number of the iteration 
	}

	init_board();

	delete[] final;
	delete[] coord;
}

void board::generate(int safe_x,int safe_y)
{
	bool zini_l=cheat_allowed&&((zn_lower_limit)||(zn_upper_limit && zn_upper_limit<size));
	bool hzini_l=cheat_allowed&&((hzn_lower_limit)||(hzn_upper_limit && hzn_upper_limit<size));

	if(zini_l || hzini_l)
		generate_with_zini(safe_x,safe_y,zini_l,hzini_l);
	else
		generate_fast(safe_x,safe_y);

	has_layout=true;
	initialized=has_opening_borders=false;	
	emit display_player(player);	// a signal for the player name label
}

void board::first_click(int x,int y,int prec_x,int prec_y)
{
	cout<<"first click "<<x<<" "<<y<<endl;
	set_state(generating);
	if(!restart_on_first_click) 
		generate(x,y);
	else
		init_board();
	set_state(playing);
	timestamp_begin=QDateTime::currentDateTime().toString(Qt::ISODate);
	if(!marathon || !n_finished_boards) start_video();
	write_game_event(ge_start_board);
	write_preflags();
	write_mouse_event(me_left_press, prec_x, prec_y);
	// don't write the click event if it's just a press
	if(!cheat_allowed || (!elmar && !nono)) write_mouse_event(me_left_click, prec_x, prec_y);
	D.distance=0;
	cur_prec_x=prec_x;cur_prec_y=prec_y;
	if(cheat_allowed && show_board)
	{
		for(int i=1;i<=D.width;++i) for(int j=1;j<=D.height;++j) show_after_end(i,j);
		update();
	}
	if(cheat_allowed && lawnmower) flag_all();
}

void board::left_press(int x,int y,int prec_x,int prec_y)
{
	if(middle) return;
	left=true;shift_left=false;
	if(D.state==playing) write_mouse_event(me_left_press, prec_x, prec_y);
	if(!is_inside_board(x,y)) return;
	if(!right && !(cheat_allowed && superclick && squares[x][y].opened))
		push(x,y);
	else
		push_around(x,y);
	emit press();	// this is a signal for the smiley
	if(cheat_allowed && (elmar || nono))
	{
		left_click(x,y,prec_x,prec_y,false);
		left=true;
	}
	cur_x=x;cur_y=y;
}
void board::left_press_with_shift(int x,int y,int prec_x,int prec_y)
{
	if(middle) return;
	left=shift_left=true;
	if(D.state==playing) write_mouse_event(me_left_press_with_shift, prec_x, prec_y);
	if(!is_inside_board(x,y)) return;
	push_around(x,y);
	emit press();	// this is a signal for the smiley
	if(cheat_allowed && (elmar || nono))
	{
		left_click(x,y,prec_x,prec_y,false);
		left=shift_left=true;
	}
	cur_x=x;cur_y=y;
}
void board::left_click(int x,int y,int prec_x,int prec_y,bool real_click)
{
	if(!left) return;
	if(D.state==playing || D.state==playing_video)
		if(x!=cur_x || y!=cur_y)
			mouse_move(x,y,prec_x,prec_y);
	if(D.state==playing && real_click) 
		write_mouse_event(shift_left?me_left_click_with_shift:me_left_click, prec_x, prec_y);
	left=false;
	if(!is_inside_board(x,y))
	{
		chorded=false;
		emit click();
		return;
	}
	if(right || shift_left || (cheat_allowed && superclick && squares[x][y].opened))	// chord
	{
		++D.d_clicks;
		if(onedotfive) ++D.clicks_15;
		if(D.state==playing_video) path_add_double_click(prec_x,prec_y);
		chord(x,y,onedotfive);
		chorded=right;
		shift_left=false;
	}
	else	// left click
	{
		if(chorded)	// rilian click
		{
			chorded=false;
			++D.rilian_clicks;
			return;
		}
		++D.l_clicks;
		if(D.state==playing_video) path_add_left_click(prec_x,prec_y);
		if(squares[x][y].openable())
		{
			if(D.state==before_game)	// 1st click
				first_click(x,y,prec_x,prec_y);
			else if(D.state==playing_video)
			{
				if(!n_viewed_boards) pop_board();
				// normally it is popped by process_video()
				// but RawVF Revision 3 allows videos without "start" event
			}
			open(x,y);
		}
		else ++D.wasted_l_clicks;
		chorded=false;
	}
	if(D.state==playing || D.state==playing_video) emit click();	// this is a signal for the smiley
	cur_x=x;cur_y=y;
}
void board::right_press(int x,int y,int prec_x,int prec_y)
{
	if(middle) return;
	right=true;shift_left=false;
	if(D.state==playing) write_mouse_event(me_right_press, prec_x, prec_y);
	if(!is_inside_board(x,y)) return;
	if(left)
	{
		push_around(x,y);
		emit press();
	}
	else
	{
		if(squares[x][y].flaggable())
		{
			onedotfive=true;chorded=false;
			if(squares[x][y].flagged)
			{
				unset_flag(x,y);
				if(qmarks) show_question(x,y);
			}
			else
			{
				if(!qmarks || !squares[x][y].questioned)
					set_flag(x,y);
				else
					unflag(x,y);	// this turns a question into an empty square
			}
			++D.r_clicks;
			if(D.state==playing_video) path_add_right_click(prec_x,prec_y); 
		}
		else if(cheat_allowed && superflag && squares[x][y].opened)
		{
			if(squares[x][y].num && squares[x][y].num>=closed_sq_around(x,y))
				flag_around(x,y);
		}
	}
	cur_x=x;cur_y=y;
}
void board::right_click(int x,int y,int prec_x,int prec_y)
{
	if(!right) return;
	if(D.state==playing) write_mouse_event(me_right_click, prec_x, prec_y);
	right=shift_left=false;
	if(!is_inside_board(x,y))
	{
		emit click();
		chorded=left;
		onedotfive=false;
		return;
	}
	if(left)	// chord
	{
		pop_around(cur_x,cur_y);
		if(D.state==playing_video) path_add_double_click(prec_x,prec_y);
		chord(x,y);
		++D.d_clicks;
		chorded=true;
		if(D.state==playing || D.state==playing_video) emit click();
	}
	else
	{
		if(!onedotfive && !chorded) 	// it hasn't produced nor a flag, neither a chord
		{
			++D.r_clicks;	// it was a right click after all, not a beginning of a chord
			++D.wasted_r_clicks;
		}
		chorded=false;
	}
	onedotfive=false;
	cur_x=x;cur_y=y;
}
void board::middle_press(int x,int y,int prec_x,int prec_y)
{
	if(D.state==playing) write_mouse_event(me_middle_press, prec_x, prec_y);
	shift_left=left=right=onedotfive=chorded=false;	// the middle button resets everything
	middle=true;
	if(!is_inside_board(x,y))
	{
		emit click();
		return;
	}
	push_around(x,y);
	emit press();
}
void board::middle_click(int x,int y,int prec_x,int prec_y)
{
	if(!middle) return;
	middle=false;
	if(D.state==playing) write_mouse_event(me_middle_click, prec_x, prec_y);
	if(!is_inside_board(x,y))
	{
		emit click();
		return;
	}
	if(D.state==playing_video) path_add_double_click(prec_x,prec_y);
	chord(x,y,false);
	++D.d_clicks;
	emit click();
}
void board::mouse_move(int x,int y,int prec_x,int prec_y)
{
	if(D.state==playing) write_mouse_event(me_mouse_move, prec_x, prec_y);
	if(is_inside_board(x,y))
	{
		if((left && right) || middle || shift_left)	// 
		{
			if(cur_x!=x || cur_y!=y)
			{
				pop_around(cur_x,cur_y);
				push_around(x,y);
			}
		}
		else if(cheat_allowed && superclick && left && squares[cur_x][cur_y].opened)
		{
			pop_around(cur_x,cur_y);
			if(squares[x][y].opened)
				push_around(x,y);
			else 
				push(x,y);
		}
		else if(left && !chorded)
		{
			if(cur_x!=x || cur_y!=y)
			{
				pop(cur_x,cur_y);
				push(x,y);
			}
			if(cheat_allowed && nono && (cur_x!=x || cur_y!=y))
			{
				bool sl=shift_left;
				left_click(x,y,cur_x,cur_y,false);
				left=true;
				shift_left=sl;
			}
		}
	}
	D.distance+=abs(cur_prec_x-prec_x)+abs(cur_prec_y-prec_y);	// we use the manhattan metric
	if(D.state==playing_video) path_add_mouse_move(prec_x,prec_y);
	cur_prec_x=prec_x;cur_prec_y=prec_y;
	if(is_inside_board(x,y))
	{
		cur_x=x;cur_y=y;
	}
}

void board::put_mine(int x,int y)
{
	if( (squares[x][y].mine=!squares[x][y].mine) ) 
		change_n_mines(D.n_mines+1);
	else 
		change_n_mines(D.n_mines-1);
	init_board();
	emit changed_properties();	// a signal for counters
	for(int i=x-1;i<=x+1;++i) for(int j=y-1;j<=y+1;++j) show_after_end(i,j);
	update();
}

void board::reset_counters()
{
	D.reset();
	shift_left=left=right=middle=onedotfive=chorded=false;
}

bool board::is_inside_board(int x,int y)
{
	return x>0 && x<=D.width && y>0 && y<=D.height;
}
bool board::is_inside(int x,int y)
{
	return x>=0 && x<=D.width+1 && y>=0 && y<=D.height+1;
}

void board::mousePressEvent(QMouseEvent* e)
{
	const Qt::KeyboardModifiers shift_ctrl=Qt::ShiftModifier|Qt::ControlModifier;
	int x=e->x()/sk->square_size+cur_scr_x;
	int y=e->y()/sk->square_size+cur_scr_y;
	if(e->button()==Qt::MidButton && !middle_chord) 
	{
		emit force_new_game();
		return;
	}
	if(D.state>playing || editing) return;
	switch(e->button())
	{
	case Qt::LeftButton:
		if(QApplication::keyboardModifiers()&shift_ctrl)
			left_press_with_shift(x,y,e->x()+cur_scr_x*sk->square_size,e->y()+cur_scr_y*sk->square_size);
		else
			left_press(x,y,e->x()+cur_scr_x*sk->square_size,e->y()+cur_scr_y*sk->square_size);
		break;
	case Qt::RightButton:
		if(!nf) right_press(x,y,e->x()+cur_scr_x*sk->square_size,e->y()+cur_scr_y*sk->square_size);
		break;
	case Qt::MidButton:
		if(!nf) middle_press(x,y,e->x()+cur_scr_x*sk->square_size,e->y()+cur_scr_y*sk->square_size);
		break;
	default:
		break;
	}
}

void board::mouseReleaseEvent(QMouseEvent* e)
{
	int x=e->x()/sk->square_size+cur_scr_x;
	int y=e->y()/sk->square_size+cur_scr_y;
	if(D.state>playing) return;
	switch(e->button())
	{
	case Qt::LeftButton:
		if(editing)
			put_mine(x,y);
		else
			left_click(x,y,e->x()+cur_scr_x*sk->square_size,e->y()+cur_scr_y*sk->square_size);
		break;
	case Qt::RightButton:
		if(!nf && !editing)
			right_click(x,y,e->x()+cur_scr_x*sk->square_size,e->y()+cur_scr_y*sk->square_size);
		break;
	case Qt::MidButton:
		if(middle_chord && !nf && !editing) 
			middle_click(x,y,e->x()+cur_scr_x*sk->square_size,e->y()+cur_scr_y*sk->square_size);
	default:
		break;
	}
}

void board::mouseMoveEvent(QMouseEvent* e)
{
	int x=e->x()/sk->square_size+cur_scr_x;
	int y=e->y()/sk->square_size+cur_scr_y;
	if(!is_inside_board(x,y) && is_inside_board(cur_x,cur_y))
	{
		pop_around(cur_x,cur_y);
	}
	if(D.state>playing) return;
	mouse_move(x,y,e->x()+cur_scr_x*sk->square_size,e->y()+cur_scr_y*sk->square_size);
	if(cheat_allowed && probabilities && D.state==playing) 
	{
		if(squares[x][y].opened)
			setToolTip("");
		else
			setToolTip(QString::number(squares[x][y].probability*100,'f',2)+'%');
	}
}

void board::paintEvent(QPaintEvent* e)
{
	painter.begin(this);
	int x1=e->rect().left()/sk->square_size,x2=(e->rect().right()-1)/sk->square_size;
	int y1=e->rect().top()/sk->square_size,y2=(e->rect().bottom()-1)/sk->square_size;

	for(int i=x1;i<=x2;++i)
		for(int j=y1;j<=y2;++j)
			painter.drawPixmap(sk->square_size*i,
				sk->square_size*j,
				*(squares[cur_scr_x+i][cur_scr_y+j].d=squares[cur_scr_x+i][cur_scr_y+j].p));
	
	if(cheat_allowed && probabilities && D.state==playing && !left)
		draw_probabilities();		
	
	if(D.state>=playing_video)	// draw yellow arrow and mouse path
		draw_video_stuff();
	
	painter.end();
}
void board::draw_probabilities()
{
	int w=D.width>sk->max_width ? sk->max_width : D.width;
	int h=D.height>sk->max_height ? sk->max_height : D.height;
	
	const int pr_rect=sk->square_size/2-1;
	const int pr_rect_b=sk->square_size/4;
	const int pr_fr1_b=pr_rect_b-1;
	const int pr_fr1_e=pr_rect_b+pr_rect;
	const int pr_fr2_b=pr_fr1_e+1;
	const int pr_fr2_e=pr_rect_b;
	for(int i=0;i<=w+1;++i)
	{
		int ii=i+cur_scr_x;
		if(ii>D.width || ii<1) continue;
		for(int j=0;j<=h+1;++j)
		{
			int jj=j+cur_scr_y;
			if(jj>D.height || jj<1) continue;
			if(squares[ii][jj].openable() && squares[ii][jj].probability>=0 &&
			 squares[ii][jj].probability<=1)
			{
				QColor prcolor;
				if(squares[ii][jj].probability>0.5)	// magic colors
				{
					prcolor.setRed(255);
					prcolor.setGreen((int)( (1-squares[ii][jj].probability)*2*255.0 ));
				}
				else
				{
					prcolor.setRed((int)( 2*squares[ii][jj].probability*255.0 ));
					prcolor.setGreen(255);
				}
				painter.fillRect(sk->square_size*i+pr_rect_b,sk->square_size*j+pr_rect_b,
					pr_rect,pr_rect,QBrush(prcolor));
				painter.setPen(sk->display_frame1);	// maybe introduce a different color
				painter.drawLine(sk->square_size*i+pr_fr1_b,sk->square_size*j+pr_fr1_b,
					sk->square_size*i+pr_fr1_e,sk->square_size*j+pr_fr1_b);
				painter.drawLine(sk->square_size*i+pr_fr1_b,sk->square_size*j+pr_fr1_b,
					sk->square_size*i+pr_fr1_b,sk->square_size*j+pr_fr1_e);
				painter.setPen(sk->display_frame2);
				painter.drawLine(sk->square_size*i+pr_fr2_b,sk->square_size*j+pr_fr2_b,
					sk->square_size*i+pr_fr2_b,sk->square_size*j+pr_fr2_e);
				painter.drawLine(sk->square_size*i+pr_fr2_b,sk->square_size*j+pr_fr2_b,
					sk->square_size*i+pr_fr2_e,sk->square_size*j+pr_fr2_b);
			}
		}
	}
}
void board::draw_video_stuff()
{
	int norm_ev_x=arrow_x-cur_scr_x*sk->square_size;
	int norm_ev_y=arrow_y-cur_scr_y*sk->square_size;
	
	if(v_mouse_path) draw_mouse_path();
	if(v_openings) draw_opening_borders();
	QPoint arrow[3]={QPoint(norm_ev_x,norm_ev_y),
				QPoint(norm_ev_x,norm_ev_y+sk->square_size),
				QPoint(norm_ev_x+2*sk->square_size/3,norm_ev_y+2*sk->square_size/3)};
	painter.setOpacity(sk->arrow_opacity);
	painter.setPen(Qt::NoPen);
	painter.setBrush(sk->arrow);
	painter.drawPolygon(arrow,3);
	painter.setPen(sk->arrow_border);
	painter.setBrush(Qt::NoBrush);
	painter.drawPolygon(arrow,3);
	painter.setOpacity(1);
}
void board::draw_mouse_path()
{
	painter.setOpacity(0.5);
	painter.fillRect(sk->square_size,sk->square_size,
		sk->square_size*D.width,sk->square_size*D.height,QBrush(QColor(0,0,0)));
	painter.setOpacity(1);

	int mod_x=-cur_scr_x*sk->square_size;
	int mod_y=-cur_scr_y*sk->square_size;
	const int mp_rect=sk->mouse_path_rect_size;
	const int mp_off=-mp_rect/2;
	
	for(int i=0;i<mouse_path.size();++i)
	{
		if(i && (mouse_path[i].id & 0x4)) // move
		{
			painter.setPen(sk->mouse_path[mouse_path[i].id & 3]);
			painter.drawLine(mouse_path[i-1].x+mod_x,mouse_path[i-1].y+mod_y,
				 mouse_path[i].x+mod_x,mouse_path[i].y+mod_y);
		}
		else	// click
		{
			painter.fillRect(mouse_path[i].x+mod_x+mp_off,mouse_path[i].y+mod_y+mp_off, mp_rect, mp_rect,
			 	QBrush(sk->mouse_path[mouse_path[i].id & 3]));
		}
	}
}
void board::draw_opening_borders()
{
	const int bw=sk->square_size/2;
	const int dbw=sk->square_size-bw;
	const int th=sk->opening_border_thickness;
	const int offset1=-th/2;
	const int offset2=th+offset1-1;
	const int offset3=th+offset1;
	int w=D.width>sk->max_width ? sk->max_width : D.width;
	int h=D.height>sk->max_height ? sk->max_height : D.height;

	const QBrush ob=QBrush(sk->opening_border);
	for(int i=0;i<=w+1;++i)
	{
		int ii=i+cur_scr_x;
		if(ii<1 || ii>D.width) continue;
		for(int j=0;j<=h+1;++j)
		{
			int jj=j+cur_scr_y;
			if(jj<1 || jj>D.height) continue;
			int o=squares[ii][jj].opborder;
			if(o==1 || o==7 || o==8 || o==11)
				painter.fillRect(sk->square_size*i+bw+offset1,sk->square_size*j,th,sk->square_size,
								 ob);
			if(o==2 || o==9 || o==10 || o==11)
				painter.fillRect(sk->square_size*i,sk->square_size*j+bw+offset1,sk->square_size,th,
								 ob);
			if(o==3 || o==5 || o==8)
				painter.fillRect(sk->square_size*i,sk->square_size*j+bw+offset1,bw+offset3,th,
								 ob);
			if(o==4 || o==6 || o==7)
				painter.fillRect(sk->square_size*i+bw+offset1,sk->square_size*j+bw+offset1,
								dbw+offset2,th,ob);
			if(o==3 || o==4 || o==9)
				painter.fillRect(sk->square_size*i+bw+offset1,sk->square_size*j,th,bw+offset2,
								 ob);
			if(o==5 || o==6 || o==10)
				painter.fillRect(sk->square_size*i+bw+offset1,sk->square_size*j+bw+offset1,th,
								dbw+offset2,ob);
			if(o<0)
			{
				painter.setPen(sk->opening_border);
				if(o<=-10)
					painter.drawText(sk->square_size*i,sk->square_size*j,
						sk->square_size,sk->square_size,0,QString::number(-o));
				else
					painter.drawText(sk->square_size*i+bw,sk->square_size*j,
						dbw,sk->square_size,0,QString::number(-o));
			}
		}
	}
}

void board::new_game()
{
	if(D.state==playing) write_game_event(ge_term_weird);	// game was interrupted by smiley
	restart_on_first_click=false;
	emit display_player(player);	//  restore player id
	if(marathon)	// resize on marathon
		resize_if_needed(marathon_description[n_finished_boards].width,
				marathon_description[n_finished_boards].height,
				marathon_description[n_finished_boards].mines);
	reset_counters();
	set_state(before_game);
	clear_board();
	update();
	if(editing) D.n_mines=0;
}

void board::erase_board()
{
	has_layout=initialized=has_opening_borders=false;
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
			squares[i][j].erase_board_info();
}
void board::clear_board()
{
	has_layout=initialized=has_opening_borders=false;
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
		{
			squares[i][j].new_game();
			squares[i][j].p=&sk->sq;
		}
}
void board::restart(bool video)
{
	reset_counters();
	if(initialized) reinit_properties(); else init_board();
	if(!video) set_state(before_game);
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
		{
			squares[i][j].restart();
			squares[i][j].p=&sk->sq;
		}
	update();
	restart_on_first_click=true;
}
void board::restart_from_blast()
{
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
			if(squares[i][j].mine && squares[i][j].opened)
			{
				squares[i][j].restart();
				squares[i][j].p=&sk->sq;
				--D.open_mines;
			}
	update();
	cur_lives=1;
	if(D.state==fail) replace_fail_with_unblast();	// fix the video
	set_state(playing);
}

int board::remaining_mines()
{
	int res=0;
	for(int i=n_finished_boards;i<marathon_description.size();++i)
		res+=marathon_description[i].mines;
	return res;
}

void board::load_board(const QString& filename)
{
	QFile brd(filename);
	int w,h,n;

	if(!brd.open(QIODevice::ReadOnly))
	{
		err(T->str[ins_e_open_file]);
		return;
	}
	has_layout=initialized=has_opening_borders=false;
	// trying to determine file type by the extension
	if(filename.right(5)=="rawvf")	// raw video file
	{
		QString opt,val;
		QMap<QString,QString> options;
		
		// read options
		while(opt!="board")
		{
			QString line;
			int sp;
			
			if(brd.atEnd())
			{
				err(T->str[ins_e_corrupt_rawvf]);
				return;
			}

			line=QString::fromUtf8(brd.readLine());
			// if it's not a valid entry ("Name : Value"), ignore
			if( (sp=line.indexOf(':')) == -1) continue;	
			opt=line.left(sp).trimmed().toLower();
			val=line.mid(sp+1).trimmed();
			if(val=="") continue;
			options[opt]=val;	
		}
		// initialize board
		// read size and mine number
		
		if(options["level"]=="marathon")
		{
			err(T->str[ins_e_rawvf_marathon]);
			return;
		}
		
		w=options["width"].toInt();
		h=options["height"].toInt();
		n=options["mines"].toInt();	
		resize_if_needed(w,h,n);
		clear_board();

		// place mines
		for(int j=1;j<=w;++j)
		{
			for(int i=1;i<=h;++i)
			{
				char c;
				brd.getChar(&c);
				squares[i][j].mine=(c=='*');
			}
			brd.readLine();
		}

		set_state(before_game);
		has_layout=true;
	}
	else if(filename.right(3)=="mbf" || filename.right(3)=="abf") // clone format
	{
		unsigned char b1,b2;
		int x,y;
		QDataStream dstr(&brd);

		dstr>>b1;w=b1;
		dstr>>b1;h=b1;
		dstr>>b1>>b2;n=b1*256+b2;
		resize_if_needed(w,h,n);
		clear_board();
		for(int i=1;i<=D.width;++i) for(int j=1;j<=D.height;++j) squares[i][j].mine=0;
		for(int i=0;i<D.n_mines;++i)
		{
			dstr>>b1>>b2;
			x=b1+1;y=b2+1;
			if(!is_inside_board(x,y))
			{
				err(T->str[ins_e_invalid_mine]);
				return;
			}
			squares[x][y].mine=1;
		}
		brd.close();

		set_state(before_game);
		has_layout=true;
	}
	else err(T->str[ins_e_unknown_brd]);

	if(editing) 
	{
		init_numbers();
		for(int i=1;i<=D.width;++i) for(int j=1;j<=D.height;++j) show_after_end(i,j);
		repaint();
	}
	else restart(false);
}

void board::save_board(const QString& filename)
{
	// we'll use the nice Clone format
	if(D.width>255 || D.height>255) 
	{
		err(T->str[ins_e_large_board]);
		return;
	}
	
	QFile brd(filename);
	QDataStream dstr(&brd);
	if(!brd.open(QIODevice::WriteOnly | QIODevice::Truncate))
	{
		err(T->str[ins_e_open_file]);
		return;
	}
	dstr<<(unsigned char)D.width<<(unsigned char)D.height;
	dstr<<(unsigned char)(D.n_mines/256)<<(unsigned char)(D.n_mines%256);
	for(int i=1;i<=D.width;++i) 
		for(int j=1;j<=D.height;++j)
			if(squares[i][j].mine)
				dstr<<(unsigned char)(i-1)<<(unsigned char)(j-1);
	brd.close();
}


