#include "counterseditor.h"

void counters_editor::init()
{
	layout=new QGridLayout(this);
	l_name.setParent(this);name.setParent(this);
	l_formula.setParent(this);formula.setParent(this);
	l_precision.setParent(this);precision.setParent(this);

	l_name.setText(T->str[ins_label]);
	l_formula.setText(T->str[ins_formula]);
	l_precision.setText(T->str[ins_decimal]);
	apply.setText(T->str[ins_apply]);
	addctr.setText(T->str[ins_add]);
	delctr.setText(T->str[ins_remove]);
	editformula.setText(T->str[ins_open_fe]);
	rec.setText(T->str[ins_records]);
	l_rectype.setText(T->str[ins_better_is]+" ");
	l_rectype.setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	rectype.setEditable(false);
	rectype.addItem(T->str[ins_lower]);
	rectype.addItem(T->str[ins_higher]);
	recpopup.setText(T->str[ins_popup]);
	ok.setText(T->str[ins_exit]);
	precision.setRange(0,5);
	t.insertColumn(0);
	t.insertColumn(1);
	t.insertColumn(2);

	layout->addWidget(&t,0,0,1,4);
	layout->addWidget(&addctr,1,0,1,1);
	layout->addWidget(&delctr,1,1,1,1);
	layout->addWidget(&editformula,1,2,1,1);
	layout->addWidget(&l_name,2,0,1,1);
	layout->addWidget(&name,2,1,1,2);
	layout->addWidget(&l_formula,3,0,1,1);
	layout->addWidget(&formula,3,1,1,2);
	layout->addWidget(&apply,3,3,1,1);
	layout->addWidget(&l_precision,4,0,1,1);
	layout->addWidget(&precision,4,1,1,1);
	layout->addWidget(&rec,5,0,1,1);
	layout->addWidget(&l_rectype,5,1,1,1);
	layout->addWidget(&rectype,5,2,1,1);
	layout->addWidget(&recpopup,5,3,1,1);
	layout->addWidget(&ok,6,0,1,4);

	id=-1;enable(false);

	connect(&apply,SIGNAL(clicked()),this,SLOT(apply_formula()));
	connect(&addctr,SIGNAL(clicked()),this,SLOT(add()));
	connect(&delctr,SIGNAL(clicked()),this,SLOT(remove()));
	connect(&editformula,SIGNAL(clicked()),this,SLOT(f_editor()));
	connect(&ok,SIGNAL(clicked()),this,SLOT(quit()));
	connect(&name,SIGNAL(textEdited(const QString &)),this,SLOT(change_name(const QString&)));
	connect(&precision,SIGNAL(valueChanged(int)),this,SLOT(change_precision(int)));
	connect(&rec,SIGNAL(clicked()),this,SLOT(change_record()));
	connect(&rectype,SIGNAL(currentIndexChanged(int)),this,SLOT(change_record_type(int)));
	connect(&recpopup,SIGNAL(clicked()),this,SLOT(change_record_popup()));

	connect(&t,SIGNAL(currentCellChanged(int,int,int,int)),this,SLOT(choose(int,int,int,int)));

	setMinimumWidth(400);
}
int counters_editor::id_to_ctr(int id)
{
	int i;
	for(i=1;i<counters.size() && id;++i)
		if(counters[i] && counters[i]->formula && (e_st || !counters[i]->type)) --id;
	for(;i<counters.size() && (!counters[i] || !counters[i]->formula || (!e_st && counters[i]->type));++i);
	return i;
}

void counters_editor::refresh()
{
	int row=0;

	disconnect(&t,SIGNAL(currentCellChanged(int,int,int,int)),this,SLOT(choose(int,int,int,int)));
	t.clear();
	t.setHorizontalHeaderItem(0,new QTableWidgetItem(T->str[ins_label]));
	t.setHorizontalHeaderItem(1,new QTableWidgetItem(T->str[ins_formula]));
	t.setHorizontalHeaderItem(2,new QTableWidgetItem(T->str[ins_precision]));
	for(int i=1;i<counters.size();++i)
		if(counters[i] && counters[i]->formula && (e_st || !counters[i]->type))
		{
			QTableWidgetItem* l_n=new QTableWidgetItem();
			QTableWidgetItem* l_f=new QTableWidgetItem();
			QTableWidgetItem* l_p=new QTableWidgetItem();
			l_n->setText(counters[i]->get_name());
			l_n->setFlags(l_f->flags() & (~Qt::ItemIsSelectable & ~Qt::ItemIsEditable));
			l_f->setText(counters[i]->formula->print());
			l_f->setFlags(l_f->flags() & (~Qt::ItemIsSelectable & ~Qt::ItemIsEditable));
			l_p->setText(QString::number(counters[i]->formula->precision));
			l_p->setFlags(l_f->flags() & (~Qt::ItemIsSelectable & ~Qt::ItemIsEditable));
			if(t.rowCount()<=row) t.insertRow(row);
			t.setItem(row,0,l_n);
			t.setItem(row,1,l_f);
			t.setItem(row++,2,l_p);
		}
	t.setRowCount(row);
	t.resizeColumnsToContents();
	connect(&t,SIGNAL(currentCellChanged(int,int,int,int)),this,SLOT(choose(int,int,int,int)));
}

void counters_editor::enable(bool on)
{
	name.setEnabled(on);
	formula.setEnabled(on);
	editformula.setEnabled(on);
	apply.setEnabled(on);
	precision.setEnabled(on);
	delctr.setEnabled(on);
	rec.setEnabled(on);
	rectype.setEnabled(on && rec.isChecked());
	recpopup.setEnabled(on && rec.isChecked());
}

void counters_editor::choose(int row,int,int,int)
{
	if(row<0 || row>=t.rowCount())
	{
		enable(false);
		return;
	}
	id=row;
	ctr=id_to_ctr(id);
	enable(true);
	name.setText(t.item(id,0)->text());
	formula.setText(t.item(id,1)->text());
	disconnect(&precision,SIGNAL(valueChanged(int)),this,SLOT(change_precision(int)));
	precision.setValue(t.item(id,2)->text().toInt());
	connect(&precision,SIGNAL(valueChanged(int)),this,SLOT(change_precision(int)));
	rec.setChecked(counters[ctr]->rec);
	if(rec.isChecked())
	{
		rectype.setCurrentIndex(counters[ctr]->rec->lower_is_better);
		recpopup.setChecked(counters[ctr]->rec->popup);
	}
	else 
	{
		rectype.setCurrentIndex(0);
		recpopup.setChecked(false);
	}
}

void counters_editor::add()
{
	emit add_counter();
	refresh();
	choose(id,0,-1,-1);
}

void counters_editor::remove()
{
	emit delete_counter(ctr);
	refresh();
	id=-1;enable(false);
}

void counters_editor::apply_formula()
{
	c_formula* f;
	if(( f=p->parse_formula(formula.text()) ))
	{
		if(counters[ctr]->formula) delete counters[ctr]->formula;
		counters[ctr]->formula=f;
		refresh();
		choose(id,0,-1,-1);
		changed=true;
	}
}

void counters_editor::quit()
{
	hide();
}

void counters_editor::f_editor()
{
	formula_editor fed(fet_counter_formula,T,sk,p);
	fed.get_formula(formula.text());
	if(fed.resc) 
	{
		if(counters[ctr]->formula) delete counters[ctr]->formula;
		counters[ctr]->formula=fed.resc;
		refresh();	
		choose(id,0,-1,-1);
		changed=true;
	}
}

void counters_editor::change_name(const QString& n)
{
	if(counters[ctr])
	{
		QString nn=n;
		nn.remove(QChar('|'));
		counters[ctr]->set_name(nn);
		refresh();
		choose(id,0,-1,-1);
		changed=true;
	}
}
void counters_editor::change_precision(int p)
{
	if(counters[ctr])
	{
		counters[ctr]->formula->precision=p;
		refresh();
		choose(id,0,-1,-1);
		changed=true;
	}
}
void counters_editor::change_record()
{
	bool r=rec.isChecked();
	rectype.setEnabled(r);
	recpopup.setEnabled(r);
	if(counters[ctr])
	{
		if(r)
			counters[ctr]->reset_records();
		else
		{
			if(counters[ctr]->rec) delete counters[ctr]->rec;
			counters[ctr]->rec=0;
		}
		refresh();
		choose(id,0,-1,-1);
		changed=true;
	}
}
void counters_editor::change_record_type(int i)
{
	if(counters[ctr] && counters[ctr]->rec) 
	{
		counters[ctr]->rec->lower_is_better=i;
		changed=true;
	}
}
void counters_editor::change_record_popup()
{
	if(counters[ctr] && counters[ctr]->rec) 
	{
		counters[ctr]->rec->popup=recpopup.isChecked();
		changed=true;
	}
}

counters_editor::counters_editor(translation* T,skin* sk,QList<counter*>& counters,parser* p,bool& changed,bool e_st):
T(T),sk(sk),counters(counters),p(p),changed(changed),e_st(e_st)
{
	init();
	refresh();
	setWindowTitle(T->str[ins_counters_editor]);
	setWindowIcon(sk->program_icon);
}

