#include "counterswindow.h"

const QString F_counters="freesweeper.counters";
const QString F_records="freesweeper.records";

counters_window::counters_window(translation* T,parser* p,evaluator* ev,skin* sk):
T(T),sk(sk),rows(0),columns(0),moving(false),cur_time(0),Parser(p),Eval(ev),
counters_changed(settings_dir!=normal_settings_dir),id_time(0)
{
	f=windowFlags();
	f&=~(Qt::WindowMinMaxButtonsHint);
	f|=Qt::Drawer;
	setWindowFlags(f);
	resize(sk->counter_width*(columns+1),sk->counter_height*(rows+1));
	connect(&menumapper,SIGNAL(mapped(int)),this,SLOT(ctr_menu(int)));
	setWindowTitle(T->str[ins_counters]);
}

void counters_window::build_menu()
{
	QString menu_names[]={"",T->str[ins_benchmarks],T->str[ins_speed],T->str[ins_quality],T->str[ins_eff],
		T->str[ins_click_rate],T->str[ins_flags],T->str[ins_brd_prop],T->str[ins_ranks],
		T->str[ins_custom_ind]};

	for(int i=0;i<=n_menues;++i) menues[i]=new QMenu(menu_names[i],0);
	
/*	a_addctr=new QAction(T->str[ins_add_ctr],this);
	connect(a_addctr,SIGNAL(triggered()),this,SLOT(add_custom_ctr()));
	a_ctrctl=new QAction(T->str[ins_cus_ctr_mgr],this);
	connect(a_ctrctl,SIGNAL(triggered()),this,SLOT(ctr_manager()));*/ //{R1}
	a_locked=new QAction(T->str[ins_locked],this);
	a_locked->setCheckable(true);	
	connect(a_locked,SIGNAL(triggered()),this,SLOT(set_locked()));
	setContextMenuPolicy(Qt::PreventContextMenu);

	for(int i=1;i<counters.size();++i)
	{
		menumapper.setMapping(&(counters[i]->menu),i);
		connect(&(counters[i]->menu),SIGNAL(triggered()),&menumapper,SLOT(map()));
	}

	update_menu();
}

void counters_window::update_menu()
{
	for(int i=0;i<=n_menues;++i) menues[i]->clear();
	for(int i=0;i<counters.size();++i)
		if(counters[i])
		{
			int t=counters[i]->type;
			if(t>0 && t<=n_menues) menues[t]->addAction(&counters[i]->menu);
			else if(t==-1) menues[0]->addAction(&counters[i]->menu);
			else menues[n_menues]->addAction(&counters[i]->menu);
		}
	for(int i=1;i<n_menues;++i) menues[0]->addMenu(menues[i]);
	menues[0]->addSeparator();
	menues[0]->addMenu(menues[n_menues]);
	menues[0]->addSeparator();
	menues[0]->addAction(a_locked);
/*	menues[n_menues]->addAction(a_addctr);
	menues[n_menues]->addAction(a_ctrctl);*/ //{R1}
}

void counters_window::set_locked()
{
	locked=a_locked->isChecked();
}

void counters_window::ctr_manager()
{
	bool changed=false;
	counters_editor CE(T,sk,counters,Parser,changed,edit_standard);
	connect(&CE,SIGNAL(add_counter()),this,SLOT(add_custom_ctr()));
	connect(&CE,SIGNAL(delete_counter(int)),this,SLOT(delete_counter(int)));
	CE.exec();
	disconnect(&CE,SIGNAL(add_counter()),this,SLOT(add_custom_ctr()));
	disconnect(&CE,SIGNAL(delete_counter(int)),this,SLOT(delete_counter(int)));
	if(changed) find_time_counter();
	counters_changed=changed||counters_changed;
}

void counters_window::find_time_counter()
{
	id_time=0;
	for(int i=1;i<counters.size();++i)
		if(!id_time && counters[i]->get_name().toLower()=="time" && counters[i]->get_menu().toLower()=="time")
			id_time=i;
	if(!id_time)
	{
		QString stand_time_counter="time [Time] {-1} | Time | r< ";
		counter* nc=Parser->parse_counter(stand_time_counter);
		new_counter(nc);
		counters.removeFirst();
		counters.push_front(nc);
		counters.push_front(0);
		id_time=1;
		counters_changed=true;
	}
}

counters_window::~counters_window()
{
	for(int i=0;i<counters.size();++i)
		if(counters[i]) delete counters[i];
}

void counters_window::add_custom_ctr()
{
	QString n=input(this,T->str[ins_request_ind_name],"");
	if(n=="") return;

	formula_editor fed(fet_counter_formula,T,sk,Parser);
	fed.get_formula(0);
	if(fed.resc)
	{
		counter* nc=new counter;
		nc->formula=fed.resc;
		nc->set_name(n);nc->set_menu(n);
		int id=counters.size();
		counters << nc;
		new_counter(nc);
		
		menumapper.setMapping(&nc->menu,id);
		connect(&nc->menu,SIGNAL(triggered()),&menumapper,SLOT(map()));

		update_menu();		

		counters_changed=true;
	}
}
void counters_window::delete_counter(int id)
{
	if(id<0 || id>=counters.size() || !counters[id]) return;
	for(QMap<coord,counter*>::iterator i=pool.begin();i!=pool.end();++i)
	{
		if(i.value()==counters[id]) pool[i.key()]=0;
	}
	delete counters[id];counters[id]=0;
	counters_changed=true;
	update_menu();
}

void counters_window::save()
{
	QFile f(settings_dir+F_counters);
	settings s(settings_dir+F_records);
	
	if(counters_changed)
	{
		if(!f.open(QIODevice::WriteOnly | QIODevice::Truncate))
			err(T->str[ins_e_open_counters]);
		else
		{
			for(QList<counter*>::iterator i=counters.begin();i!=counters.end();++i)	
				if(*i && (*i)->formula) 
				{
					writeln(f,(*i)->print());
				}
			f.close();
		}
	}
	for(QList<counter*>::iterator i=counters.begin();i!=counters.end();++i)	
		if(*i && (*i)->formula) 
		{
			if((*i)->rec) (*i)->save_records(&s);
		}
	s.write(T->str[ins_e_open_records]);
}

void counters_window::new_counter(counter* nc)
{
	nc->initialize(this,sk);
	if(nc->row>=0 && nc->column>=0) 
	{
		place(nc->row,nc->column,nc);
		nc->setVisible(true);
	}
	else
		nc->setVisible(false);
}

void counters_window::load()
{
	QFile f(settings_dir+F_counters);
	settings s(settings_dir+F_records);
	
	s.read(T->str[ins_e_open_records]);
	counters << 0; // there is no counter #0
	if(!f.open(QIODevice::ReadOnly))
	{
		counters_changed=true;
		err(T->str[ins_e_open_counters]);
	}
	else
	{
		while(!f.atEnd())
		{
			QString counter_desc=readln_without_comments(f).trimmed();
			if(counter_desc=="") continue;
			counter* nc=Parser->parse_counter(counter_desc);
			if(!nc) continue;
			new_counter(nc);
			counters << nc;
			nc->load_records(&s);
		}
		f.close();
	}
	find_time_counter();
	build_menu();
	a_locked->setChecked(locked);
	adjust_size();
}

void counters_window::add_counter(counter* ctr)
{
	place(++rows,0,ctr);
	ctr->refresh(Eval->eval(ctr->formula));
	ctr->setVisible(true);
	counters_changed=true;
	resize(sk->counter_width*(columns+1),sk->counter_height*(rows+1));
}
void counters_window::add_counter(counter* ctr,int row,int column)
{
	if(ctr && ctr->row==-1)
	{
		place(row,column,ctr);
		ctr->setVisible(true);
	}
}

void counters_window::hide_counter(coord c)
{
	if(!pool[c]) return;
	pool[c]->setVisible(false);
	pool[c]->row=pool[c]->column=-1;
	pool[c]=0;
	counters_changed=true;
}

void counters_window::refresh(int time)
{
	if(!isVisible()) return;
	Eval->refresh(cur_time=time);
	refresh();
}
void counters_window::refresh()
{
	for(QMap<coord,counter*>::iterator i=pool.begin();i!=pool.end();++i)
	{
		if(i.value())
		{
			i.value()->refresh(Eval->eval(i.value()->formula)); 
			if(i.value()->bold)
				i.value()->check_boldness(Eval->last_value,Eval->has_last_value);
			
		}
	}
}

QString counters_window::check_records(int level,bool flagging,QDateTime d)
{
	QString res="";
	only_time_record=true;
	for(int i=1;i<counters.size();++i)
		if(counters[i] && counters[i]->formula && counters[i]->rec)
		{
			double val=Eval->eval(counters[i]->formula->f);
			QString cur=counters[i]->rec->check(level,val,flagging,d);
			if(cur!="") 
			{
				res+=cur+counters[i]->get_menu();
				if(i!=id_time) only_time_record=false;
			}
		}
	return res;
}

void counters_window::set_record_names(const QString& name,int level)
{
	for(int i=1;i<counters.size();++i)
		if(counters[i] && counters[i]->formula && counters[i]->rec)
			counters[i]->rec->set_name(name,level);
}

void counters_window::mousePressEvent(QMouseEvent* e)
{
	if(e->button()==Qt::RightButton) 
	{
		menues[0]->popup(e->globalPos());
		return;
	}
	if(locked) return;
	cur_column=e->x()/sk->counter_width;
	cur_row=e->y()/sk->counter_height;
	if(!pool.contains(coord(cur_row,cur_column)))
		c_moving=0;
	else
		c_moving=pool.value(coord(cur_row,cur_column));
	if(c_moving) moving=true;
}
void counters_window::mouseReleaseEvent(QMouseEvent*)
{
	moving=false;
}
void counters_window::mouseMoveEvent(QMouseEvent* e)
{
	if(!moving) return;
	int column=e->x()/sk->counter_width,row=e->y()/sk->counter_height;
	if(row==cur_row && column==cur_column) return;

	place(cur_row,cur_column,pool[coord(row,column)]);
	place(row,column,c_moving);
	counters_changed=true;

	adjust_size();
	cur_column=column;cur_row=row;
}

void counters_window::showEvent(QShowEvent*)
{
	if(separated) setWindowFlags(f);
	adjust_size();
	show();
}
void counters_window::closeEvent(QCloseEvent*)
{
	emit closed();
}
void counters_window::place(int row,int column,counter* ctr)
{
	pool[coord(row,column)]=ctr;
	if(!ctr) return;
	ctr->move(column*sk->counter_width,row*sk->counter_height);
	ctr->row=row;
	ctr->column=column;
}

void counters_window::adjust_size()
{
	rows=columns=0;
	for(QMap<coord,counter*>::iterator i=pool.begin();i!=pool.end();++i)
	{
		if(i.value())
		{
			if(i.key().first>rows) rows=i.key().first;
			if(i.key().second>columns) columns=i.key().second;
		}
	}
	resize(sk->counter_width*(columns+1),sk->counter_height*(rows+1));
	if(!separated) emit resized();
	repaint();
}

void counters_window::ctr_menu(int id)
{
	if(id<=0 || id>=counters.size() || !counters[id]) return;
	if(counters[id]->menu.isChecked())
		add_counter(counters[id]);
	else
		hide_counter(coord(counters[id]->row,counters[id]->column));
}

