#ifndef _FS_COUNTERSWINDOW_H
#define _FS_COUNTERSWINDOW_H

#include "freesweeper.h"
#include "counter.h"
#include "settings.h"
#include "parser.h"
#include "eval.h"
#include "skin.h"
#include "counterseditor.h"
#include "formulaeditor.h"

typedef QPair<int,int> coord;

class counters_window : public QWidget
{
	Q_OBJECT

	translation* T;
	skin* sk;

	QMap<coord,counter*> pool;

	QAction* a_addctr;
	QAction* a_ctrctl;
	QAction* a_locked;
	int rows,columns;

	counter* c_moving;
	bool moving;
	int cur_row,cur_column;

	int cur_time;
	
	static const int n_menues=9;
	QMenu* menues[n_menues+1];

	QSignalMapper menumapper;

	Qt::WindowFlags f;
public:
	bool locked;
	
	QList<counter*> counters;

	parser* Parser;
	evaluator* Eval;

	bool attached;
	bool separated;

	bool counters_changed;

	bool edit_standard;

	int id_time;
	bool only_time_record;
	
	void add_counter(counter* ctr);
	void add_counter(counter* ctr,int row,int column);
	QString eval(int time,QString formula);

	void new_counter(counter* ctr);

	void find_time_counter();

	counters_window(translation* T,parser* p,evaluator* ev,skin* sk);
	~counters_window();

	void build_menu();
	void update_menu();
	
	void place(int x,int y,counter* ctr);
	void adjust_size();

	void load();
	void save();

	virtual void mousePressEvent(QMouseEvent *);
	virtual void mouseReleaseEvent(QMouseEvent *);
	virtual void mouseMoveEvent(QMouseEvent *);
	virtual void showEvent(QShowEvent *);
	virtual void closeEvent(QCloseEvent *);
public slots:
	void hide_counter(coord c);
	void refresh(int time);
	void refresh();
	QString check_records(int level,bool flagging,QDateTime d=QDateTime::currentDateTime());
	void set_record_names(const QString& name,int level);
	void ctr_menu(int id);

	void add_custom_ctr();
	void delete_counter(int id);
	void ctr_manager();

	void set_locked();
	
signals:
	void closed();
	void resized();
};

#endif

