#include "eval.h"
#include "history.h"

evaluator::evaluator(board_data* Board,const gamemode* mode):Board(Board),mode(mode),History(0)
{
	time=est=bbbvs=cl=wcl=nwcl=0;
}

QString evaluator::eval_special_cf(c_formula_type cft)
{
	QDateTime t=h_date?get_date():QDateTime::currentDateTime();
	const QString Level_names[n_levels+1]={"","Beginner","Intermediate","Expert","24x30","Custom"};
	const QString level_names[n_levels+1]={"","beginner","intermediate","expert","24x30","custom"};
	const QString Lvl_names[n_levels+1]={"","Beg","Int","Exp","Meg","Cus"};
	const QString lvl_names[n_levels+1]={"","beg","int","exp","meg","cus"};
	int tmp;
	
	switch(cft)
	{
	case cft_day:
		return QString::number(t.date().day());
	case cft_day_2:
		return (tmp=t.date().day())>9 ? QString::number(tmp) : "0"+QString::number(tmp);
	case cft_month_long:
		return QDate::longMonthName(t.date().month());
	case cft_month_short:
		return QDate::shortMonthName(t.date().month());
	case cft_month_2:
		return (tmp=t.date().month())>9 ? QString::number(tmp) : "0"+QString::number(tmp);
	case cft_year_2:
		return (tmp=t.date().year()%100)>9 ? QString::number(tmp) : "0"+QString::number(tmp);
	case cft_year_4:
		return QString::number(t.date().year());
	case cft_hour:
		return (tmp=t.time().hour())>9 ? QString::number(tmp) : "0"+QString::number(tmp);
	case cft_minute:
		return (tmp=t.time().minute())>9 ? QString::number(tmp) : "0"+QString::number(tmp);
	case cft_second:
		return (tmp=t.time().second())>9 ? QString::number(tmp) : "0"+QString::number(tmp);
	case cft_level_name_3:
		return lvl_names[Board->level];
	case cft_level_name:
		return level_names[Board->level];
	case cft_Level_name_3:
		return Lvl_names[Board->level];
	case cft_Level_name:
		return Level_names[Board->level];
	default:
		return "-";
	}
}

QString evaluator::eval(c_formula* cf)
{
	if(!cf) return "-";
	has_last_value=true;
	try
	{
		if(cf->type==cft_normal)
		{
			if(cf->separator=="")
				return QString::number(last_value=eval(cf->f),'f',cf->precision);
			else if(cf->tail)
				return QString::number(eval(cf->f),'f',cf->precision)+cf->separator+eval(cf->tail);
			else
				return QString::number(last_value=eval(cf->f),'f',cf->precision)+cf->separator;
		}
		else
		{
			has_last_value=false;
			return eval_special_cf(cf->type);
		}
	}
	catch(exceval)
	{
		has_last_value=false;
		return "-";
	}
}

double evaluator::eval(formula* f)
{
//	const double avg_obv[n_stand_levels+1]={0, 15, 62.5, 170};
	if(!f || throw_anyway) throw exceval();
	switch(f->id)	// counters showed in legal mode
	{
		double op1,op2;
	case f_none:
		return f->value;
	case f_int:
		return f->intvalue;
	case f_time:
		return time;
	case f_plus:
		return eval(f->op1)+eval(f->op2);
	case f_minus:
		return eval(f->op1)-eval(f->op2);
	case f_times:
		return eval(f->op1)*eval(f->op2);
	case f_div:
		op2=eval(f->op2);
		if(!op2) throw exceval();
		return eval(f->op1)/op2;
	case f_power:
		op1=eval(f->op1);
		op2=eval(f->op2);
		if(op1<0 || (!op1 && !op2)) throw exceval();
		if(!op1) return 0;
		return pow(op1,op2);
	case f_unary_minus:
		return -eval(f->op1);
	case f_sqr:
		op1=eval(f->op1);
		if(op1<0) throw exceval();
		return sqrt(op1);
	case f_exp:
		return exp(eval(f->op1));
	case f_log:
		op1=eval(f->op1);
		if(op1<=0) throw exceval();
		return log(op1);
	case f_level:
		return Board->level;
	case f_l_cl:
		return Board->l_clicks;
	case f_l_cl_s:
		if(time) return Board->l_clicks/time; else throw exceval();
	case f_r_cl:
		return Board->r_clicks;
	case f_r_cl_s:
		if(time) return Board->r_clicks/time; else throw exceval();
	case f_d_cl:
		return Board->d_clicks;
	case f_d_cl_s:
		if(time) return Board->d_clicks/time; else throw exceval();
	case f_cl_15:
		return Board->clicks_15;
	case f_cl_15_s:
		if(time) return Board->clicks_15/time; else throw exceval();
	case f_cl:
		return cl;
	case f_cl_s:
		if(time) return cl/time; else throw exceval();
	case f_width:
		return Board->width;
	case f_height:
		return Board->height;
	case f_mines:
		return Board->n_mines;
	case f_sh_squares:
		return Board->open_squares;
	case f_flags:
		return Board->flags;
	case f_path:
		return Board->distance;
	case f_mode:
		return (*mode);
	case f_unflags:
		return Board->unflags;
	case f_wasted_flags:
		return Board->wasted_flags;
	case f_games_finished:
		return History ? History->total_games[Board->level] : 0;
	case f_number_of_boards:
		return Board->n_boards;
	default:
		if(f->id>=f_sh_numbers && f->id<=f_sh_eights) 
			return Board->shown_numbers[f->id-f_sh_numbers];
	}
	if(!hist_request && !allow_stats && Board->state<=playing) throw exceval();
	switch(f->id) // prohibited counters
	{
	case f_est:
		if(Board->solved_bbbv) return est; else throw exceval();
	case f_3bv:
		return Board->bbbv;
	case f_3bv_s:
		if(time) return bbbvs; else throw exceval();
	case f_solved_3bv:
		return Board->solved_bbbv;
	case f_obv:
		throw exceval();
	case f_obv_s:
		throw exceval();
	case f_zini:
		return Board->zini;
	case f_zini_s:
		if(est) return Board->zini/est; else throw exceval();
	case f_hzini:
		return Board->hzini;
	case f_hzini_s:
		if(est) return Board->hzini/est; else throw exceval();
	case f_rqp:
		if(Board->solved_bbbv) return (est+1)/bbbvs; else throw exceval();
	case f_bat:
		throw exceval();
	case f_ioe:
		if(cl) return Board->solved_bbbv / cl; else throw exceval();
	case f_thrp:
		if(nwcl) return Board->solved_bbbv / nwcl; else throw exceval();
	case f_corr:
		if(cl) return nwcl / cl; else throw exceval();
	case f_zne:
		if(Board->solved_bbbv) return Board->zini*Board->solved_bbbv / (cl*Board->bbbv); 
		else throw exceval();
	case f_hzne:
		if(Board->solved_bbbv) return Board->hzini*Board->solved_bbbv / (cl*Board->bbbv); 
		else throw exceval();
	case f_znt:
		if(Board->solved_bbbv) return Board->zini*Board->solved_bbbv / (nwcl*Board->bbbv); 
		else throw exceval();
	case f_hznt:
		if(Board->solved_bbbv) return Board->hzini*Board->solved_bbbv / (nwcl*Board->bbbv); 
		else throw exceval();
	case f_ios:
		if(est>1 && Board->solved_bbbv) return log(Board->bbbv)/log(est); else throw exceval();
	case f_w_l_cl:
		return Board->wasted_l_clicks;
	case f_w_r_cl:
		return Board->wasted_r_clicks;
	case f_w_d_cl:
		return Board->wasted_d_clicks;
	case f_w_cl_15:
		return Board->wasted_clicks_15;
	case f_w_cl:
		return wcl;
	case f_ops:
		return Board->openings;
	case f_ops_hit:
		return Board->solved_ops;
	case f_isls:
		return Board->islands;
	case f_isls_solved:
		return Board->solved_isls;
	case f_wall_mines:
		return Board->mines_on_walls;
	case f_corner_mines:
		return Board->mines_in_corners;
	case f_misflags:
		return Board->misflags;
	case f_misunflags:
		return Board->misunflags;
	default:
		if(f->id>=f_numbers && f->id<=f_eights) return Board->numbers[f->id-f_numbers];
	}
	if(Board->state!=win || (*mode)>lucky || !History) throw exceval();
	// ranks
	if(f->id>=f_rank_time && f->id<=f_rank_corr)
		return History->get_named_rank(f->id-f_rank_time);
	else if(f->id==f_rank)	// custom rank
		return History->get_rank(f->intvalue);
	throw exceval();
}

QString evaluator::eval(evaluable_string* s)
{
	QString res;
	try
	{
		for(int i=0;i<s->strs.size();++i)
		{
			res+=s->strs[i];
			if(s->fs[i]) res+=eval(s->fs[i]);
		}
	}
	catch(exceval)
	{
	}
	return res;
}

bool evaluator::eval_expr(formula* ex)
{
	if(!ex) return false;
	try
	{
		switch(ex->id)
		{
		case f_and:
			return eval_expr(ex->op1) && eval_expr(ex->op2);
		case f_or:
			return eval_expr(ex->op1) || eval_expr(ex->op2);
		case f_less:
			return eval(ex->op1) < eval(ex->op2);
		case f_greater:
			return eval(ex->op1) > eval(ex->op2);
		case f_equal:
			return eval(ex->op1) == eval(ex->op2);
		default:
			return false;
		}
	}	
	catch(exceval)
	{
		return false;
	}
}

void evaluator::refresh(int _time)
{
	throw_anyway=!hist_request && !show_stats && Board->state<=playing;
	
	time=_time/1000.0;
	est=time*Board->bbbv / Board->solved_bbbv;
	bbbvs=Board->solved_bbbv/time;
	cl=Board->d_clicks+Board->r_clicks+Board->l_clicks;
	wcl=Board->wasted_d_clicks+Board->wasted_r_clicks+Board->wasted_l_clicks;
	nwcl=cl-wcl;
}

QDateTime evaluator::get_date()
{
	return QDateTime(QDate(year,month,day),QTime(hour,minute,second));
}

