#ifndef _FS_FORMULAID_H
#define _FS_FORMULAID_H

#include "freesweeper.h"

enum formula_id
{
	f_none=-1,
	f_int=-2,

	// operators
	f_plus=256,
	f_minus=257,
	f_times=258,
	f_div=259,
	f_power=260,
	f_unary_minus=261,

	f_sqr=262,
	f_log=263,
	f_exp=264,

	f_rank=265,

	//
	f_less=266,
	f_greater=267,
	f_equal=268,
	
	f_or=269,
	f_and=270,
	

	// counters that can be shown while playing
	f_time=0,
	f_path=1,
	f_flags=2,
	f_unflags=3,
	// board properties
	f_mode=4,
	f_level=5,
	f_width=6,
	f_height=7,
	f_mines=8,
	// click rate
	f_l_cl=9,
	f_l_cl_s=10,
	f_r_cl=11,
	f_r_cl_s=12,
	f_d_cl=13,
	f_d_cl_s=14,
	f_cl=15,
	f_cl_s=16,
	f_cl_15=17,
	f_cl_15_s=18,
	// shown numbers
	f_sh_numbers=19,
	f_sh_ones=20,
	f_sh_twos=21,
	f_sh_threes=22,
	f_sh_fours=23,
	f_sh_fives=24,
	f_sh_sixes=25,
	f_sh_sevens=26,
	f_sh_eights=27,
	f_sh_squares=28,
	// history
	f_games_finished=29,
	
	f_number_of_boards=30,
	
	// secret counters, they should be >=f_est
	f_est=31,
	// benchmarks
	f_3bv=32,
	f_obv=33,
	f_zini=34,
	f_hzini=35,
	// speed
	f_3bv_s=36,
	f_obv_s=37,
	f_zini_s=38,
	f_hzini_s=39,
	//
	f_solved_3bv=40,
	// magic indices	
	f_rqp=41,
	f_ios=42,
	// efficiency
	f_ioe=43,
	f_corr=44,
	f_thrp=45,
	f_zne=46,
	f_znt=47,
	f_hzne=48,
	f_hznt=49,
	//
	f_bat=50,
	// wasted clicks
	f_w_l_cl=51,
	f_w_r_cl=52,
	f_w_d_cl=53,
	f_w_cl=54,
	f_w_cl_15=55,
	// numbers
	f_numbers=56,
	f_ones=57,
	f_twos=58,
	f_threes=59,
	f_fours=60,
	f_fives=61,
	f_sixes=62,
	f_sevens=63,
	f_eights=64,
	// openings and islands
	f_ops=65,
	f_ops_hit=66,
	f_isls=67,
	f_isls_solved=68,
	// mines config
	f_wall_mines=69,
	f_corner_mines=70,
	// flags
	f_misflags=71,
	f_misunflags=72,
	f_wasted_flags=73,
	
	// ranks
	f_rank_time=74,
	f_rank_3bv_s=75,
	f_rank_obv_s=76,
	f_rank_zini_s=77,
	f_rank_hzini_s=78,
	f_rank_ioe=79,
	f_rank_thrp=80,
	f_rank_corr=81,
};
const int n_named_counters=82;
const QString counter_name[n_named_counters]={"Time","Path","Flags","Unflags",
	"Mode","Level","Width","Height","Mines",
	"LCl","LCl\\s","RCl","RCl\\s","DCl","DCl\\s","Cl","Cl\\s","OneDotFiveCl","OneDotFiveCl\\s",
	"ShownNumbers","ShownOnes","ShownTwos","ShownThrees","ShownFours",
		"ShownFives","ShownSixes","ShownSevens","ShownEights","OpenSquares",
	"TotalGames",
	"NumberOfBoards",
	"Estimate","BBBV","OBV","ZiNi","HZiNi",
	"BBBV\\s","OBV\\s","ZiNi\\s","HZiNi\\s",
	"SolvedBBBV",
	"RQP","IOS",
	"IOE","Correctness","ThroughPut","ZNE","ZNT","HZNE","HZNT",
	"BAT",
	"WastedLCl","WastedRCl","WastedDCl","WastedCl","WastedOneDotFiveCl",
	"Numbers","Ones","Twos","Threes","Fours","Fives","Sixes","Sevens","Eights",
	"Openings","HitOpenings","Islands","SolvedIslands",
	"MinesOnWalls","MinesInCorners",
	"Misflags","Misunflags","WastedFlags",
	"RankTime","RankBBBV\\s","RankOBV\\s","RankZiNi\\s","RankHZiNi\\s","RankIOE","RankThroughPut","RankCorrectness"
	};


enum c_formula_type
{
	cft_normal=0,
	cft_day=1,
	cft_day_2=2,
	cft_month_long=3,
	cft_month_short=4,
	cft_month_2=5,
	cft_year_2=6,
	cft_year_4=7,
	cft_hour=8,
	cft_minute=9,
	cft_second=10,
	cft_level_name_3=11,
	cft_level_name=12,
	cft_Level_name_3=13,
	cft_Level_name=14
};
const int n_c_formula_types=14;
const QString c_formula_desc[n_c_formula_types+1]={"","day","dd","month","mon","mm","yy","yyyy","hh","mi","ss",
	"lvl","level","biglvl","biglevel"};

const int n_constants=16;
const QString constants_id[n_constants]=
{"beginner","beg","intermediate","int","expert","exp","megaexpert","meg","custom","cus",
	"classic","lucky","density","upk","cheat","edit"};
const int constants[n_constants]={l_beginner,l_beginner,l_intermediate,l_intermediate,l_expert,l_expert,
l_megaexpert,l_megaexpert,l_custom,l_custom,classic,lucky,density,upk,cheat,edit};

enum named_ranks
{
	nr_time=0,nr_bbbvs=1,nr_obvs=2,nr_zinis=3,nr_hzinis=4,nr_ioe=5,nr_thrp=6,nr_corr=7
};
const int n_named_ranks=8;
const QString rank_strings[n_named_ranks]={counter_name[f_time],counter_name[f_3bv_s],
	counter_name[f_obv_s],counter_name[f_zini_s],counter_name[f_hzini_s],
	counter_name[f_ioe],counter_name[f_thrp],counter_name[f_corr]};

#endif

