#include "game.h"
#include "settings-names.h"

const QString E_open_settings="Couldn't open the settings file!";

const QString F_default_video_file_name="default.rawvf";
const QString F_secure_extension=".fsvf";
const int F_secure_ext_l=F_secure_extension.size();
const QString F_default_secure_video_file_name="default"+F_secure_extension;
const QString F_settings_fo="freesweeper.settings";

game::game():VideoControl(0),history_program(0),
red_mines_state(false),smiley_restarts(false),timer_video_state(false),counters_state(false),
time_limits_enabled(false)
{
	loaded=subdialogs=false;
	F_settings=settings_dir+F_settings_fo;
	
	settings s(F_settings);
	s.defaultsettings();	// to prevent troubles caused by a corrupted settings file
	s.read(E_open_settings);

	Translation=new translation;
	Translation->load(s.get(sTranslation));

	Skin=new skin(Translation);
	Skin->load(""/*s.get(sSkin)*/);
	if(!Skin->loaded) Skin->load_default();

	Timer=new timer;

	// create smiley
	Smiley=new smiley(Skin,(gamemode)(s.getint(sMode)));
	Smiley->setParent(this);

	// create red displays
	RedTimer=new display(Skin);
	RedTimer->setParent(this);
	RedTimer->switch_tenth(s.getint(sTimerWithTenths));
	RedMines=new display(Skin);
	RedMines->setParent(this);
	
	// create the board
	Board=new board(Translation,Skin,Timer);
	Board->setParent(this);
	init_board_options(s);
	
	// create the label showing the player id
	PlayerID=new labelid(Translation,Skin);
	PlayerID->setParent(this);
	change_player(s.get(sPlayer));
	
	// create and set up scroll bars
	ScrX=new QScrollBar(Qt::Horizontal,this);
	ScrX->setMinimum(0);
	ScrX->setSingleStep(1);ScrX->setPageStep(Skin->max_width-1);ScrX->setValue(0);

	ScrY=new QScrollBar(Qt::Vertical,this);
	ScrY->setMinimum(0);
	ScrY->setSingleStep(1);ScrY->setPageStep(Skin->max_height-1);ScrY->setValue(0);

	init_general_options(s);

	Parser=new parser(Translation,video_dir);
	
	Eval=new evaluator(&(Board->D),&(Smiley->mode));
	Eval->show_stats=s.getint(sShowCounters);

	Counters=new counters_window(Translation,Parser,Eval,Skin);
	Counters->locked=s.getint(sCountersLocked);
	Counters->attached=s.getint(sCountersAttached);
	counters_shown=s.getint(sShowCountersWindow);
	Counters->edit_standard=s.getint(sEnableStandardCountersEditing);
	Counters->load();
	
	Autosaver=new autosaver(Translation,Skin,Parser,Eval);
	Autosaver->load();
	
	IRCModule=new ircmodule(Translation,Parser,Eval);
	IRCModule->load();
	
	History=new history(Translation,Parser,Eval);
	History->dens_threshold=s.getdouble(sDensityThreshold);
	Eval->History=History;
	
	// make things work
	init_menu();

	set_level(s.getint(sLevel));
	reset_displays();
/*	m_irc_copy_last->setChecked(s.getint(sIRCCopyLast));
	m_irc_copy_blast->setChecked(s.getint(sIRCCopyBlast));*/ //{R1}

	if( (Counters->separated=s.getint(sCountersSeparated)) )
		separate_counters();
	else
		dock_counters();

	place_things();
	connect_things();
	if(Board->player=="") PlayerID->get_name();
	Smiley->set_mode(Smiley->mode);
	
	// show the main window : disable the maximizing button, move it, allow drag&drop 
	Qt::WindowFlags f=windowFlags();
	f&=~Qt::WindowMaximizeButtonHint;
	f|=Qt::MSWindowsFixedSizeDialogHint;
	if(s.getint(sStayOnTop))
		f|=Qt::WindowStaysOnTopHint;
	else
		f&=~Qt::WindowStaysOnTopHint;
	setWindowFlags(f);
	int x=s.getint(sMainWindowX);
	int y=s.getint(sMainWindowY)+(y_fix=s.getint(sWindowsYFix));
	if(x<0) x=0; if(y<0) y=0;
	move(x,y);
	setAcceptDrops(true);
	setWindowIcon(Skin->program_icon);
	loaded=true;
}

game::~game()
{
	F_settings=settings_dir+F_settings_fo;
	settings s(F_settings);

	s.set(sSkin,Skin->skin_path);
	s.set(sTranslation,Translation->path);
	s.set(sMode,Smiley->mode);
	s.set(sLevel,level);
	
	// window settings
	if(VideoControl) hide_video_control();
	s.set(sMainWindowX,x());
	s.set(sMainWindowY,y());
	s.set(sWindowsYFix,y_fix);
	s.set(sStayOnTop,windowFlags()&Qt::WindowStaysOnTopHint?"1":"0");

	s.set(sVideoControlWindowX,x_vid);
	s.set(sVideoControlWindowY,y_vid);
	s.set(sHiScoresWindowX,x_hs);
	s.set(sHiScoresWindowY,y_hs);
	s.set(sAHiScoresWindowX,x_ahs);
	s.set(sAHiScoresWindowY,y_ahs);
	s.set(sOptionsWindowX,x_ow);
	s.set(sOptionsWindowY,y_ow);
	
	// game settings
	s.set(sPlayer,Board->player);
	s.set(sTimerWithTenths,RedTimer->has_tenth);
	s.set(sHideTimer,hide_timer);
	s.set(sNullMinesCounter,null_mines_counter);
	s.set(sWideSmileyArea,wide_smiley_area);
	
	s.set(sRestartByTimeLimit,restart_by_time);
	s.set(sRestartBegLimit,restart_beg);
	s.set(sRestartIntLimit,restart_int);
	s.set(sRestartExpLimit,restart_exp);
	
	s.set(sBeepByTime,beep);
	s.set(sBeepBeg,beep_beg);
	s.set(sBeepInt,beep_int);
	s.set(sBeepExp,beep_exp);
	
	s.set(sDateTemplate,date_template);

	for(int i=l_beginner;i<=n_stand_levels;++i)
	{
		s.set(sMarathonLength+sLevel+QString::number(i),marathon_length[i]);
		s.set(sTimeTrial+sLevel+QString::number(i),time_trial_length[i]);
	}

	// video settings
	s.set(sVideoMousePath,Board->v_mouse_path);
	s.set(sVideoOpenings,Board->v_openings);
	s.set(sVideoShowBoard,Board->v_show_board);

	// board settings
	s.set(sBoardWidth,Board->D.width);
	s.set(sBoardHeight,Board->D.height);
	s.set(sBoardMines,Board->D.n_mines);

	s.set(sWheelAsChord,Board->middle_chord);
	s.set(sRevealBoardAfterBlast,Board->reveal_after_end);
	s.set(sForceNF,Board->nf);
	s.set(sFlagMinesAfterWin,Board->flag_mines_after_win);
	s.set(sShowWastedFlagsAfterEnd,Board->show_wasted_flags_after_end);
	s.set(sClearBoardThreshold,Board->clear_board_threshold);
	
	s.set(sQuestionMarks,Board->qmarks);
	
	s.set(sLuckLibrary,Board->luck_library);
	s.set(sLuckSolver,Board->luck_solver);

	s.set(sCustomWidth,custom_width);
	s.set(sCustomHeight,custom_height);
	s.set(sCustomMines,custom_mines);
	
	s.set(sCustomMarathonLength,custom_marathon_description.size());
	for(int i=0;i<custom_marathon_description.size();++i)
	{
		s.set(sCustomMarathonBoard+QString::number(i+1)+sWidth,custom_marathon_description[i].width);
		s.set(sCustomMarathonBoard+QString::number(i+1)+sHeight,custom_marathon_description[i].height);
		s.set(sCustomMarathonBoard+QString::number(i+1)+sMines,custom_marathon_description[i].mines);
	}

	s.set(sLives,Board->lives);
	s.set(sSuperClick,Board->superclick);
	s.set(sSuperFlag,Board->superflag);
	s.set(sElmarTechnique,Board->elmar);
	s.set(sNonoMouse,Board->nono);
	s.set(sAutoFlag,Board->autoflag);
	s.set(sShowProbabilities,Board->probabilities);
	s.set(sShowBoard,Board->show_board);
	s.set(sLawnmower,Board->lawnmower);
	s.set(sDemandSevenOnBoard,Board->force_seven);
	s.set(sDemandEightOnBoard,Board->force_eight);
	
	s.set(sGenerationIterations,cheat_generation_iterations);
	s.set(sRestrictBBBV,bbbv_limit_enabled);
	s.set(sRestrictZiNi,zini_limit_enabled);
	s.set(sRestrictHZiNi,hzini_limit_enabled);
	s.set(sBBBVLowerLimit,cheat_bbbv_lower_limit);
	s.set(sBBBVUpperLimit,cheat_bbbv_upper_limit);
	s.set(sZiNiLowerLimit,cheat_zini_lower_limit);
	s.set(sZiNiUpperLimit,cheat_zini_upper_limit);
	s.set(sHZiNiLowerLimit,cheat_hzini_lower_limit);
	s.set(sHZiNiUpperLimit,cheat_hzini_upper_limit);
	
	s.set(sProbabilitiesIterations,Board->n_probabilities_iterations);
	s.set(sLuckIterations,Board->n_luck_iterations);
	
	s.set(sCloneEvents,Board->clone_events_enabled);
	s.set(sCloneHesitationTime,Board->ce_hesitation_time);
	
/*	s.set(sIRCCopyLast,m_irc_copy_last->isChecked());
	s.set(sIRCCopyBlast,m_irc_copy_blast->isChecked());*/ //{R1}

	s.set(sVideoDir,video_dir);
	s.set(sBoardDir,board_dir);

	s.set(sShowCountersWindow,counters_shown);
	s.set(sCountersLocked,Counters->locked);
	s.set(sCountersAttached,Counters->attached);
	s.set(sCountersSeparated,Counters->separated);
	if(Counters->separated)
	{
		s.set(sCountersX,Counters->QWidget::x());
		s.set(sCountersY,Counters->QWidget::y()+y_fix);
	}
	else
	{
		s.set(sCountersX,x_ctr);
		s.set(sCountersY,y_ctr);
	}
	Counters->save();
	
	s.set(sShowCounters,Eval->show_stats);

	Autosaver->save();

	IRCModule->save();
	
	History->save_ranks();
	s.set(sDensityThreshold,History->dens_threshold);

	s.set(sRecordsOnAHSWindow,records_on_ahs);
	s.set(sGoToEventOffset,go_to_event_offset);
	
	s.write(E_open_settings);
	
	delete Translation;
	delete Skin;
	delete Timer;
	delete Parser;
	delete Eval;
	delete Autosaver;
	delete IRCModule;
	delete History;
}

void game::init_menu()
{
	translation* T=Translation;
	MenuBar=new QMenuBar(this);

	m_game=MenuBar->addMenu(T->str[ins_game]);
	m_options=MenuBar->addMenu(T->str[ins_options]);
//	m_irc=MenuBar->addMenu(T->str[ins_irc]); //{R1}
	m_help=MenuBar->addMenu(T->str[ins_about]);

	QString mLevels[]={"",T->str[ins_beg],T->str[ins_int],T->str[ins_exp],T->str[ins_mega],T->str[ins_cus],
		QString::number(marathon_length[l_beginner])+" x "+T->str[ins_beg],
		QString::number(marathon_length[l_intermediate])+" x "+T->str[ins_int],
		QString::number(marathon_length[l_expert])+" x "+T->str[ins_exp],
		T->str[ins_tt],T->str[ins_m_cus]};
	
	m_new=m_game->addAction(T->str[ins_new],this,SLOT(new_game()),Qt::Key_F2);
/*	m_restart=m_game->addAction(T->str[ins_restart],this,SLOT(restart()),Qt::Key_F3);
	m_restart_from_blast=m_game->addAction(T->str[ins_restart_blast],this,SLOT(restart_from_blast()),Qt::Key_F4); 
	m_restart_from_blast->setEnabled(false);*/ //{R1} 
	m_game->addSeparator();
/*	for(int i=l_beginner;i<=n_levels;++i)
		m_level[i]=m_game->addAction(mLevels[i],&levels_mapper,SLOT(map()));*/ //{R1}
	for(int i=l_beginner;i<=n_stand_levels;++i)
	{
		m_level[i]=m_game->addAction(mLevels[i],&levels_mapper,SLOT(map()));
		m_level[i]->setCheckable(true);
		levels_mapper.setMapping(m_level[i],i);
	}
	m_game->addSeparator();
/*	m_marathons=m_game->addMenu(T->str[ins_marathons]);
	m_new_marathon=m_marathons->addAction(T->str[ins_new_mar],this,SLOT(new_marathon()),
			(int)Qt::ControlModifier | Qt::Key_F2);
	m_marathons->addSeparator();
	for(int i=n_levels+1;i<=n_game_levels;++i)
		m_level[i]=m_marathons->addAction(mLevels[i],&levels_mapper,SLOT(map()));
	for(int i=1;i<=n_game_levels;++i)
	{
		m_level[i]->setCheckable(true);
		levels_mapper.setMapping(m_level[i],i);
	}*/ //{R1}

	m_game->addSeparator();
	m_preview_video=m_game->addAction(T->str[ins_preview_vid],this,SLOT(preview_video()),
			(int) Qt::ControlModifier | Qt::Key_P);
	m_read_video=m_game->addAction(T->str[ins_load_vid],this,SLOT(load_video()),
			(int) Qt::ControlModifier | Qt::Key_L);
	m_write_video=m_game->addAction(T->str[ins_save_vid],this,SLOT(save_video()),
			(int) Qt::ControlModifier | Qt::Key_S);
	m_game->addSeparator();
/*	m_read_board=m_game->addAction(T->str[ins_load_brd],this,SLOT(load_board()));
	m_write_board=m_game->addAction(T->str[ins_save_brd],this,SLOT(save_board()));
	m_game->addSeparator();*/ //{R1}
	m_show_hs=m_game->addAction(T->str[ins_show_hs],this,SLOT(show_hiscores()),Qt::Key_F8);
	m_show_ahs=m_game->addAction(T->str[ins_show_ahs],this,SLOT(show_adv_hiscores()),
			(int) Qt::ShiftModifier | Qt::Key_F8);
	m_show_hist=m_game->addAction(T->str[ins_show_hist],this,SLOT(show_history()),Qt::Key_F9);
	m_game->addSeparator();
	m_exit_fs=m_game->addAction(T->str[ins_exit],this,SLOT(exit_freesweeper()));

	QString mMode[]={T->str[ins_classic],T->str[ins_luck],T->str[ins_dens],
		T->str[ins_upk],T->str[ins_cheat],T->str[ins_edit]};
/*	m_show_opt=m_options->addAction(T->str[ins_show_options],this,SLOT(show_options()),Qt::Key_F5);
	m_show_cindopt=m_options->addAction(T->str[ins_edit_ctr],Counters,SLOT(ctr_manager()));
	m_show_asopt=m_options->addAction(T->str[ins_edit_as],Autosaver,SLOT(autosave_manager()));
	m_show_skinopt=m_options->addAction(T->str[ins_select_skin],this,SLOT(select_skin()));
	m_edit_m_custom=m_options->addAction(T->str[ins_edit_cm],this,SLOT(show_custom_marathon_options()));
	m_options->addSeparator();*/ //{R1}
	for(int i=0;i<n_enabled_modes;++i) 
	{
		m_mode[i]=m_options->addAction(mMode[i],&modes_mapper,SLOT(map()));
		m_mode[i]->setCheckable(true);
		modes_mapper.setMapping(m_mode[i],i);
	}
	// {R1}
	m_options->addSeparator();
	QAction* tmp;
	tmp=m_ll=m_options->addAction(T->str[ins_luck_lib]);
	connect(tmp,SIGNAL(triggered(bool)),this,SLOT(toggle_luck_library(bool)));
	tmp->setCheckable(true);
	tmp->setChecked(Board->luck_library);
	tmp=m_ls=m_options->addAction(T->str[ins_luck_solv]);
	connect(tmp,SIGNAL(triggered(bool)),this,SLOT(toggle_luck_solver(bool)));
	tmp->setCheckable(true);
	tmp->setChecked(Board->luck_solver);
	
/*	m_irc_last=m_irc->addAction(T->str[ins_info_fin],this,SLOT(irc_copy_last_finish()));
	m_irc_blast=m_irc->addAction(T->str[ins_info_blast],this,SLOT(irc_copy_last_blast()));
	m_irc_marathon=m_irc->addAction(T->str[ins_info_mar],this,SLOT(irc_copy_marathon()));
	m_irc->addSeparator();
	m_irc_hist_rec=m_irc->addAction(T->str[ins_info_rec],&irc_history_mapper,SLOT(map()));
	irc_history_mapper.setMapping(m_irc_hist_rec,"r");
	m_irc_hist_avg=m_irc->addAction(T->str[ins_info_avg],&irc_history_mapper,SLOT(map()));
	irc_history_mapper.setMapping(m_irc_hist_avg,"a");
	m_irc_hist_ar=m_irc->addAction(T->str[ins_info_ar],&irc_history_mapper,SLOT(map()));
	irc_history_mapper.setMapping(m_irc_hist_ar,"A");
	m_irc_hist_today_beg=m_irc->addAction(T->str[ins_info_beg],&irc_history_mapper,SLOT(map()));
	irc_history_mapper.setMapping(m_irc_hist_rec,"b");
	m_irc_hist_today_int=m_irc->addAction(T->str[ins_info_int],&irc_history_mapper,SLOT(map()));
	irc_history_mapper.setMapping(m_irc_hist_rec,"i");
	m_irc_hist_today_exp=m_irc->addAction(T->str[ins_info_exp],&irc_history_mapper,SLOT(map()));
	irc_history_mapper.setMapping(m_irc_hist_rec,"e");
	m_irc->addSeparator();
	m_irc_edit=m_irc->addMenu(T->str[ins_edit_irc]);
	const QString mIRCEdit[]={T->str[ins_edit_fin],T->str[ins_edit_dens],
		T->str[ins_edit_cheat],T->str[ins_edit_video],
		T->str[ins_edit_blast],T->str[ins_edit_dens_b],
		T->str[ins_edit_cheat_b],T->str[ins_edit_video_b],
		T->str[ins_edit_mar]};
	for(int i=0;i<n_irc_strings;++i)
	{
		m_irc_edit_template[i]=m_irc_edit->addAction(mIRCEdit[i],&irc_edit_mapper,SLOT(map()));
		irc_edit_mapper.setMapping(m_irc_edit_template[i],i);
	}
	m_irc->addSeparator();
	m_irc_copy_last=m_irc->addAction(T->str[ins_autocopy_fin]);
	m_irc_copy_blast=m_irc->addAction(T->str[ins_autocopy_blast]);
	m_irc_copy_last->setCheckable(true);
	m_irc_copy_blast->setCheckable(true);*/ //{R1}
	
	m_help->addAction(T->str[ins_about_fs],this,SLOT(show_about_window()));
	m_help->addAction(T->str[ins_about_qt],qApp,SLOT(aboutQt()));

	MenuBar->show();	// to get its real height in place_things()
}

void game::toggle_luck_library(bool enable)
{
	Board->luck_library=enable;
}
void game::toggle_luck_solver(bool enable)
{
	Board->luck_solver=enable;
}

void game::init_board_options(settings& s)
{
	// general settings
	Board->middle_chord=s.getint(sWheelAsChord);
	Board->reveal_after_end=s.getint(sRevealBoardAfterBlast);
	Board->nf=s.getint(sForceNF);
	Board->flag_mines_after_win=s.getint(sFlagMinesAfterWin);
	Board->show_wasted_flags_after_end=s.getint(sShowWastedFlagsAfterEnd);
	Board->clear_board_threshold=s.getint(sClearBoardThreshold);
	
	Board->qmarks=s.getint(sQuestionMarks);
	
	// luck options
	Board->luck_library=s.getint(sLuckLibrary);
	Board->luck_solver=s.getint(sLuckSolver);
	
	// density options
	custom_width=s.getint(sCustomWidth);
	custom_height=s.getint(sCustomHeight);
	custom_mines=s.getint(sCustomMines);
	Board->correct_board(custom_width,custom_height,custom_mines);

	int cm=s.getint(sCustomMarathonLength);
	cm=(cm>=0)?cm:0;
	for(int i=0;i<cm;++i)
	{
		int w=s.getint(sCustomMarathonBoard+QString::number(i+1)+sWidth);
		int h=s.getint(sCustomMarathonBoard+QString::number(i+1)+sHeight);
		int n=s.getint(sCustomMarathonBoard+QString::number(i+1)+sMines);
		Board->correct_board(w,h,n);
		custom_marathon_description << board_description(w,h,n);
	}

	for(int i=0;i<=n_stand_levels;++i)
	{
		marathon_length[i]=s.getint(sMarathonLength+sLevel+QString::number(i));
		time_trial_length[i]=s.getint(sTimeTrial+sLevel+QString::number(i));
	}
	
	// cheat options
	Board->lives=s.getint(sLives);

	Board->superclick=s.getint(sSuperClick);
	Board->superflag=s.getint(sSuperFlag);
	Board->elmar=s.getint(sElmarTechnique);
	Board->nono=s.getint(sNonoMouse);
	Board->autoflag=s.getint(sAutoFlag);
	Board->probabilities=s.getint(sShowProbabilities);
	Board->show_board=s.getint(sShowBoard);
	Board->lawnmower=s.getint(sLawnmower);
	Board->force_seven=s.getint(sDemandSevenOnBoard);
	Board->force_eight=s.getint(sDemandEightOnBoard);
	
	cheat_generation_iterations=s.getint(sGenerationIterations);
	bbbv_limit_enabled=s.getint(sRestrictBBBV);
	zini_limit_enabled=s.getint(sRestrictZiNi);
	hzini_limit_enabled=s.getint(sRestrictHZiNi);
	cheat_bbbv_lower_limit=s.getint(sBBBVLowerLimit);
	cheat_bbbv_upper_limit=s.getint(sBBBVLowerLimit);
	cheat_zini_lower_limit=s.getint(sZiNiLowerLimit);
	cheat_zini_upper_limit=s.getint(sZiNiUpperLimit);
	cheat_hzini_lower_limit=s.getint(sHZiNiLowerLimit);
	cheat_hzini_upper_limit=s.getint(sHZiNiUpperLimit);
	
	Board->n_probabilities_iterations=s.getint(sProbabilitiesIterations);
	Board->n_luck_iterations=s.getint(sLuckIterations);
		
	Board->clone_events_enabled=s.getint(sCloneEvents);
	Board->ce_hesitation_time=s.getint(sCloneHesitationTime);
	
	Board->v_mouse_path=s.getint(sVideoMousePath);
	Board->v_openings=s.getint(sVideoOpenings);
	Board->v_show_board=s.getint(sVideoShowBoard);
}

void game::init_general_options(settings& s)
{
	hide_timer=s.getint(sHideTimer);
	null_mines_counter=s.getint(sNullMinesCounter);
	wide_smiley_area=s.getint(sWideSmileyArea);
	
	restart_by_time=s.getint(sRestartByTimeLimit);
	restart_beg=s.getint(sRestartBegLimit);
	restart_int=s.getint(sRestartIntLimit);
	restart_exp=s.getint(sRestartExpLimit);
	
	beep=s.getint(sBeepByTime);
	beep_beg=s.getint(sBeepBeg);
	beep_int=s.getint(sBeepInt);
	beep_exp=s.getint(sBeepExp);

	if(restart_by_time || beep) enable_time_limits(); else disable_time_limits();
	
	x_hs=s.getint(sHiScoresWindowX);
	y_hs=s.getint(sHiScoresWindowY);
	x_ahs=s.getint(sAHiScoresWindowX);
	y_ahs=s.getint(sAHiScoresWindowY);
	x_ctr=s.getint(sCountersWindowX);
	y_ctr=s.getint(sCountersWindowY);
	x_vid=s.getint(sVideoControlWindowX);
	y_vid=s.getint(sVideoControlWindowY);
	x_ow=s.getint(sOptionsWindowX);
	y_ow=s.getint(sOptionsWindowY);
	
	date_template=s.get(sDateTemplate);
	
	video_dir=s.get(sVideoDir);
	board_dir=s.get(sBoardDir);
	if(video_dir=="" || !QDir(video_dir).exists()) video_dir=normal_settings_dir;
	if(board_dir=="" || !QDir(board_dir).exists()) board_dir=normal_settings_dir;
	
	records_on_ahs=s.getint(sRecordsOnAHSWindow);
	go_to_event_offset=s.getint(sGoToEventOffset);
}

void game::init_level_actions()
{
/*	for(int i=1;i<=n_game_levels;++i) m_level[i]->setChecked(i==level);*/ //{R1}
	for(int i=1;i<=n_stand_levels;++i) m_level[i]->setChecked(i==level);//{R1}
}

void game::set_level(int l)
{
	if(l<l_beginner || l>n_game_levels || (l==l_m_custom && !custom_marathon_description.size())) l=l_beginner; 
	if(l>n_stand_levels) l=l_beginner;//{R1}
	level=l;
	init_level_actions();
	if(l>n_levels)	// marathon
	{
//		m_new_marathon->setEnabled(true);//{R1}
		Board->marathon=true;
		Board->n_finished_boards=0;
		Board->marathon_description.clear();
		if(l<l_m_beginner+n_stand_levels)	// a standard marathon
		{
			l=l-l_custom;
			for(int i=0;i<marathon_length[l];++i)
				Board->marathon_description
				 << board_description(level_width[l],level_height[l],level_mines[l]);
		}
		else if(l==l_m_time_trial)
		{
			for(int i=1;i<=n_stand_levels;++i)
				for(int j=0;j<time_trial_length[i];++j)
					Board->marathon_description
					 << board_description(level_width[i],level_height[i],level_mines[i]);
		}
		else if(l==l_m_custom)
		{
			Board->marathon_description = custom_marathon_description; // QVector::operator=
		}
		Board->D.n_boards=Board->marathon_description.size();
	}
	else
	{
		Board->marathon=false;
		//m_new_marathon->setEnabled(false);//{R1}
		Smiley->allowed_set_mode=true;
		if(l==l_custom)
			Board->resize_if_needed(custom_width,custom_height,custom_mines);
		else
			Board->resize_if_needed(level_width[l],level_height[l],(Board->editing)?0:level_mines[l]);
		Board->D.n_boards=1;
	}
	new_game();
}
int game::check_level()
{
	bool b;
	level=l_no_level;
	m_ll->setChecked(Board->luck_library);
	m_ls->setChecked(Board->luck_solver);
	if(!Board->marathon)
		level=Board->D.level;
	else
	{
		for(int i=1;i<=n_stand_levels;++i)
			if(Board->marathon_description.size()==marathon_length[i])
			{
				b=true;
				for(int j=0;j<marathon_length[i];++j)
					b=b&&(Board->marathon_description[j].width==level_width[i] &&
						Board->marathon_description[j].height==level_height[i] &&
						Board->marathon_description[j].mines==level_mines[i]);
				if(b)
				{
					level=l_m_beginner-1+i;
					break;
				}
			}
		if(level==l_no_level)
		{
			b=true;
			for(int i=1,k=0;i<=n_stand_levels;++i)
				for(int j=0;j<time_trial_length[i];++j,++k)
				{
					b=b&&(Board->marathon_description.size()>k &&
						Board->marathon_description[k].width==level_width[i] &&
						Board->marathon_description[k].height==level_height[i] &&
						Board->marathon_description[k].mines==level_mines[i]);
				}
			level=b?l_m_time_trial:l_m_custom;
		}
	}
	init_level_actions();
	return level;
}

void game::changed_state(boardstate state)
{
	cout<<"state "<<state<<endl;
	switch(state)
	{
	case before_game:
		beepped=false;
		hide_video_control();
		toggle_video_actions(false);
/*		m_write_board->setEnabled(Board->editing);
		m_restart->setEnabled(Board->editing);
		m_restart_from_blast->setEnabled(false);*/ //{R1}
		enable_id_changes();
		disconnect_counters();
		if(!Board->marathon || !Board->n_finished_boards) Timer->set_time(0);
		RedTimer->switch_display(true);
//		m_irc_marathon->setEnabled(false); //{R1}
		Smiley->allowed_set_mode=true;
		break;
	case generating:
		break;
	case playing:
		if(!Board->marathon || !Board->n_finished_boards) 
			Timer->start();
		else // continue marathon
			Timer->restart();
		connect_counters();
		toggle_video_actions(false);
/*		m_write_board->setEnabled(true);
		m_restart->setEnabled(!Board->marathon);
		m_restart_from_blast->setEnabled(false);*/ //{R1}
		disable_id_changes();
		if(hide_timer) RedTimer->switch_display(false);
//		m_irc_marathon->setEnabled(false); //{R1}
		break;
	case win:
		if(!video)
		{
			Timer->stop(Board->video_duration);
			game_won();
		}
		else
		{
			Timer->stop(Board->last_processed_event);
			/*if(m_irc_copy_last->isChecked())*/ irc_copy_last_finish();//{R1}
		}
		toggle_video_actions(!Board->marathon || !Board->n_finished_boards);
/*		m_write_board->setEnabled(true);
		m_restart->setEnabled(!Board->marathon);
		m_restart_from_blast->setEnabled(false);*/ //{R1}
		disable_id_changes();
		RedTimer->switch_display(true);
		if(null_mines_counter) 
		{
			if(Board->marathon)
				RedMines->show(Board->remaining_mines());
			else
				RedMines->show(0);
		}
		if(Board->marathon) 
		{
			if(Board->n_finished_boards)	// unfinished marathon
			{
				disconnect_counters();
				Timer->restart();
				Smiley->allowed_set_mode=false;
//				m_irc_marathon->setEnabled(false); //{R1}
			}
			else				// finished marathon
			{
//				m_irc_marathon->setEnabled(true); //{R1}
				Smiley->allowed_set_mode=true;
			}
		}
		break;
	case fail:
		if(!video)
		{
			Timer->stop(Board->video_duration);
			game_lost();
		}
		else 
		{
			Timer->stop(Board->last_processed_event);
			/*if(m_irc_copy_blast->isChecked())*/ irc_copy_last_blast();//{R1}
		}
		toggle_video_actions(!Board->marathon);
/*		m_write_board->setEnabled(true);
		m_restart->setEnabled(!Board->marathon);
		m_restart_from_blast->setEnabled(!Board->marathon);*/ //{R1}
		disable_id_changes();
		RedTimer->switch_display(true);
		if(Board->marathon && Board->n_finished_boards)	// marathon continues
		{
			disconnect_counters();
			Timer->restart();
		}
//		m_irc_marathon->setEnabled(false); //{R1}
		break;
	case playing_video:
		video=true;
		reset_displays();	// new video
		connect_counters();
		connect_timer_to_video();
		if(Board->marathon && Board->n_viewed_boards)
			Timer->restart();
		else
			Timer->start(VideoControl ? VideoControl->get_speed() : 100);
		m_preview_video->setEnabled(false);
		m_write_video->setEnabled(true);
/*		m_write_board->setEnabled(true);
		m_restart->setEnabled(true);
		m_restart_from_blast->setEnabled(false);
		m_irc_marathon->setEnabled(false);*/ //{R1}
		disable_id_changes();
		break;
	case stopped_video:
		disconnect_timer_from_video();
		Timer->stop();
		if(Timer->get_time()>Board->video_duration) Timer->set_time(Board->video_duration);
		m_preview_video->setEnabled(false);
		m_write_video->setEnabled(true);
/*		m_write_board->setEnabled(true);
		m_restart->setEnabled(true);
		m_restart_from_blast->setEnabled(false);
		m_irc_marathon->setEnabled(false);*/ //{R1}
		disable_id_changes();
		break;
	}
}
void game::toggle_video_actions(bool enable)
{
	m_preview_video->setEnabled(enable);
	m_write_video->setEnabled(enable);
}

void game::game_won()
{
	QString popup;
	QString name=PlayerID->text;
	translation* T=Translation;
	if(Smiley->is_legal_mode()) 
		popup=Counters->check_records(Board->D.level,Board->D.r_clicks || Board->D.d_clicks);
	if(!Board->marathon) Autosaver->apply();
	/*if(m_irc_copy_last->isChecked())*/ irc_copy_last_finish();//{R1}
	if(popup!="") 
	{
		popup=popup.right(popup.length()-1); // there was a leading comma
		if(Autosaver->popup=="")
			popup=T->str[ins_have_new] + popup + " " + T->str[ins_hi_score] + "\n" + 
				T->str[ins_request_name];
		else
			popup=T->str[ins_have_new] + popup + " " + T->str[ins_hi_score] + "\n" + 
				Autosaver->popup + "\n" + T->str[ins_request_name];
		name=retrieve_name(popup);
	}
	else if(Autosaver->popup!="")
	{
		msg(Autosaver->popup);
	}
	if(Smiley->is_legal_mode())
	{
		Counters->set_record_names(name,Board->D.level);
		History->add(&(Board->D),Smiley->mode,Timer->get_time_since_stop());
		Counters->refresh(Timer->get_time_since_stop());	// to show ranks
		if(popup!="")
		{
			if(Counters->only_time_record) show_hiscores(); else show_adv_hiscores();
		}
	}
	else if(Smiley->mode==density)
		History->add_dens(&(Board->D),Timer->get_time_since_stop(),true);
}
void game::game_lost()
{
	Autosaver->apply();
	/*if(m_irc_copy_blast->isChecked())*/ irc_copy_last_blast();//{R1}
	if(Autosaver->popup!="") msg(Autosaver->popup);
	if(!video && Smiley->mode==density) History->add_dens(&(Board->D),Timer->get_time_since_stop(),false);
}
QString game::retrieve_name(QString popup)
{
	QString	name=QInputDialog::getText(this,Translation->str[ins_grats],popup,QLineEdit::Normal,PlayerID->text);
	if(name=="") name=Translation->str[ins_anon];
	return name;
}


void game::connect_timer_to_video()
{
	if(!timer_video_state) connect(Timer,SIGNAL(tick(int)),Board,SLOT(process_video(int)));
	timer_video_state=true;
}
void game::disconnect_timer_from_video()
{
	if(timer_video_state) disconnect(Timer,SIGNAL(tick(int)),Board,SLOT(process_video(int)));
	timer_video_state=false;
}

void game::changed_mode(gamemode m)
{
	cout<<"mode "<<m<<endl;
	for(int i=0;i<n_enabled_modes;++i) m_mode[i]->setChecked(i==m);
	switch(m)
	{
	case classic:
		Board->luck_allowed=false;
		toggle_marathons(true);
		toggle_custom_marathon(false);
		toggle_upk(false);
		Board->cheat_allowed=false;
		Board->editing=false;
		toggle_custom_boards(false);
		smiley_as_new();
		limits=blt_imc_board_limits;
		Eval->allow_stats=false;
		break;
	case lucky:
		Board->luck_allowed=true;
		toggle_marathons(true);
		toggle_custom_marathon(false);
		toggle_upk(false);
		Board->cheat_allowed=false;
		Board->editing=false;
		toggle_custom_boards(false);
		smiley_as_new();
		limits=blt_imc_board_limits;
		Eval->allow_stats=false;
		break;
	case density:
		Board->luck_allowed=false;
		toggle_marathons(true);
		toggle_custom_marathon(true);
		toggle_upk(false);
		Board->cheat_allowed=false;
		Board->editing=false;
		toggle_custom_boards(true);
		smiley_as_new();
		limits=blt_no_board_limits;
		Eval->allow_stats=false;
		break;
	case upk:
		Board->luck_allowed=false;
		toggle_marathons(false);
		toggle_custom_marathon(false);
		toggle_upk(true);
		Board->cheat_allowed=false;
		Board->editing=false;
		toggle_custom_boards(true);
		smiley_as_restart();
		limits=blt_no_board_limits;
		Eval->allow_stats=true;
		break;
	case cheat:
		Board->luck_allowed=true;
		toggle_marathons(true);
		toggle_custom_marathon(false);
		toggle_upk(true);
		Board->cheat_allowed=true;
		Board->editing=false;
		toggle_custom_boards(true);
		smiley_as_new();
		limits=blt_cheat_board_limits;
		Eval->allow_stats=true;
		break;
	case edit:
		m_write_video->setEnabled(false);		// no edit videos!
		toggle_marathons(false);
		toggle_custom_marathon(false);
		toggle_upk(true);
		Board->editing=true;
		toggle_custom_boards(true);
		smiley_as_new();
		set_level(Board->D.level);
		Board->change_n_mines(0);
		Eval->allow_stats=true;
		break;
	}
	set_limits();
}

void game::toggle_custom_boards(bool allow)
{
	Board->custom_allowed=allow;
//	for(int i=n_stand_levels+1;i<=n_levels;++i) m_level[i]->setEnabled(allow); //{R1}
	if(!allow) 
	{
		set_level((Board->D.level<=n_stand_levels) ? 
				Board->D.level+Board->marathon*n_levels : 
				1+Board->marathon*n_levels);
		downgrade_red_mines();
	}
	else if(!Board->marathon)
		upgrade_red_mines();
	else
		downgrade_red_mines();
			
}
void game::toggle_marathons(bool allow)
{
//	for(int i=n_levels+1;i<n_game_levels;++i) m_level[i]->setEnabled(allow); //{R1}
}
void game::toggle_custom_marathon(bool allow)
{
//	m_level[l_m_custom]->setEnabled(allow && (custom_marathon_description.size())); //{R1}
}
void game::toggle_upk(bool allow)
{
//	m_read_board->setEnabled(allow);//{R1}
	upk_allowed=allow;
}

void game::set_limits()
{
	switch(limits)
	{
	case blt_imc_board_limits:
		apply_imc_board_limits();
		break;
	case blt_no_board_limits:
		no_board_limits();
		break;
	case blt_cheat_board_limits:
		apply_cheat_board_limits();
		break;
	}
}

void game::apply_imc_board_limits()
{
	const int imc_3bv_limits[n_levels+1]={0,2,30,99,0,0};
	const int imc_iterations=1000;
	Board->lower_limit=imc_3bv_limits[Board->D.level];
	Board->upper_limit=0;
	Board->zn_lower_limit=0;
	Board->zn_upper_limit=0;
	Board->hzn_lower_limit=0;
	Board->hzn_upper_limit=0;
	Board->n_iter=imc_iterations;
}
void game::no_board_limits()
{
	Board->lower_limit=0;
	Board->upper_limit=0;
	Board->zn_lower_limit=0;
	Board->zn_upper_limit=0;
	Board->hzn_lower_limit=0;
	Board->hzn_upper_limit=0;
	Board->n_iter=1;
}
void game::apply_cheat_board_limits()
{
	Board->lower_limit=bbbv_limit_enabled?cheat_bbbv_lower_limit:0;
	Board->upper_limit=bbbv_limit_enabled?cheat_bbbv_upper_limit:0;
	Board->zn_lower_limit=zini_limit_enabled?cheat_zini_lower_limit:0;
	Board->zn_upper_limit=zini_limit_enabled?cheat_zini_upper_limit:0;
	Board->hzn_lower_limit=hzini_limit_enabled?cheat_hzini_lower_limit:0;
	Board->hzn_upper_limit=hzini_limit_enabled?cheat_hzini_upper_limit:0;
	Board->n_iter=cheat_generation_iterations;
}

void game::enable_id_changes()
{
	PlayerID->enable_changes=true;
}
void game::disable_id_changes()
{
	PlayerID->enable_changes=false;
}

bool game::is_custom_level()
{
	return Board->D.level>n_stand_levels || Board->D.n_mines!=level_mines[Board->D.level];
}

void game::new_game()
{
	disconnect_timer_from_video();
	if(!Board->marathon || !Board->n_finished_boards) Timer->stop();
	reset_displays();
	if(Smiley->mode==upk) Smiley->set_mode(is_custom_level()?density:classic);
	else Board->new_game();
}
void game::restart()
{
	disconnect_timer_from_video();
	Timer->stop();
	reset_displays();
	if(Board->editing) Board->has_layout=true;
	if(!upk_allowed || Board->editing) Smiley->set_mode(upk);	// and it will restart it
	else Board->restart();
}
void game::restart_from_blast()
{
	if(!upk_allowed) Smiley->set_mode_without_restart(upk);
	Timer->restart();
	Board->restart_from_blast();
}
void game::new_marathon()
{
	Board->n_finished_boards=0;
	Board->new_game();
	Smiley->allowed_set_mode=true;
	Timer->stop();
	reset_displays();
}

void game::display_player(QString player)
{
	PlayerID->set_text(player);
}
void game::change_player(QString player)
{
	Board->player=player;
	PlayerID->set_text(player);
}

void game::preview_video()
{
	Timer->set_time(0);
	reset_displays();
	Board->make_event_list();
	Board->play_video();
	show_video_control();
}
void game::load_video(QString filename)
{
	if(filename=="")
		filename=QFileDialog::getOpenFileName(this,Translation->str[ins_load_vid],video_dir,
			"RawVF (*.rawvf);;AVF (*.avf *.fsvf)");
	if(filename=="") return;
	reset_displays();
	disconnect_timer_from_video();
	Timer->stop();
	Timer->set_time(0);
	try
	{
		if(filename.right(6).toLower()==".rawvf")
			Board->load_rawvf(filename);
		else if(filename.right(4).toLower()==".avf")
			Board->load_avf(filename);
		else if(filename.right(F_secure_ext_l).toLower()==F_secure_extension)
			Board->load_avf(filename);
		else return;
		check_level();
	}
	catch(excvid e)
	{
		err(Translation->str[ins_corr_vid]+" "+e.s);
		new_game();
		return;
	}
	catch(excbrd)
	{
		err(Translation->str[ins_corr_vid_brd]);
		new_game();
		return;
	}

/*	m_new_marathon->setEnabled(Board->marathon);*/ //{R1}
	show_video_control();
}
void game::save_video(QString filename)
{
	if(Board->has_layout)
	{
		if(filename=="")
			filename=QFileDialog::getSaveFileName(this,Translation->str[ins_save_vid],video_dir,
				"RawVF (*.rawvf)");
		if(filename=="") return;
		if(filename.right(6)!=".rawvf") filename+=".rawvf";
		Board->save_rawvf(filename,Smiley->mode);
	}
	else
		err(Translation->str[ins_no_vid]);
}
void game::save_secure_video(QString filename)
{
	if(Board->has_layout)
	{
		if(filename=="")
			filename=QFileDialog::getSaveFileName(this,Translation->str[ins_save_vid],video_dir,
				"Arbiter Video File (*.fsvf)");
		if(filename=="") return;
		if(filename.right(F_secure_ext_l)!=F_secure_extension) filename+=F_secure_extension;
		Board->save_avf(filename,Smiley->mode);
	}
	else
		err(Translation->str[ins_no_vid]);
}
void game::save_rawvf_auto(QString filename)
{
	int cur=0;
	const QString rawvf_ext=".rawvf";
	QString file=filename+rawvf_ext;
	if(QFile::exists(file))
		while(QFile::exists(file=filename+'-'+QString::number(++cur)+rawvf_ext));
	save_video(file);
}
void game::save_secure_auto(QString filename)
{
	int cur=0;
	QString file=filename+F_secure_extension;
	if(QFile::exists(file))
		while(QFile::exists(file=filename+'-'+QString::number(++cur)+F_secure_extension));
	save_secure_video(file);
}

void game::load_board(QString filename)
{
	if(!upk_allowed) return;
	if(filename=="")
		filename=QFileDialog::getOpenFileName(this,Translation->str[ins_load_brd],board_dir,
			"Minesweeper board file (*.mbf *.abf);;RawVF (*.rawvf)");
	if(filename=="") return;
	reset_displays();	// to show the number of mines correctly
	Board->load_board(filename);	
}
void game::save_board(QString filename)
{
	if(Board->has_layout || Board->editing)
	{
		if(filename=="")
			filename=QFileDialog::getSaveFileName(this,Translation->str[ins_save_brd],board_dir,
				"Minesweeper board file (*.mbf)");
		if(filename=="") return;
		if(filename.right(4)!=".mbf") filename+=".mbf";
		Board->save_board(filename);
	}
	else
		err(Translation->str[ins_no_brd]);
}

void game::show_hiscores()
{
	counter* c_time=Counters->counters[Counters->id_time];
	if(!c_time || !c_time->rec) return;

	hiscores_window HS(Translation,Skin);
	connect(&HS,SIGNAL(reset_hiscores()),this,SLOT(reset_time_hiscores()));
	HS.l[l_beginner-1].setText(Translation->str[ins_beg]);
	HS.l[l_intermediate-1].setText(Translation->str[ins_int]);
	HS.l[l_expert-1].setText(Translation->str[ins_exp]);
	for(int i=l_beginner;i<=l_expert;++i)
	{
		if(c_time->rec->has_global[i]==rs_no_record)
			HS.hs[i-1].setNum(999.999);
		else 
			HS.hs[i-1].setText(QString::number(c_time->rec->global[i],'f',3));
		if(c_time->rec->has_global[i]==rs_has_record_and_name)
			HS.n[i-1].setText(c_time->rec->n_global[i]);
		else
			HS.n[i-1].setText(Translation->str[ins_anon]);
	}
	HS.move(x_hs,y_hs);
	subdialogs=true;
	HS.exec();
	subdialogs=false;
	x_hs=HS.x();
	y_hs=HS.y()+y_fix;
	disconnect(&HS,SIGNAL(reset_hiscores()),this,SLOT(reset_time_hiscores()));
}
void game::show_adv_hiscores()
{
	adv_hiscores_window AHS(Translation,Skin,Counters,date_template,records_on_ahs);
	hiscores_changed=false;
	connect(&AHS,SIGNAL(reset_hiscores()),this,SLOT(show_adv_hiscores()));
	connect(&AHS,SIGNAL(load_hiscores()),this,SLOT(load_records_from_history()));
	AHS.move(x_ahs,y_ahs);
	subdialogs=true;
	AHS.exec();
	subdialogs=false;
	x_ahs=AHS.x();
	y_ahs=AHS.y()+y_fix;
	disconnect(&AHS,SIGNAL(reset_hiscores()),this,SLOT(show_adv_hiscores()));
	disconnect(&AHS,SIGNAL(load_hiscores()),this,SLOT(load_records_from_history()));
}

void game::reset_time_hiscores()
{
	Counters->counters[Counters->id_time]->reset_records();
}
void game::hiscores_reset()
{
	hiscores_changed=true;
}
void game::load_records_from_history()
{
	QFile f(settings_dir+F_history);
	QString popup;
	board_data brd;
	gamemode t_mode;
	evaluator temp_ev(&brd,&t_mode);
	
	if(!f.open(QIODevice::ReadOnly))
	{
		err(Translation->str[ins_e_open_history]);
		return;
	}

	temp_ev.hist_request=true;
	while(!f.atEnd())
	{
		History->read_game(&brd,&t_mode,&temp_ev,f);
		popup+=Counters->check_records(brd.level,brd.r_clicks || brd.d_clicks,
			temp_ev.get_date());		
	}
	QString msg,name;
	popup=popup.right(popup.length()-1); // there was a leading comma
	msg=Translation->str[ins_have_new]+" "+Translation->str[ins_hi_scores]+"\n"+Translation->str[ins_request_name];
	name=input(this,msg,PlayerID->text);
	if(name=="") name=Translation->str[ins_anon];
	for(int i=l_beginner;i<=l_expert;++i) Counters->set_record_names(name,i);
	
	new_game();
	
	hiscores_changed=true;
	cout<<"records loaded from history, level is "<<Board->D.level<<endl;
}

void game::show_history(const QString& param)
{
	QStringList s;
	if(param!="") s << param;
	history_program.start(programs_dir+"freesweeper-history",s);
}

void game::irc_copy_last_finish()
{
	if(video)
		IRCModule->copy_string(ii_finished_video,PlayerID->text);
	else
		switch(Smiley->mode)
		{
		case classic:case lucky:
			IRCModule->copy_string(ii_finished);
			break;
		case density:
			IRCModule->copy_string(ii_finished_dens);
			break;
		default:	
			IRCModule->copy_string(ii_finished_cheat);
			break;
		}
}
void game::irc_copy_last_blast()
{
	if(video)
		IRCModule->copy_string(ii_blasted_video,PlayerID->text);
	else
		switch(Smiley->mode)
		{
		case classic:case lucky:
			IRCModule->copy_string(ii_blasted);
			break;
		case density:
			IRCModule->copy_string(ii_blasted_dens);
			break;
		default:	
			IRCModule->copy_string(ii_blasted_cheat);
			break;
		}
}
void game::irc_copy_marathon()
{
	Eval->refresh(Timer->get_time());
	IRCModule->copy_string(ii_marathon);
}


void game::show_options()
{
	options_window OW(Translation,Skin);
	OW.move(x_ow,y_ow);
	
	OW.mode[Smiley->mode].setChecked(true);
	OW.mode_changed();
	OW.level[Board->D.level-1].setChecked(true);
	OW.viennaTimer.setChecked(RedTimer->has_tenth);
	if(!counters_shown)
		OW.hideCounters.setChecked(true);
	else
		if(Eval->show_stats)
			OW.showCounters.setChecked(true);
		else
			OW.showCountersAfterEnd.setChecked(true);
	if(Counters->separated)
		OW.separated.setChecked(true);
	else
		OW.onMainWindow.setChecked(true);		
	if(Board->middle_chord)
		OW.chordWheel.setChecked(true);
	else
		OW.f2Wheel.setChecked(true);
	OW.hideTimer.setChecked(hide_timer);
	OW.voidRedMines.setChecked(null_mines_counter);
	OW.alwaysAtTop.setChecked(windowFlags()&Qt::WindowStaysOnTopHint);
	OW.wideSmiley.setChecked(wide_smiley_area);
	OW.revealSquares.setChecked(Board->reveal_after_end);
	OW.forceNF.setChecked(Board->nf);
	OW.flagMines.setChecked(Board->flag_mines_after_win);
	OW.questionMarks.setChecked(Board->qmarks);
	OW.markWastedFlags.setChecked(Board->show_wasted_flags_after_end);
	OW.restartByTimeLimit.setChecked(restart_by_time);
	OW.begTimeLimit.setValue(restart_beg/1000);
	OW.intTimeLimit.setValue(restart_int/1000);
	OW.expTimeLimit.setValue(restart_exp/1000);
	OW.beepByTime.setChecked(beep);
	OW.begTimeBeep.setValue(beep_beg/1000);
	OW.intTimeBeep.setValue(beep_int/1000);
	OW.expTimeBeep.setValue(beep_exp/1000);
	OW.time_limit();
	OW.boardClearance.setChecked(Board->clear_board_threshold); 
	OW.boardClearancePercent.setValue(Board->clear_board_threshold);
	OW.board_clearance();
	
	OW.luckLibrary.setChecked(Board->luck_library);
	OW.luckSolver.setChecked(Board->luck_solver);

	OW.customWidth.setValue(custom_width);
	OW.customHeight.setValue(custom_height);
	OW.customMines.setValue(custom_mines);

	OW.showBoard.setChecked(Board->show_board);
	OW.autoFlag.setChecked(Board->autoflag);
	OW.lawnmower.setChecked(Board->lawnmower);
	OW.elmarTechnique.setChecked(Board->elmar);
	OW.nonoMouse.setChecked(Board->nono);
	OW.superFlag.setChecked(Board->superflag);
	OW.superClick.setChecked(Board->superclick);
	OW.showProbabilities.setChecked(Board->probabilities);

	OW.g_restrictBBBV.setChecked(bbbv_limit_enabled);
	OW.minimumBBBV.setValue(cheat_bbbv_lower_limit);
	OW.maximumBBBV.setValue(cheat_bbbv_upper_limit);
	OW.g_restrictZiNi.setChecked(zini_limit_enabled);
	OW.minimumZiNi.setValue(cheat_zini_lower_limit);
	OW.maximumZiNi.setValue(cheat_zini_upper_limit);
	OW.g_restrictHZiNi.setChecked(hzini_limit_enabled);
	OW.minimumHZiNi.setValue(cheat_hzini_lower_limit);
	OW.maximumHZiNi.setValue(cheat_hzini_upper_limit);
	OW.iterationsForBoard.setValue(cheat_generation_iterations);
	OW.sevenOnBoard.setChecked(Board->force_seven);
	OW.eightOnBoard.setChecked(Board->force_eight);

	if(!Board->lives)	// infinite
		OW.infiniteContinues.setChecked(true);
	else if(Board->lives==1)	// none
		OW.noContinues.setChecked(true);
	else
	{
		OW.severalContinues.setChecked(true);
		OW.continuesNumber.setValue(Board->lives-1);
	}
	OW.check_lives();
	
	
	subdialogs=true;
	if(OW.exec())
	{
		RedTimer->switch_tenth(OW.viennaTimer.isChecked());
		if(OW.hideCounters.isChecked()) hide_counters();
		else 
		{
			show_counters();
			Eval->show_stats=OW.showCounters.isChecked();
			if(OW.separated.isChecked())
				separate_counters();
			else
				dock_counters();
		}
		Board->middle_chord=!OW.f2Wheel.isChecked();
		hide_timer=OW.hideTimer.isChecked();
		null_mines_counter=OW.voidRedMines.isChecked();
		wide_smiley_area=OW.wideSmiley.isChecked();
		Qt::WindowFlags f=windowFlags();
		if(OW.alwaysAtTop.isChecked())
			f|=Qt::WindowStaysOnTopHint;
		else
			f&=~Qt::WindowStaysOnTopHint;
		if(f!=windowFlags()) 
		{
			setWindowFlags(f);
			show();
		}
		Board->reveal_after_end=OW.revealSquares.isChecked();
		Board->nf=OW.forceNF.isChecked();
		Board->flag_mines_after_win=OW.flagMines.isChecked();
		Board->qmarks=OW.questionMarks.isChecked();
		Board->show_wasted_flags_after_end=OW.markWastedFlags.isChecked();
		Board->luck_library=OW.luckLibrary.isChecked();
		Board->luck_solver=OW.luckSolver.isChecked();
		restart_by_time=OW.restartByTimeLimit.isChecked();
		restart_beg=OW.begTimeLimit.value()*1000;
		restart_int=OW.intTimeLimit.value()*1000;
		restart_exp=OW.expTimeLimit.value()*1000;
		beep=OW.beepByTime.isChecked();
		beep_beg=OW.begTimeBeep.value()*1000;
		beep_int=OW.intTimeBeep.value()*1000;
		beep_exp=OW.expTimeBeep.value()*1000;
		if(restart_by_time || beep) enable_time_limits(); else disable_time_limits();
		
		Board->show_board=OW.showBoard.isChecked();
		Board->autoflag=OW.autoFlag.isChecked();
		Board->lawnmower=OW.lawnmower.isChecked();
		Board->elmar=OW.elmarTechnique.isChecked();
		Board->nono=OW.nonoMouse.isChecked();
		Board->superflag=OW.superFlag.isChecked();
		Board->superclick=OW.superClick.isChecked();
		Board->probabilities=OW.showProbabilities.isChecked();
		bbbv_limit_enabled=OW.g_restrictBBBV.isChecked();
		zini_limit_enabled=OW.g_restrictZiNi.isChecked();
		hzini_limit_enabled=OW.g_restrictHZiNi.isChecked();
		cheat_bbbv_lower_limit=OW.minimumBBBV.value();
		cheat_bbbv_upper_limit=OW.maximumBBBV.value();
		cheat_zini_lower_limit=OW.minimumZiNi.value();
		cheat_zini_upper_limit=OW.maximumZiNi.value();
		cheat_hzini_lower_limit=OW.minimumHZiNi.value();
		cheat_hzini_upper_limit=OW.maximumHZiNi.value();
		cheat_generation_iterations=OW.iterationsForBoard.value();
		Board->force_seven=OW.sevenOnBoard.isChecked();
		Board->force_eight=OW.eightOnBoard.isChecked();
		if(OW.infiniteContinues.isChecked())	// infinite
			Board->lives=0;
		else if(OW.noContinues.isChecked())	// none
			Board->lives=1;
		else
			Board->lives=1+OW.continuesNumber.value();

		custom_width=OW.customWidth.value();
		custom_height=OW.customHeight.value();
		custom_mines=OW.customMines.value();
		Board->correct_board(custom_width,custom_height,custom_mines);
		
		if(OW.boardClearance.isChecked()) Board->clear_board_threshold=OW.boardClearancePercent.value();
		else Board->clear_board_threshold=0;
		
		for(int i=0;i<n_modes;++i) if(OW.mode[i].isChecked()) Smiley->set_mode(i);
		for(int i=0;i<n_levels;++i) if(OW.level[i].isChecked()) set_level(i+1);
	}
	subdialogs=false;
	x_ow=OW.x();
	y_ow=OW.y()+y_fix;
}
void game::select_skin()
{
	QString s=QFileDialog::getOpenFileName(this,Translation->str[ins_select_skin],settings_dir+"skins/","*.skin");
	if(s=="") return;
	Skin->load(s);
	Board->resize(Board->D.width,Board->D.height,Board->D.n_mines);
	place_things();
	repaint();
}
void game::show_custom_marathon_options()
{
	custom_marathon_editor CME(Translation,Skin,&custom_marathon_description);
	
	subdialogs=true;
	CME.exec();
	subdialogs=false;
	
	if(level==l_m_custom) set_level(l_m_custom);	// load edited marathon
	if(Board->marathon) new_marathon();
}

void game::hide_counters()
{
	if(!counters_shown) return;
	counters_shown=false;
	x_ctr=Counters->x();
	y_ctr=Counters->y()+y_fix;
	Counters->hide();
}
void game::show_counters()
{
	if(counters_shown) return;
	counters_shown=true;
	Counters->move(x_ctr,y_ctr);
	Counters->show();
}

void game::hide_video_control()
{
	video=false;
	if(!VideoControl) return;
	x_vid=VideoControl->x();
	y_vid=VideoControl->y()+y_fix;
	delete VideoControl;VideoControl=0;
	if(Board->marathon) new_marathon();
}
void game::show_video_control()
{
	if(VideoControl) delete VideoControl;
	VideoControl=new videocontrol(Translation,Skin,
		Board->video_duration,Board->clone_events_enabled,&(Board->click_events));
	VideoControl->move(x_vid,y_vid);
	VideoControl->goto_offset=go_to_event_offset;
	connect(Timer,SIGNAL(tick(int)),VideoControl,SLOT(tick(int)));
	connect(VideoControl,SIGNAL(stop_video()),this,SLOT(stop_video()));
	connect(VideoControl,SIGNAL(play_video()),this,SLOT(resume_video()));
	connect(VideoControl,SIGNAL(pause_video()),this,SLOT(pause_video()));
	connect(VideoControl,SIGNAL(exit_video()),Smiley,SIGNAL(new_game()));
	connect(VideoControl,SIGNAL(time_changed(int)),this,SLOT(move_video(int)));
	connect(VideoControl,SIGNAL(speed_changed(int)),Timer,SLOT(set_coeff(int)));

	connect(VideoControl,SIGNAL(toggle_path(bool)),Board,SLOT(toggle_mouse_path(bool)));
	connect(VideoControl,SIGNAL(toggle_reveal(bool)),Board,SLOT(toggle_reveal(bool)));
	connect(VideoControl,SIGNAL(toggle_openings(bool)),Board,SLOT(toggle_opening_borders(bool)));
	
	for(int i=1;i<Board->clone_events.size();++i)
		if(Board->clone_events[i].event_id)
			VideoControl->add_event(Board->clone_events[i].timestamp,Board->clone_events[i].event_id);
	
	VideoControl->mousePath.setChecked(Board->v_mouse_path);
	VideoControl->revealSquares.setChecked(Board->v_show_board);
	VideoControl->showOpenings.setChecked(Board->v_openings);	
	VideoControl->show();
}

void game::resizeEvent(QResizeEvent* e)
{
	e->ignore();	// it is unresizeable
}
void game::closeEvent(QCloseEvent*)
{
	exit_freesweeper();
}

void game::paintEvent(QPaintEvent* ev)
{
	if(ev->rect().top()>Skin->head_size+MenuBar->height()) return;

	painter.begin(this);
	painter.fillRect(ev->rect(),QBrush(Skin->background));
	int w=Board->real_width;
	// left border
	painter.drawPixmap(0,0+MenuBar->height(),Skin->lborder,0,0,Skin->square_size,Skin->head_size);
	// right border
	painter.drawPixmap((w+1)*Skin->square_size,0+MenuBar->height(),
			Skin->rborder,0,0,Skin->square_size,Skin->head_size);
	painter.end();

	if(ev->rect().top()>Skin->topborder_size+MenuBar->height()) return;

	painter.begin(this);
	// top border
	int b=Skin->square_size>ev->rect().left() ? Skin->square_size : ev->rect().left();
	int e=Skin->square_size*(w+1)<ev->rect().right() ? Skin->square_size*(w+1) : ev->rect().right();
	for(int i=b;i<e;++i)
		painter.drawPixmap(i,0+MenuBar->height(),Skin->tborder,0,0,1,Skin->topborder_size);
	painter.end();

	// this window was probably activated from inactive state; so we raise the subwindows
	if(isActiveWindow() && !subdialogs)
	{
		Counters->raise();
		if(VideoControl) VideoControl->raise();
	}
}
void game::draw_invisible(const QPixmap& pix)
{
	painter.begin(this);
	painter.drawPixmap(0,0,pix);
	painter.end();
}

void game::mousePressEvent(QMouseEvent* e)
{
	if(wide_smiley_area && is_wide_smiley(e)) Smiley->press(e->button());
}
void game::mouseReleaseEvent(QMouseEvent* e)
{
	if(wide_smiley_area) Smiley->release(e->button());
}
void game::mouseMoveEvent(QMouseEvent* e)
{
	if(wide_smiley_area)
	{
		if(is_wide_smiley(e))
			Smiley->mouse_return();
		else
			Smiley->cancel();
	}
}

void game::moveEvent(QMoveEvent* e)
{
	if(Counters && Counters->attached && loaded)
	{
		Counters->move(Counters->QWidget::x()+e->pos().x()-e->oldPos().x(),
						Counters->QWidget::y()+e->pos().y()-e->oldPos().y());
	}
}

bool game::is_recognizable_filename(QString s)
{
	if(s.right(6)==".rawvf")
		return true;
	else if(s.right(4).toLower()==".avf" || 
        	s.right(F_secure_ext_l).toLower()==F_secure_extension)
        	return true;
	else if(!Board->marathon && (s.right(4)==".mbf" || s.right(4)==".abf"))
		return true;
	else
		return false;
}

void game::dragEnterEvent(QDragEnterEvent *e)
{
	cout<<"drag enter"<<endl;
	if(e->mimeData()->hasUrls())
	{
		QString path=e->mimeData()->urls()[0].path();
		if(is_recognizable_filename(path))
			e->acceptProposedAction();
	}
	cout<<"drag enter ends"<<endl;
}
void game::dragMoveEvent(QDragMoveEvent *e)
{
	cout<<"drag move"<<endl;
/*	if(e->mimeData()->hasUrls())
	{
		QString path=e->mimeData()->urls()[0].path();
		if(is_recognizable_filename(path))
			e->acceptProposedAction();
	}*/
	cout<<"drag move ends"<<endl;
}
void game::dragLeaveEvent(QDragLeaveEvent *e)
{
	e->accept();
}
void game::dropEvent(QDropEvent* e)
{
	if(e->mimeData()->hasUrls())
	{
		QString path=e->mimeData()->urls()[0].path();
		QFile f(path);
		if(!f.open(QIODevice::ReadOnly))
			path=path.right(path.size()-1);	// window$(r)(c)(tm) bugs
		else
			f.close();
		if(drop_file(path)) e->acceptProposedAction();
	}
}
bool game::drop_file(QString path)
{
	if(path.right(6).toLower()==".rawvf")
	{
		load_video(path);				
		return true;
	}
	else if(path.right(4).toLower()==".avf" || 
        	path.right(F_secure_ext_l).toLower()==F_secure_extension)
        {
		load_video(path);
		return true;
 	}
	else if(!Board->marathon && (path.right(4).toLower()==".mbf" || path.right(4).toLower()==".abf"))
	{
		if(!upk_allowed) Smiley->set_mode(upk);
		load_board(path);
		return true;
	}
	return false;
}
bool game::is_wide_smiley(QMouseEvent* e)
{
	return e->x()>RedMines->x()+RedMines->width() && e->x()<RedTimer->x() &&
		e->y()>MenuBar->y()+MenuBar->height() && e->y()<Board->y();
}

void game::place_things()
{
	int w=Board->real_width;
	int h=Board->D.height;
	int dis_width=3*Skin->display_width+2;	// the width of the red displays (2 is the width of frames)
	int dis_height=Skin->display_height+2;	// the height of the red displays (2 is the height of frames)
	int s_d=Skin->small_display_width;	// the modifier for the display with tenths
	int mod_h=0,mod_w=0;	// space for the scroll bars
		
	if(w>Skin->max_width) w=Skin->max_width;
	if(h>Skin->max_height) h=Skin->max_height;
	
	Board->move(0,Skin->head_size+MenuBar->height());
	// make scrollbars if needed
	if(Board->scroll_x) 
	{
		ScrX->setMaximum(Board->D.width-Skin->max_width);
		mod_h=ScrX->height();
		ScrX->setGeometry(0,MenuBar->height()+Skin->head_size+Board->QWidget::height(),
				Board->QWidget::width(),Skin->square_size); // bottom
		ScrX->setValue(0);
		ScrX->show();
	}
	else ScrX->hide();
	if(Board->scroll_y)
	{
		ScrY->setMaximum(Board->D.height-Skin->max_height);
		mod_w=ScrY->width();
		ScrY->setGeometry(Board->QWidget::width(),Skin->head_size+MenuBar->height(),
				Skin->square_size,Board->QWidget::height()); // right
		ScrY->setValue(0);
		ScrY->show();
	}
	else ScrY->hide();
	
	int red_display_y=MenuBar->height()+(Skin->head_size+Skin->topborder_size-dis_height)/2;
	RedTimer->setGeometry(Skin->square_size*(w+1)-dis_width-s_d*RedTimer->has_tenth+Skin->rt_h_offset,
							red_display_y,
								dis_width+RedTimer->has_tenth*s_d,
									dis_height);
	RedMines->setGeometry(Skin->square_size+Skin->rm_h_offset,
							red_display_y,
								dis_width,
									dis_height);
	Smiley->setGeometry(Skin->square_size*(w+2)/2-(Skin->smiley_width+1)/2+
				Skin->smiley_h_o_offset*(RedTimer->has_tenth && w<=Skin->crit_width_2),
							Skin->topborder_size+Skin->smiley_v_offset+MenuBar->height(),
								Skin->smiley_width,
									Skin->smiley_height);
	PlayerID->setGeometry(Skin->label_h_offset,
				MenuBar->height()+Skin->head_size+Skin->square_size*(h+2)+Skin->label_v_offset+mod_h,
					Skin->square_size*(w+2)-2*Skin->label_h_offset,
						Skin->label_height);
	if(Counters && !Counters->separated && counters_shown)
	{
		Counters->move(0,MenuBar->height()+
					Skin->head_size+
						Skin->square_size*(h+2)+
							2*Skin->label_v_offset+PlayerID->height()+
								mod_h);
		setFixedSize(Skin->square_size*(w+2)+mod_w,
					MenuBar->height()+
						Skin->head_size+
							Skin->square_size*(h+2)+
								PlayerID->height()+2*Skin->label_v_offset+
									Counters->height()+
										mod_h);
	}
	else
		setFixedSize(Skin->square_size*(w+2)+mod_w,
					MenuBar->height()+
						Skin->head_size+
							Skin->square_size*(h+2)+
								PlayerID->height()+2*Skin->label_v_offset+
									mod_h);
}
void game::dock_counters()
{
	Counters->separated=false;
	Counters->setParent(this);
	if(counters_shown) Counters->show(); else Counters->hide();
	place_things();
}
void game::separate_counters()
{
	Counters->separated=true;
	Counters->setParent(0);
	Counters->move(x_ctr>0?x_ctr:0,y_ctr>0?y_ctr:0);
	if(counters_shown) Counters->show(); else Counters->hide();
	place_things();
}

void game::connect_things()
{
	// process board signals
	// to the smiley
	connect(Board,SIGNAL(press()),Smiley,SLOT(stress()));
	connect(Board,SIGNAL(click()),Smiley,SLOT(unstress()));
	connect(Board,SIGNAL(changed_state(boardstate)),Smiley,SLOT(changed_state(boardstate)));

	connect(Board,SIGNAL(force_new_game()),Smiley,SIGNAL(new_game()));

	connect(Board,SIGNAL(video_mode(int)),Smiley,SLOT(set_mode_without_restart(int)));
	
	// to the red mines counter
	connect(Board,SIGNAL(flag()),RedMines,SLOT(decrement()));
	connect(Board,SIGNAL(unflag()),RedMines,SLOT(increment()));
	
	// monitor the state of the board
	connect(Board,SIGNAL(changed_state(boardstate)),this,SLOT(changed_state(boardstate)));
	connect(Board,SIGNAL(video_finished()),this,SLOT(save_video_by_default()));

	// adjust the window size to the field size
	connect(Board,SIGNAL(resized()),this,SLOT(adjust_to_board()));

	// scroll bars
	connect(ScrX,SIGNAL(valueChanged(int)),Board,SLOT(scrolling_x(int)));
	connect(ScrY,SIGNAL(valueChanged(int)),Board,SLOT(scrolling_y(int)));
	
	// red timer
	connect(Timer,SIGNAL(tick(int)),RedTimer,SLOT(show_time(int)));

	// smiley
	connect(Smiley,SIGNAL(changed_mode(gamemode)),this,SLOT(changed_mode(gamemode)));
	connect(Smiley,SIGNAL(new_game()),this,SLOT(new_game()));	

	connect(&levels_mapper,SIGNAL(mapped(int)),this,SLOT(set_level(int)));
	connect(&modes_mapper,SIGNAL(mapped(int)),Smiley,SLOT(set_mode(int)));

	// player id
	connect(PlayerID,SIGNAL(change_player(QString)),this,SLOT(change_player(QString)));
	connect(Board,SIGNAL(display_player(QString)),this,SLOT(display_player(QString)));

	connect(Counters,SIGNAL(resized()),this,SLOT(place_things()));

	connect(Autosaver,SIGNAL(save_rawvf(QString)),this,SLOT(save_rawvf_auto(QString)));
	connect(Autosaver,SIGNAL(save_secure(QString)),this,SLOT(save_secure_auto(QString)));

	connect(&irc_edit_mapper,SIGNAL(mapped(int)),IRCModule,SLOT(edit_string(int)));
	connect(&irc_history_mapper,SIGNAL(mapped(const QString&)),this,SLOT(show_history(const QString&)));
}

void game::smiley_as_new()
{
	if(smiley_restarts)
	{
		disconnect(Smiley,SIGNAL(new_game()),this,SLOT(restart()));	
		connect(Smiley,SIGNAL(new_game()),this,SLOT(new_game()));	
		smiley_restarts=false;
	}
}
void game::smiley_as_restart()
{
	if(!smiley_restarts)
	{
		disconnect(Smiley,SIGNAL(new_game()),this,SLOT(new_game()));	
		connect(Smiley,SIGNAL(new_game()),this,SLOT(restart()));	
		smiley_restarts=true;
	}
}

void game::upgrade_red_mines()
{
	if(!red_mines_state) connect(RedMines,SIGNAL(double_click()),this,SLOT(change_n_mines()));
	red_mines_state=true;
}
void game::downgrade_red_mines()
{
	if(red_mines_state) disconnect(RedMines,SIGNAL(double_click()),this,SLOT(change_n_mines()));
	red_mines_state=false;
}
void game::connect_counters()
{
	if(!counters_state) connect(Timer,SIGNAL(tick_since_stop(int)),Counters,SLOT(refresh(int)));
	counters_state=true;
}
void game::disconnect_counters()
{
	if(counters_state) disconnect(Timer,SIGNAL(tick_since_stop(int)),Counters,SLOT(refresh(int)));	
	counters_state=false;
}

void game::change_n_mines()
{
	Board->change_n_mines(input_int(RedMines,Translation->str[ins_request_mines],
		Board->D.n_mines,0,Board->size-1));
}

void game::adjust_to_board()
{
	set_limits();
	reset_displays(); 
	place_things();
}

void game::reset_displays()
{
	if(!Timer->paused()) RedTimer->show_time(0);
	if(Board->marathon)
		RedMines->show(Board->remaining_mines());
	else
		RedMines->show(Board->D.n_mines);
}

void game::wheel_as_chord()
{
	Board->middle_chord=true;
}
void game::wheel_as_new_game()
{
	Board->middle_chord=false;
}

void game::enable_time_limits()
{
	if(!time_limits_enabled)
	{
		time_limits_enabled=true;
		connect(Timer,SIGNAL(tick_since_stop(int)),this,SLOT(check_time_limits(int)));
	}
}
void game::disable_time_limits()
{
	if(time_limits_enabled)
	{
		time_limits_enabled=false;
		disconnect(Timer,SIGNAL(tick_since_stop(int)),this,SLOT(check_time_limits(int)));
	}
}

void game::check_time_limits(int time)
{
	if(restart_by_time &&
		((Board->D.level==l_beginner && time>restart_beg) ||
		(Board->D.level==l_intermediate && time>restart_int) ||
		(Board->D.level==l_expert && time>restart_exp)))
		Smiley->force_new_game();
	if(beep && !beepped &&
		((Board->D.level==l_beginner && time>beep_beg) ||
		(Board->D.level==l_intermediate && time>beep_int) ||
		(Board->D.level==l_expert && time>beep_exp)))
	{
		beepped=true;
		QApplication::beep();
	}
}


void game::save_video_by_default()
{
	Board->save_rawvf(video_dir+F_default_video_file_name,Smiley->mode);
	Board->save_avf(video_dir+F_default_secure_video_file_name,Smiley->mode);
}

void game::stop_video()
{
	Board->stop_video();
}
void game::pause_video()
{
	Timer->pause();
}
void game::resume_video()
{
	if(Board->D.state==playing_video)
		Timer->resume();
	else
	{
		Board->restart(true);
		Board->play_video();
	}
}
void game::move_video(int time)
{
	bool pause=Timer->paused();
	if(pause) Timer->resume();
	if(time>Timer->get_time())
	{
		Timer->set_time(time);
	}
	else
	{
		Board->stop_video();
		Board->restart(true);
		Board->play_video();
		Timer->set_time(time);
	}
	if(pause) Timer->pause();
}

void game::exit_freesweeper()
{
	QCoreApplication::exit(0);
}

