#include "ircmodule.h"

const QString F_irc="freesweeper.irc";

ircmodule::ircmodule(translation* T,parser* p,evaluator* ev):T(T),Parser(p),Eval(ev),
strings_changed(settings_dir!=normal_settings_dir)
{
	for(int i=0;i<n_irc_strings;++i) strings[i]=0;
}

ircmodule::~ircmodule()
{
	for(int i=0;i<n_irc_strings;++i) if(strings[i]) delete strings[i];
}

void ircmodule::load()
{
	QFile f(settings_dir+F_irc);
	
	if(!f.open(QIODevice::ReadOnly))
	{
		strings_changed=true;
		err(T->str[ins_e_open_irc]);
	}
	else
	{
		for(int i=0;i<n_irc_strings;++i)
		{
			QString string_desc=readln_without_comments(f).trimmed();
			strings[i]=Parser->parse_string(string_desc);
		}
		f.close();
	}
}

void ircmodule::save()
{
	QFile f(settings_dir+F_irc);

	if(!strings_changed) return;
	if(!f.open(QIODevice::WriteOnly | QIODevice::Truncate))
		err(T->str[ins_e_open_irc]);
	else
	{
		for(int i=0;i<n_irc_strings;++i)	
		{
			if(strings[i]) writeln(f,strings[i]->print()); else writeln(f,"");
		}
		f.close();
	}
}

void ircmodule::copy_string(int id)
{
	if(!strings[id]) return;
	QString s=Eval->eval(strings[id]);
	QApplication::clipboard()->setText(s);
}
void ircmodule::copy_string(int id,QString player)
{
	if(!strings[id]) return;
	QString s=Eval->eval(strings[id]);
	QApplication::clipboard()->setText(s.replace("[player]",player));
}

void ircmodule::edit_string(int id)
{
	int desc[n_irc_strings]={ins_fin,ins_fin_dens,ins_fin_cheat,ins_fin_video,
		ins_blast,ins_blast_dens,ins_blast_cheat,ins_blast_video,ins_marathon};
	QString s=input(0,T->str[ins_request_template]+" "+T->str[desc[id]]+" :",strings[id]->print());
	if(s=="") return;
	if(strings[id]) delete strings[id];
	strings[id]=Parser->parse_string(s);
	strings_changed=true;
}

