#ifndef _FS_PARSER_H
#define _FS_PARSER_H

#include "freesweeper.h"
#include "formulaid.h"
#include "autosavecondition.h"
#include "counter.h"
#include "evaluable.h"
#include "rank.h"
#include "translation.h"

struct excparser
{
	int sym;
	QString msg;
	excparser(int sym,const QString& msg):sym(sym),msg(msg){};
};

class parser
{
	QMap<QString,formula_id> name_to_id;
	QMap<QString,c_formula_type> name_to_type;
	
	static const char pipe='|';

	inline QCharRef sym(QString& s, int cur);	
	
	int get_expression1(QString& s,int beg,formula*& f);
	// expression1 ::= expression2 or expression1 | expression2
	int get_expression2(QString& s,int beg,formula*& f);
	// expression2 ::= expression3 and expression2 | expression2
	int get_expression3(QString& s,int beg,formula*& f);
	// expression3 ::= [expression1] | formula1 < formula1 | formula1 > formula1 | formula1 = formula1
	int get_formula1(QString& s,int beg,formula*& f);	
	// formula1 ::= formula2 + formula1 | formula2 - formula1 | formula2
	int get_formula2(QString& s,int beg,formula*& f);
	// formula2 ::= formula3 * formula2 | formula3 / formula2 | formula3
	int get_formula3(QString& s,int beg,formula*& f);
	// formula3 ::= formula4 ^ formula3
	int get_formula4(QString& s,int beg,formula*& f);
	// formula4 ::= -formula4 | (formula1) | sqr(formula1) | log(formula1) | exp(formula1) | rank(int) | token 
	int get_token(QString& s,int beg,formula*& f);

	int get_cf(QString& s,int beg,c_formula*& f);	
	// counterformula ::= formula1 | formula1{precision} |  formula1 'separator' counterformula |
	// formula1{precision} 'separator' counterformula
	// 
	int get_function(QString& s,int& cur,formula*& f,formula_id x);
	int get_special_cf(QString& s,int beg,c_formula*& f);
	
	void ignore_spaces(QString& s,int& cur);
	
	// token readers
	QString get_id(QString& s,int& beg);
	QString get_string(QString& s,int& beg);
	QString get_string_with_spaces(QString& s,int& beg);
	double get_number(QString& s,int& beg);
	int get_int(QString& s,int& beg);
	
	void find_pipe(QString& s,int& cur);
	void find_double_pipe(QString& s,int& cur);
public:
	translation* T;
	QString video_dir;
	
	parser(translation* T,QString video_dir);

	formula* parse_expression(QString s);
	c_formula* parse_formula(QString s);
	evaluable_string* parse_string(QString s);
	rank* parse_rank(QString s);
	
	counter* parse_counter(QString s);
	autosave_condition* parse_autosave_condition(QString s);

	int read_int(QString& s,int& beg);
};

#endif

