#include "record.h"

record::record()
{
	popup=true;
	lower_is_better=true;
	for(int i=1;i<=n_stand_levels;++i)
	{
		n_global[i]=n_f[i]=n_nf[i]="";
		has_global[i]=has_f[i]=has_nf[i]=rs_no_record;
		global[i]=f[i]=nf[i]=0;
	}
}

record::record(bool lower_is_better,bool popup):lower_is_better(lower_is_better),popup(popup)
{
	for(int i=1;i<=n_stand_levels;++i)
	{
		n_global[i]=n_f[i]=n_nf[i]="";
		has_global[i]=has_f[i]=has_nf[i]=rs_no_record;
		global[i]=f[i]=nf[i]=0;
	}
}

void record::set_global(int level,double val,bool flagging,const QDateTime d)
{
	global[level]=val;
	n_global[level]="";
	has_global[level]=rs_has_record_without_name;
	d_global[level]=d;
	if(flagging) set_f(level,val,d);
	else set_nf(level,val,d);
}
void record::set_f(int level,double val,const QDateTime d)
{
	f[level]=val;
	n_f[level]="";
	has_f[level]=rs_has_record_without_name;
	d_f[level]=d;
}
void record::set_nf(int level,double val,const QDateTime d)
{
	nf[level]=val;
	n_nf[level]="";
	has_nf[level]=rs_has_record_without_name;
	d_nf[level]=d;
}

QString record::check(int level,double val,bool flagging,const QDateTime d)
{
	const QString zero_popup="";
	const QString global_record_prefix=", ";
	const QString f_record_prefix=", F ";
	const QString nf_record_prefix=", NF ";
	if( !has_global[level] || (lower_is_better && val<global[level]) || (!lower_is_better && val>global[level]) )
	{
		set_global(level,val,flagging,d);
		if(popup) return global_record_prefix; else return zero_popup;
	}
	else if(flagging)
	{
		if( !has_f[level] || (lower_is_better && val<f[level]) || (!lower_is_better && val>f[level]) )
		{
			set_f(level,val,d);
			if(popup) return f_record_prefix; else return zero_popup;
		}
	}
	else // NF
	{
		if( !has_nf[level] || (lower_is_better && val<nf[level]) || (!lower_is_better && val>nf[level]) )
		{
			set_nf(level,val,d);
			if(popup) return nf_record_prefix; else return zero_popup;
		}
	}
 	return zero_popup;
}

void record::set_name(const QString& name,int l)
{
	if(has_global[l]==rs_has_record_without_name) 
	{
		n_global[l]=name; 
		has_global[l]=rs_has_record_and_name;
	}
	if(has_f[l]==rs_has_record_without_name) 
	{
		n_f[l]=name; 
		has_f[l]=rs_has_record_and_name;
	}
	else if(has_nf[l]==rs_has_record_without_name)
	{
		n_nf[l]=name; 
		has_nf[l]=rs_has_record_and_name;
	}
}

