#include "freesweeper.h"
#include "timer.h"
#include "translation.h"
#include "skin.h"
#include "parser.h"
#include "board.h"

void show_help(QTextStream& d)
{
	d<<"Command: <n_boards> <width> <height> <mines> [fbzhrsadce]"<<endl;
	d<<" f - run fast 3bv calculator"<<endl;
	d<<" b - run normal 3bv calculator"<<endl;
	d<<" z - run zini"<<endl;
	d<<" h - run human zini"<<endl;
	d<<" r - check for 3-1 guess patterns"<<endl;
	d<<" s - check for 2x2 boxes"<<endl;
	d<<" a - show averages"<<endl;
	d<<" d - show distribution"<<endl;
	d<<" c - safe square in a corner"<<endl;
	d<<" e - safe square on an edge"<<endl;
	d<<"Type ? to see this message"<<endl;
	d<<"Type q to quit"<<endl;
}

int test()
{
	timer tm;
	translation trans;
	skin sk(&trans);
	parser p(&trans,"");
	board brd(&trans,&sk,&tm);
	QTextStream stream(stdin);
	QTextStream cout(stdout);

	sk.crit_width=1;
	sk.max_width=sk.max_height=999;
	sk.square_size=1;
	
	QMap<int,int> d,dz,dhz;
	QMap<int,int> dr1,dr2;
	QString program;
	QTime t;

	int width=8,height=8,mines=10;
	unsigned int n=100000;
	unsigned int sum,z_sum,hz_sum;
	unsigned int r1_sum,r2_sum;
	bool show_avg,show_distr;
	bool fast_3bv,full_3bv,zini,hzini;
	bool r1,r2;
	int safe_x=2,safe_y=2;
	int cur_z,cur_hz;
	int cur;
	int cur_r1,cur_r2;
	bool nosafe;
	
	show_help(cout);
	while(1)
	{
		nosafe=true;
		cout<<"> ";
		cout.flush();
		program=stream.readLine();
		if(program.contains('?'))
		{
			show_help(cout);
			continue;
		}
		if(program.contains('q')) break;
	
		cur=0;
		n=p.read_int(program,cur);
		width=p.read_int(program,cur);
		height=p.read_int(program,cur);
		mines=p.read_int(program,cur);

		if(program[cur].isDigit())
		{
			safe_x=p.read_int(program,cur);
			safe_y=p.read_int(program,cur);
			nosafe=false;
		}
		
		program=program.mid(cur);

		fast_3bv=program.contains('f');
		full_3bv=program.contains('b');
		zini=program.contains('z');
		hzini=program.contains('h');
		show_avg=program.contains('a');
		show_distr=program.contains('d');
		r1=program.contains('r');
		r2=program.contains('s');
		if(program.contains('c'))
			safe_x=safe_y=1;
		else if(program.contains('e'))
		{	safe_x=1; safe_y=height/2;	}
		else if(nosafe)
		{	safe_x=width/2; safe_y=height/2;	}
		
		cout<<"Executing test:"<<endl;
		cout<<"Width: "<<width<<", height: "<<height<<", mines: "<<mines<<endl;
		cout<<"Safe square: "<<safe_x<<" "<<safe_y<<endl;
		cout<<"Number of iterations: "<<n<<endl;
		cout<<"Procedures: "<<
			(fast_3bv?"fast3bv ":"")<<
			(full_3bv?"normal3bv ":"")<<
			(zini?"zini ":"")<<
			(hzini?"hzini ":"")<<
			(r1?"check3-1 ":"")<<
			(r2?"check2x2 ":"")<<endl;
		cout<<"Output: "<<
			(show_avg?"averages ":"")<<
			(show_distr?"distribution ":"")<<endl<<endl;
		brd.resize_if_needed(width,height,mines);
		sum=z_sum=hz_sum=r1_sum=r2_sum=0;
		d.clear();
		dz.clear();
		dhz.clear();
		dr1.clear();
		dr2.clear();
		t.start();
		for(unsigned int i=0;i<n;++i)
		{
			brd.erase_board();
			brd.random_board(safe_x,safe_y);
			if(fast_3bv) brd.get_bbbv_fast();
			if(full_3bv || zini || hzini) brd.init_board_basic();
			if(zini) 
			{
				z_sum+=(cur_z=brd.get_zini(false));
				++dz[cur_z];
			}
			if(hzini) 
			{
				hz_sum+=(cur_hz=brd.get_zini(true));
				++dhz[cur_hz];
			}
			if(fast_3bv || full_3bv)
			{
				++d[brd.D.bbbv];
				sum+=brd.D.bbbv;
			}
			if(r1)
			{
				cur_r1=0;
				for(int i=0;i<=width-1;++i)
					for(int j=0;j<=height-2;++j)
						if(brd.squares[i][j].unopenable() && 
						brd.squares[i+1][j].unopenable() &&
						brd.squares[i+2][j].unopenable() &&
						brd.squares[i][j+3].unopenable() &&
						brd.squares[i+1][j+3].unopenable() &&
						brd.squares[i+2][j+3].unopenable() &&
						((brd.squares[i+1][j+1].mine && !brd.squares[i+1][j+2].mine) ||
						(!brd.squares[i+1][j+1].mine && brd.squares[i+1][j+2].mine)))
						{
							cout<<"p1 "<<i<<" "<<j<<endl;
							++cur_r1;
						}
				for(int i=0;i<=width-2;++i)
					for(int j=0;j<=height-1;++j)
						if(brd.squares[i][j].unopenable() && 
						brd.squares[i][j+1].unopenable() &&
						brd.squares[i][j+2].unopenable() &&
						brd.squares[i+3][j].unopenable() &&
						brd.squares[i+3][j+1].unopenable() &&
						brd.squares[i+3][j+2].unopenable() &&
						((brd.squares[i+1][j+1].mine && !brd.squares[i+2][j+1].mine) ||
						(!brd.squares[i+1][j+1].mine && brd.squares[i+2][j+1].mine)))
						{
							++cur_r1;
							cout<<"p2 "<<i<<" "<<j<<endl;
						}
				++dr1[cur_r1];
				r1_sum+=cur_r1;
				for(int i=1;i<=brd.D.height;++i)
				{
					for(int j=1;j<=brd.D.width;++j) 
						cout<<(brd.squares[j][i].mine?"*":QString::number(brd.squares[j][i].num));
					cout<<endl;
				}
				cout<<cur_r1<<endl;
			}
			if(i && !(i & 0xFFFF)) cout<<"iteration "<<i<<"..."<<endl;
		}
		cout<<"Execution finished."<<endl;
		cout<<"time elapsed: "<<t.elapsed()<<"ms"<<endl;
		if(show_avg)
		{
			if(fast_3bv || full_3bv) cout<<"avg 3bv: "<<(double)sum/n<<endl;
			if(zini) cout<<"avg zini: "<<(double)z_sum/n<<endl;
			if(hzini) cout<<"avg hzini: "<<(double)hz_sum/n<<endl;
			if(r1) cout<<"avg 3-1: "<<(double)r1_sum/n<<endl;
			if(r2) cout<<"avg 2x2: "<<(double)r2_sum/n<<endl;
		}
		if(show_distr)
		{
			if(fast_3bv || full_3bv)
			{
				cout<<endl<<"3bv distribution:"<<endl; 
				for(QMap<int,int>::iterator i=d.begin();i!=d.end();++i)
					cout<<i.key()<<' '<<i.value()<<endl;
			}
			if(zini)
			{
				cout<<endl<<"zini distribution:"<<endl; 
				for(QMap<int,int>::iterator i=dz.begin();i!=dz.end();++i)
					cout<<i.key()<<' '<<i.value()<<endl;
			}
			if(hzini)
			{
				cout<<endl<<"hzini distribution:"<<endl; 
				for(QMap<int,int>::iterator i=dhz.begin();i!=dhz.end();++i)
					cout<<i.key()<<' '<<i.value()<<endl;
			}
			if(r1)
			{
				cout<<endl<<"3-1 distribution:"<<endl;
				for(QMap<int,int>::iterator i=dr1.begin();i!=dr1.end();++i)
					cout<<i.key()<<' '<<i.value()<<endl;
			}
			if(r2)
			{
				cout<<endl<<"2x2 box distribution:"<<endl;
				for(QMap<int,int>::iterator i=dr2.begin();i!=dr2.end();++i)
					cout<<i.key()<<' '<<i.value()<<endl;
			}
		}
	}
	return 0;
}

