#include "eval.h"

evaluator::evaluator()
{
}


double evaluator::eval(formula* f)
{
//	const double avg_obv[n_stand_levels+1]={0, 15, 62.5, 170};
	if(!f) throw exceval();
	switch(f->id)	// counters showed in legal mode
	{
		double op1,op2;
	case f_none:
		return f->value;
	case f_int:
		return f->intvalue;
	case f_time:
		return G->time;
	case f_plus:
		return eval(f->op1)+eval(f->op2);
	case f_minus:
		return eval(f->op1)-eval(f->op2);
	case f_times:
		return eval(f->op1)*eval(f->op2);
	case f_div:
		op2=eval(f->op2);
		if(!op2) throw exceval();
		return eval(f->op1)/op2;
	case f_power:
		op1=eval(f->op1);
		op2=eval(f->op2);
		if(op1<0 || (!op1 && !op2)) throw exceval();
		if(!op1) return 0;
		return pow(op1,op2);
	case f_unary_minus:
		return -eval(f->op1);
	case f_sqr:
		op1=eval(f->op1);
		if(op1<0) throw exceval();
		return sqrt(op1);
	case f_exp:
		return exp(eval(f->op1));
	case f_log:
		op1=eval(f->op1);
		if(op1<=0) throw exceval();
		return log(op1);
	case f_level:
		return G->level;
	case f_l_cl:
		return G->lcl;
	case f_l_cl_s:
		if(G->time) return G->lcl/G->time; else throw exceval();
	case f_r_cl:
		return G->rcl;
	case f_r_cl_s:
		if(G->time) return G->rcl/G->time; else throw exceval();
	case f_d_cl:
		return G->dcl;
	case f_d_cl_s:
		if(G->time) return G->dcl/G->time; else throw exceval();
	case f_cl_15:
		return G->cl15;
	case f_cl_15_s:
		if(G->time) return G->cl15/G->time; else throw exceval();
	case f_cl:
		return G->cl;
	case f_cl_s:
		if(G->time) return G->cl/G->time; else throw exceval();
	case f_width:
		return level_width[G->level];
	case f_height:
		return level_height[G->level];
	case f_mines:
		return level_mines[G->level];
	case f_flags:
		return G->flags;
	case f_path:
		return G->distance;
	case f_mode:
		return G->mode;
	case f_unflags:
		return G->unflags;
	case f_wasted_flags:
		return G->wflags;
	case f_3bv:
		return G->bbbv;
	case f_3bv_s:
		if(G->time) return G->bbbvs; else throw exceval();
	case f_obv:
		throw exceval();
	case f_obv_s:
		throw exceval();
	case f_zini:
		return G->zini;
	case f_zini_s:
		if(G->time) return G->zinis; else throw exceval();
	case f_hzini:
		return G->hzini;
	case f_hzini_s:
		if(G->time) return G->hzinis; else throw exceval();
	case f_rqp:
		if(G->time) return (G->time+1)/G->bbbvs; else throw exceval();
	case f_bat:
		throw exceval();
	case f_ioe:
		if(G->cl) return (double)G->bbbv / G->cl; else throw exceval();
	case f_thrp:
		if(G->nwcl) return (double)G->bbbv / G->nwcl; else throw exceval();
	case f_corr:
		if(G->cl) return (double)G->nwcl / G->cl; else throw exceval();
	case f_zne:
		if(G->cl) return (double)G->zini / G->cl; else throw exceval();
	case f_hzne:
		if(G->cl) return (double)G->hzini / G->cl; else throw exceval();
	case f_znt:
		if(G->cl) return (double)G->zini / G->nwcl; else throw exceval();
	case f_hznt:
		if(G->cl) return (double)G->hzini / G->nwcl; else throw exceval();
	case f_ios:
		if(G->time>1 && G->bbbv) return log(G->bbbv)/log(G->time); else throw exceval();
	case f_w_l_cl:
		return G->wlcl;
	case f_w_r_cl:
		return G->wrcl;
	case f_w_d_cl:
		return G->wdcl;
	case f_w_cl_15:
		return G->wcl15;
	case f_w_cl:
		return G->wcl;
	case f_ops:
		return G->ops;
	case f_isls:
		return G->isls;
	case f_wall_mines:
		return G->mines_on_walls;
	case f_corner_mines:
		return G->mines_in_corners;
	case f_misflags:
		return G->misflags;
	case f_misunflags:
		return G->misunflags;
	case f_total_games:
		return total_games[G->level];
	default:
		if(f->id>=f_numbers && f->id<=f_eights) return G->numbers[f->id-f_numbers];
	}
	throw exceval();
}

bool evaluator::eval_expr(formula* ex)
{
	if(!ex) return false;
	try
	{
		switch(ex->id)
		{
		case f_and:
			return eval_expr(ex->op1) && eval_expr(ex->op2);
		case f_or:
			return eval_expr(ex->op1) || eval_expr(ex->op2);
		case f_less:
			return eval(ex->op1) < eval(ex->op2);
		case f_greater:
			return eval(ex->op1) > eval(ex->op2);
		case f_equal:
			return eval(ex->op1) == eval(ex->op2);
		default:
			return false;
		}
	}	
	catch(exceval)
	{
		return false;
	}
}

void evaluator::refresh(gameinfo* g)
{
	G=g;
}

