#include "historywidget.h"

inline void read2(QFile& f,char& c1,char& c2)
{
	f.getChar(&c1);f.getChar(&c2);
}
inline unsigned int read2(QFile& f)
{
	char c1,c2;
	unsigned int res;
	read2(f,c1,c2);
	res=(unsigned int)(unsigned char)c1<<8;
	res+=(unsigned int)(unsigned char)c2;
	return res;
}

void read_density(QFile& f,density_gameinfo* g)
{
	char c1,c2;
	
	// read block & write it to the field
	read2(f,c1,c2);
	g->level=c1;
	g->finished=c2;
	read2(f,c1,c2);
	g->mines=read2(f);
	
	read2(f,c1,c2);
	g->time=(((unsigned int)(unsigned char)c1<<8) + (unsigned int)(unsigned char)c2)*100;
	read2(f,c1,c2);
	g->time+=(((unsigned int)(unsigned char)c1<<8) + (unsigned int)(unsigned char)c2)/10;
	g->time/=100.0;
	
	g->right_flags=read2(f);
		
	g->year=read2(f);
	read2(f,c1,c2);
	g->month=c1; // month
	g->day=c2; // day
	read2(f,c1,c2);
	g->hour=c1; // hour
	g->minute=c2;	// minute
	read2(f,c1,c2);
	g->second=c1; // second

	const char marker=(char)0xFF;
	do
	{
		read2(f,c1,c2);
	}
	while((c1!=marker || c2!=marker) && !f.atEnd());
}

static void set_table_item(QTableWidget& t, int row, int column, double d, int prec)
{
	QTableWidgetItem* i=t.item(row,column);
	if(!i)
	{
		i=new QTableWidgetItem();
		t.setItem(row,column,i);
	}
	i->setFlags(i->flags() & (~Qt::ItemIsSelectable & ~Qt::ItemIsEditable));
	i->setText(QString::number(d,'f',prec));
}
static void set_table_item(QTableWidget& t, int row, int column, const QString& s)
{
	QTableWidgetItem* i=t.item(row,column);
	if(!i)
	{
		i=new QTableWidgetItem(s);
		t.setItem(row,column,i);
	}
	else
		i->setText(s);
	i->setFlags(i->flags() & (~Qt::ItemIsSelectable & ~Qt::ItemIsEditable));
}

const int n_times=5;

struct dens_f
{
	int finished;
	int unfinished;
	double time[n_times];
	QDateTime best;
	dens_f(){for(int i=0;i<n_times;++i) time[i]=0.0;finished=unfinished=0;}
};
	
void historywidget::show_density_finished(QFile& f,int level)
{
	QMap<int,dens_f> map;	// number of mines -> best times
	density_gameinfo g;
	
	cout<<"density finished"<<endl;
	while(!f.atEnd())
	{
		read_density(f,&g);
		
		if(g.level!=level || !g.time) continue;
		dens_f& t=map[g.mines];
		
		if(!g.finished)
			++t.unfinished;
		else
		{
			int i;
			
			++t.finished;
			for(i=0;i<n_times;++i)
				if(g.time<t.time[i] || !t.time[i])
				{
					for(int j=n_times-1;j>i;--j) t.time[j]=t.time[j-1];
					t.time[i]=g.time;
					break;
				}
			if(!i || t.best.isNull())
				t.best=QDateTime(QDate(g.year,g.month,g.day),QTime(g.hour,g.minute,g.second));
		}
	}
			
	dens_table.setRowCount(map.size());
	dens_table.setColumnCount(5+n_times);
	dens_table.setHorizontalHeaderItem(0,new QTableWidgetItem("Mines"));
	dens_table.setHorizontalHeaderItem(1,new QTableWidgetItem("Finished"));
	dens_table.setHorizontalHeaderItem(2,new QTableWidgetItem("Unfinished"));
	dens_table.setHorizontalHeaderItem(3,new QTableWidgetItem("Date"));
	dens_table.setHorizontalHeaderItem(4,new QTableWidgetItem("Hour"));
	dens_table.setHorizontalHeaderItem(5,new QTableWidgetItem("Best"));
	for(int i=1;i<n_times;++i) 
		dens_table.setHorizontalHeaderItem(5+i,new QTableWidgetItem(QString::number(i+1)+" best"));
	int row=0;
	for(QMap<int,dens_f>::iterator i=map.begin();i!=map.end();++i,++row)
	{
		set_table_item(dens_table,row,0,i.key(),0);
		set_table_item(dens_table,row,1,i->finished,0);
		set_table_item(dens_table,row,2,i->unfinished,0);
		if(i->finished)
		{
			set_table_item(dens_table,row,3,date_to_string(i->best.date().day(),
				i->best.date().month(),
				i->best.date().year()));
			set_table_item(dens_table,row,4,hour_to_string(i->best.time().hour(),
				i->best.time().minute(),
				i->best.time().second()));
		}
		else
		{
			set_table_item(dens_table,row,3,"");
			set_table_item(dens_table,row,4,"");
		}
		if(i->time[0]) 
			set_table_item(dens_table,row,5,i->time[0],2);
		else
			set_table_item(dens_table,row,5,"");
		for(int j=1;j<n_times;++j)
			if(i->time[j]) 
				set_table_item(dens_table,row,5+j,i->time[j],2);
			else
				set_table_item(dens_table,row,5+j,"");
	}
	dens_table.resizeRowsToContents();
	dens_table.resizeColumnsToContents();
}
	
struct dens_uf
{
	bool has_finished;
	int unfinished;
	double flags[n_times];
	QDateTime best;
	dens_uf(){for(int i=0;i<n_times;++i) flags[i]=0;unfinished=0;has_finished=false;}
};

void historywidget::show_density_unfinished(QFile& f,int level)
{
	QMap<int,dens_uf> map; // number of mines -> best numbers of flags
	density_gameinfo g;
	
	cout<<"density unfinished"<<endl;
	while(!f.atEnd())
	{
		read_density(f,&g);
		
		if(g.level!=level || !g.time) continue;
		
		dens_uf& t=map[g.mines];
		
		if(t.has_finished)
			continue;
		if(g.finished)
		{
			t.has_finished=true;
			continue;
		}
		else
		{
			int i;

			++t.unfinished;
			for(i=0;i<n_times;++i)
				if(g.right_flags>t.flags[i] || !t.flags[i])
				{
					for(int j=n_times-1;j>i;--j) t.flags[j]=t.flags[j-1];
					t.flags[i]=g.right_flags;
					break;
				}
			if(!i || t.best.isNull()) 
				t.best=QDateTime(QDate(g.year,g.month,g.day),QTime(g.hour,g.minute,g.second));
		}
	}
	
	dens_table.setRowCount(map.size());
	dens_table.setColumnCount(4+n_times);
	dens_table.setHorizontalHeaderItem(0,new QTableWidgetItem("Mines"));
	dens_table.setHorizontalHeaderItem(1,new QTableWidgetItem("Unfinished"));
	dens_table.setHorizontalHeaderItem(2,new QTableWidgetItem("Date"));
	dens_table.setHorizontalHeaderItem(3,new QTableWidgetItem("Hour"));
	dens_table.setHorizontalHeaderItem(4,new QTableWidgetItem("Best"));
	for(int i=1;i<n_times;++i) 
		dens_table.setHorizontalHeaderItem(4+i,new QTableWidgetItem(QString::number(i+1)+" best"));
	int row=0;
	for(QMap<int,dens_uf>::iterator i=map.begin();i!=map.end();++i)
	{
		if(i->has_finished)
		{
			continue;
		}
		set_table_item(dens_table,row,0,i.key(),0);
		set_table_item(dens_table,row,1,i->unfinished,0);
		set_table_item(dens_table,row,2,date_to_string(i->best.date().day(),
				i->best.date().month(),
				i->best.date().year()));
		set_table_item(dens_table,row,3,hour_to_string(i->best.time().hour(),
				i->best.time().minute(),
				i->best.time().second()));
		set_table_item(dens_table,row,4,i->flags[0],0);
		for(int j=1;j<n_times;++j)
			if(i->flags[j])
				set_table_item(dens_table,row,4+j,i->flags[j],0);
			else
				set_table_item(dens_table,row,4+j,"");
		++row;
	}
	dens_table.setRowCount(row);
	dens_table.resizeRowsToContents();
	dens_table.resizeColumnsToContents();
}

void historywidget::show_density()
{
	int level=4;
	cout<<"show_density"<<endl;
	if(dens_beg.isChecked())
		level=1;
	else if(dens_int.isChecked())
		level=2;
	else if(dens_exp.isChecked())
		level=3;
	else if(dens_mega.isChecked())
		level=4;

	const QString history=settings_dir+"freesweeper.history-density";
	QFile f(history);
	if(!f.open(QIODevice::ReadOnly))
	{
		msg("Couldn't open history file");
		return;
	}
	
	if(dens_finished.isChecked())
		show_density_finished(f,level);
	else
		show_density_unfinished(f,level);
}

