#include "historywidget.h"

inline void read2(QFile& f,char& c1,char& c2)
{
	f.getChar(&c1);f.getChar(&c2);
}
inline unsigned int read2(QFile& f)
{
	char c1,c2;
	unsigned int res;
	read2(f,c1,c2);
	res=(unsigned int)(unsigned char)c1<<8;
	res+=(unsigned int)(unsigned char)c2;
	return res;
}

void historywidget::read_game(QFile& f)
{
	int level;
	char c1,c2;
	gameinfo g;
	
	// read block & write it to the field
	read2(f,c1,c2);
	g.mode=(unsigned int)c1;
	g.level=level=c2;
	
	read2(f,c1,c2);
	g.time=(((unsigned int)(unsigned char)c1<<8) + (unsigned int)(unsigned char)c2)*1000;
	read2(f,c1,c2);
	g.time+=(((unsigned int)(unsigned char)c1<<8) + (unsigned int)(unsigned char)c2);
	g.time/=1000.0;
	if(g.time==0) return;
	
	g.bbbv=read2(f);
	
	read2(f,c1,c2);
	g.ops=(unsigned int)(unsigned char)c1;
	g.isls=(unsigned int)(unsigned char)c2;
	
	g.zini=read2(f);
	
	g.hzini=read2(f);
	
	g.lcl=read2(f);
	g.rcl=read2(f);
	g.dcl=read2(f);
	g.cl=g.lcl+g.rcl+g.dcl;
	g.cl15=read2(f);
	g.wlcl=read2(f);
	g.wrcl=read2(f);
	g.wdcl=read2(f);
	g.wcl=g.wlcl+g.wrcl+g.wdcl;
	g.nwcl=g.cl-g.wcl;
	g.wcl15=read2(f);
	g.flags=read2(f);
	g.unflags=read2(f);
	g.misflags=read2(f);
	g.misunflags=read2(f);
	g.wflags=read2(f);
	g.distance=read2(f);
	for(int i=0;i<9;++i)
	{
		g.numbers[i]=read2(f);
	}
	
	read2(f,c1,c2);
	g.mines_in_corners=(unsigned int)(unsigned char)c1;
	g.mines_on_walls=(unsigned int)(unsigned char)c2;

	// OBV
	const double obv_num[n_stand_levels+1]={0, 0.07, 0.20, 0.38};
	const double obv_3bv[n_stand_levels+1]={0, 0.43, 0.32, 0.23};
	const double obv_ops[n_stand_levels+1]={0, 2.27, 1.38, 0.99};
	const double obv_isl[n_stand_levels+1]={0, 0.00, 0.00, 0.00};
	if(level>n_stand_levels) 
		g.obv=-1;
	else
		g.obv=obv_num[level]*g.numbers[0] +	
				obv_3bv[level]*g.bbbv + 
				obv_ops[level]*g.ops + 
				obv_isl[level]*g.isls;
		
	g.year=read2(f);
	read2(f,c1,c2);
	g.month=c1; // month
	g.day=c2; // day
	read2(f,c1,c2);
	g.hour=c1; // hour
	g.minute=c2;	// minute
	read2(f,c1,c2);
	g.second=c1; // second
	g.day_of_week=QDate(g.year,g.month,g.day).dayOfWeek();
	
	g.bbbvs=g.bbbv/g.time;
	g.zinis=g.zini/g.time;
	g.hzinis=g.hzini/g.time;
	g.flagging=g.rcl||g.dcl;
	
	h[level-1] << g;
	++Eval.total_games[level];
}

void historywidget::read_history()
{
	const QString history=settings_dir+"freesweeper.history";
	QFile f(history);
	
	for(int i=0;i<n_stand_levels;++i) h[i].clear();
	if(!f.open(QIODevice::ReadOnly))
	{
		msg("Couldn't open history file");
		return;
	}
	while(!f.atEnd())
	{
		const char marker=(char)0xFF;
		char c1,c2;
		read_game(f);
		do
		{
			read2(f,c1,c2);
		}
		while((c1!=marker || c2!=marker) && !f.atEnd());
	}
}

