#include "parser.h"

const QString constants_id[n_constants]=
{"beginner","beg","intermediate","int","expert","exp","megaexpert","meg","custom","cus",
	"classic","lucky","density","upk","cheat","edit"};
const int constants[n_constants]={l_beginner,l_beginner,l_intermediate,l_intermediate,l_expert,l_expert,
l_megaexpert,l_megaexpert,l_custom,l_custom,classic,lucky,density,upk,cheat,edit};

parser::parser(translation* T):T(T)
{
	for(int id=0;id<n_named_counters;++id)
		name_to_id[counter_name[id].toLower()]=(formula_id)id;
}

inline QCharRef parser::sym(QString& s, int cur)
{
	if(s.length()>cur) return s[cur]; else throw excparser(cur,T->str[ins_unexpected_end]);
}


void parser::ignore_spaces(QString& s,int& cur)
{
	while(s.length()>cur && s[cur]==' ') ++cur;
}

QString parser::get_id(QString& s,int& beg)
{
	if(s.length()<=beg || (!s[beg].isLetter() && s[beg]!='\\'))
		throw excparser(beg,T->str[ins_id_or_number]);
	int c_beg=beg;
	do
	{
		beg++;
	}
	while(s.length()>beg && (s[beg].isLetter() || s[beg]=='\\'));
	return s.mid(c_beg,beg-c_beg);
}
double parser::get_number(QString& s,int& beg)
{
	double res=0.0;
	bool min=s[beg]=='-';
	if(min) ++beg;
	while(s.length()>beg && s[beg].isDigit())
	{
		res=res*10+s[beg++].digitValue();
	}
	if(s.length()<=beg) return res;
	if(s[beg]=='.')
	{
		double mult=1;
		while(s.length()>beg && s[++beg].isDigit())
		{
			mult/=10;
			res+=mult*(s[beg].digitValue());
		}
	}
	return min ? -res : res;
}
int parser::get_int(QString& s,int& beg)
{
	int res=0;
	bool min=s[beg]=='-';
	if(min) ++beg;
	while(s.length()>beg && s[beg].isDigit())
	{
		res=res*10+s[beg++].digitValue();
	}
	return min ? -res : res;
}

int parser::get_function(QString& s,int& cur,formula*& f,formula_id x)
{
	if(s[cur]=='(')
	{
		cur=get_formula1(s,++cur,f);
		formula* ff=f;
		f=new formula(ff);
		f->id=x;
		if(s[cur]!=')') throw excparser(cur,T->str[ins_bracket_db]);
	}
	else throw excparser(cur,T->str[ins_bracket]);
	return cur;
}

formula* parser::parse_expression(QString s)
{
	int end;
	formula* f=0;
	try
	{
		s=s.toLower();
		s.replace(" and ","&");
		s.replace(" or ","|");
		end=get_expression1(s,0,f);
		if(end<s.length()) throw excparser(end,T->str[ins_garbage]);
	}
	catch(excparser e)
	{
		err(s+"\n\n"+T->str[ins_expr_err]+"("+e.msg+"); "+T->str[ins_was_noticed]+
			QString::number(e.sym));
		if(f) delete f;
		return 0;
	}
	return f;
}
formula* parser::parse_formula(QString s)
{
	int end;
	formula* f=0;
	try
	{
		s=s.toLower();
		end=get_formula1(s,0,f);
		if(end<s.length()) throw excparser(end,T->str[ins_garbage]);
	}
	catch(excparser e)
	{
		err(s+"\n\n"+T->str[ins_form_err]+"("+e.msg+"); "+T->str[ins_was_noticed]+
			QString::number(e.sym));
		if(f) delete f;
		return 0;
	}
	return f;
}

int parser::get_expression1(QString& s,int beg,formula*& f)
{
	int cur=get_expression2(s,beg,f);
	if(s.length()>cur+1)
	{
		if(s[cur]=='|' && s[cur+1]!='|')
		{
			formula* ff=f;
			f=new formula;
			f->id=f_or;
			f->op1=ff;
			cur=get_expression1(s,++cur,f->op2);
			if(!f->op2) throw excparser(cur,T->str[ins_2_arg_or]);
		}
	}
	ignore_spaces(s,cur);
	return cur;
}
int parser::get_expression2(QString& s,int beg,formula*& f)
{
	int cur=get_expression3(s,beg,f);
	if(s.length()>cur)
	{
		if(s[cur]=='&')
		{
			formula* ff=f;
			f=new formula;
			f->id=f_and;
			f->op1=ff;
			cur=get_expression2(s,++cur,f->op2);
			if(!f->op2) throw excparser(cur,T->str[ins_2_arg_and]);
		}
	}
	ignore_spaces(s,cur);
	return cur;
}
int parser::get_expression3(QString& s,int beg,formula*& f)
{
	int cur=beg;
	ignore_spaces(s,cur);
	if(s.length()>cur)
	{
		if(s[cur]=='[')
		{
			cur=get_expression1(s,++cur,f);
			if(sym(s,cur++)!=']') throw excparser(cur,T->str[ins_sqbracket_db]);
		}
		else
		{
			f=new formula;
			cur=get_formula1(s,cur,f->op1);
			if(sym(s,cur)=='<')
				f->id=f_less;
			else if(sym(s,cur)=='>')
				f->id=f_greater;
			else if(sym(s,cur)=='=')
				f->id=f_equal;
			else
				throw excparser(cur,T->str[ins_comparison]);
			cur=get_formula1(s,++cur,f->op2);
		}
	}
	ignore_spaces(s,cur);
	return cur;
}

int parser::get_formula1(QString& s,int beg,formula*& f)
{
	int cur=get_formula2(s,beg,f);
	while(s.length()>cur)
	{
		if(sym(s,cur)=='+' || sym(s,cur)=='-')
		{
			formula* ff=f;
			QCharRef sign=s[cur];
			f=new formula;
			f->id=(s[cur]=='+')?f_plus:f_minus;
			f->op1=ff;
			cur=get_formula2(s,++cur,f->op2);
			if(!f->op2) throw excparser(cur,f->id==f_plus?T->str[ins_2_arg_plus]:T->str[ins_2_arg_minus]);
		}
		else break;
	}
	ignore_spaces(s,cur);
	return cur;
}
int parser::get_formula2(QString& s,int beg,formula*& f)
{
	int cur=get_formula3(s,beg,f);
	while(s.length()>cur)
	{
		if(s[cur]=='*' || s[cur]=='/')
		{
			formula* ff=f;
			QCharRef sign=s[cur];
			f=new formula;
			f->id=(s[cur]=='*')?f_times:f_div;
			f->op1=ff;
			cur=get_formula3(s,++cur,f->op2);
			if(!f->op2) throw excparser(cur,f->id==f_times?T->str[ins_2_arg_times]:T->str[ins_2_arg_div]);
		}
		else break;
	}
	ignore_spaces(s,cur);
	return cur;
}
int parser::get_formula3(QString& s,int beg,formula*& f)
{
	int cur=get_formula4(s,beg,f);
	if(s.length()>cur)
	{
		if(s[cur]=='^')
		{
			formula* ff=f;
			f=new formula;
			f->id=f_power;
			f->op1=ff;
			cur=get_formula3(s,++cur,f->op2);
			if(!f->op2) throw excparser(cur,T->str[ins_2_arg_pow]);
		}
	}
	ignore_spaces(s,cur);
	return cur;
}
int parser::get_formula4(QString& s,int beg,formula*& f)
{
	int cur=beg;
	ignore_spaces(s,cur);
	if(sym(s,cur)=='-')
	{
		cur=get_formula4(s,++cur,f);
		formula* ff=f;
		f=new formula(ff);
		f->id=f_unary_minus;
	}
	else if(sym(s,cur)=='(')
	{
		cur=get_formula1(s,++cur,f);
		if(sym(s,cur++)!=')') throw excparser(cur,T->str[ins_bracket_db]);
	}
	else if(sym(s,cur).isDigit())
	{
		f=new formula;
		f->id=f_none;
		f->value=get_number(s,cur);
	}
	// functions
	else
	{
		QString id=get_id(s,cur);
		f=new formula;
		if(id=="sqr")
		{
			get_function(s,cur,f,f_sqr);
		}
		else if(id=="exp")
		{
			get_function(s,cur,f,f_exp);
		}
		else if(id=="log")
		{
			get_function(s,cur,f,f_log);
		}
		else if(!name_to_id.contains(id))
		{
			// constants
			for(int i=0;i<n_constants;++i)
				if(id==constants_id[i])
				{
					f->id=f_int;
					f->intvalue=constants[i];
					ignore_spaces(s,cur);
					return cur;
				}
			throw excparser(cur,T->str[ins_unknown_id]+" "+id);
		}
		else
			f->id=name_to_id.value(id);
	}
	ignore_spaces(s,cur);
	return cur;
}


